﻿# -*- coding: utf-8 -*-

import bpy
from mathutils import Vector

from .. import FnCalc

#### =======================================================================
__all__ = ("MakeBone",)

#### =======================================================================
def __BuildMx(pt0, pt1, upvec):
	fwd = Vector( (pt1[0]-pt0[0], pt1[1]-pt0[1], pt1[2]-pt0[2]) ).normalized()
	up  = Vector(upvec).normalized()
	rgt = fwd.cross(up).normalized()
	up  = rgt.cross(fwd).normalized()
	return( ((rgt.x, rgt.y, rgt.z, 0.0), (fwd.x, fwd.y, fwd.z, 0.0), (up.x, up.y, up.z, 0.0), (pt0[0],pt0[1],pt0[2], 1.0)) )

#### =======================================================================
def MakeBone(context, lyr, opts, obLoc):
	u'''ボーン作成(LWO)'''
	FnObjectLink = context.view_layer.active_layer_collection.collection.objects.link
	ab = bpy.data.armatures.new("{}_rig".format(lyr.name))
	ob = bpy.data.objects.new("{}_rig".format(lyr.name), ab)
	FnObjectLink(ob)							# link->active->modeを遵守
	bpy.context.view_layer.objects.active = ob	# EDITに入る為選択
	bpy.ops.object.mode_set(mode="EDIT")		#

	# armatureから配下のedit_boneにも継承される. bone毎の設定も可能.
	ab.layers[opts.boneLyr] = True		# 一つはTrueでないといけないので目的のidxを先に立てる
	ab.layers[0] = (opts.boneLyr == 0)	# デフォルト分の[0]==Trueを処置

	childParent = [None] * len(lyr.bones)

	editBones = []
	APD = editBones.append

	buildedBonesDct = {}

	pnts = lyr.pnts
	for (i, bone) in sorted(list(lyr.bones.items()), key=lambda x: x[0]):
		bn = ab.edit_bones.new(bone.name)
		APD(bn)

		# editBones(list)と同義だけど現状のbonesと同じ形態(dict)で保持する事にする
		# bonesをlistにしたら再考
		buildedBonesDct.update({i: bn})

		pt0 = pnts[bone.pnt0]	# [0]根本,[1]先端
		pt1 = pnts[bone.pnt1]	# [0]根本,[1]先端
		bn.head = pt0
		bn.tail = pt1

		if(bone.bnup != None):
			if(bone.skbm != None):			# SkelegonEditor製Skelegon
				bankMode = bone.skbm
				if(bone.skbm == 0):
					# DrawSkelegonによってBNUP(vec)が<0,0,0>の場合が有り得る
					if(bone.bnup != (0.0, 0.0, 0.0)):
						bn.matrix = __BuildMx(pt0, pt1, bone.bnup)

				elif((bone.skbm == 8) and (bone.skbr != None)):
					v = Vector( (pt1[0]-pt0[0], pt1[1]-pt0[1], pt1[2]-pt0[2]) ).normalized()
					bn.matrix = bpy.types.Bone.MatrixFromAxisRoll((v[0], v[1], v[2]), bone.skbr)

			else:							# 素のSkelegon
				# DrawSkelegonによってBNUP(vec)が<0,0,0>の場合が有り得る
				if(bone.bnup != (0.0, 0.0, 0.0)):
					bn.matrix = __BuildMx(pt0, pt1, bone.bnup)

		if(bone.childs):			# 子持ちなら[子のidx]に親として自身をプロット
			for ch in bone.childs:
				childParent[ch] = bn

	ab.transform(opts.mxAxisConv)

	# 親がプロットされているなら親子組み
	for (chIdx, bn) in enumerate(childParent):
		if(bn != None):
			ch = editBones[chIdx]
			ch.parent = bn
			ch.use_connect = True	# 頂点共有による親子関係なので接続

	# skelegonEditorの親指定
	if(opts.useFltPar):
		for (i, bone) in sorted(list(lyr.bones.items()), key=lambda x: x[0]):
			if(bone.skpr):
				for (k, b) in lyr.bones.items():
					if(b.skid == bone.skpr):
						editBones[i].parent = editBones[k]
						# skprは浮動親子のみなのでuse_connectはしない


	bpy.ops.object.mode_set(mode="OBJECT")

	if(opts.usePivot):
		ob.location = obLoc

	ob.hide_viewport = lyr.isHide	# 不可視だとEDITに入れなくなるので最後に.

	return(ob, buildedBonesDct)
