﻿# -*- coding: utf-8 -*-

import sys, math

import bpy
from mathutils import Matrix

from .. import Chunk_Object
from . import Meshes, Bones, Curves, Opts, Material

if(sys.modules["Load_LWO"].DO_RELOAD):
	import importlib
	loc = set(locals().keys())
	for f in ("Meshes", "Bones", "Curves", "Opts", "Material", "Chunk_Object",):
		if(f in loc):
			importlib.reload(eval(f))
	del f, loc

#### =======================================================================
__all__ = ("BuildObjects",)

#### =======================================================================
def BuildObjects(context, lwo, opts= Opts.OptsObject()):
	u'''ブレンダーデータ作成, LWO'''
	if(not (opts.buildMesh | opts.buildCrv | opts.buildBone)):
		return

	deteminedMtlDct = Material.Make(lwo.surfs, lwo.clips, opts)	# 先行して作成

	buildedVertGrpsLsts = []
	buildedBonesDcts = {}

	lyrCnt = 0
	for lyr in lwo.layers:
		if(not (lyr.isHide & opts.ignoreHide)):
			# 先にpivとpntsを改変してしまう
			obLocation = Meshes.PreCalcVertsBasePos(lyr, opts.scale, opts.usePivot)
			meOB = None

			if(opts.buildMesh and lyr.polNum):
				(meOB, buildedVertGrps) = Meshes.MakeMesh(context, lyr, lwo.tags, lwo.surfs, deteminedMtlDct, opts, obLocation)
				buildedVertGrpsLsts.append(buildedVertGrps)

			if(opts.buildCrv and lyr.crvNum):
				Curves.MakeCurve(context, lyr, opts, obLocation)

			if(opts.buildBone and lyr.bonNum):
				(abOB, buildedBonesDct) = Bones.MakeBone(context, lyr, opts, obLocation)
				buildedBonesDcts.update({lyrCnt: buildedBonesDct})
				if(opts.assignAM and meOB):
					bpy.context.view_layer.objects.active = meOB
					bpy.ops.object.modifier_add(type="ARMATURE")
					meOB.modifiers["Armature"].object = abOB
		lyrCnt += 1

	if(opts.buildBone and opts.renWght and opts.useWght):
		lyrCnt = 0
		for lyr in lwo.layers:
			if((not (lyr.isHide & opts.ignoreHide)) and lyr.bonNum):
				for (k, bone) in lyr.bones.items():
					if(bone.weight):
						for (vgNames, vgs) in buildedVertGrpsLsts:
							try:
								idx = vgNames.index(bone.weight)
								vgs[idx].name = buildedBonesDcts[lyrCnt][k].name
								vgNames[idx] = None
							except:
								pass
			lyrCnt += 1
