﻿# -*- coding: utf-8 -*-

import bpy

from ..LWO_ParseHeader import ClipIdT

#### =======================================================================
__all__ = ("Make",)

#### =======================================================================
def Make(surfs, clips, opts):
	u'''マテリアル作成'''
	loadedImgDct = {}
	mtlDct = {}
	if(opts.useExistMtl):
		for (kName, surf) in surfs.items():
			mt = bpy.data.materials.get(surf.name)
			if(not mt):		# 既設でなければ作成
				mt = __Mater(surf, clips, opts.useImgs, loadedImgDct)
			mtlDct[kName] = mt
	else:
		for (kName, surf) in surfs.items():
			mtlDct[kName] = __Mater(surf, clips, opts.useImgs, loadedImgDct)

	return(mtlDct)

#### --------------------------------
def __Mater(surf, clips, useImgs, loadedImgDct):
	mt = bpy.data.materials.new(surf.name)
	mt.diffuse_color = surf.colr + (1.0,)
	mt.specular_intensity = surf.spec
	mt.roughness = 0.0
	mt.use_nodes = True

	if(useImgs):
		for mp in surf.imap:
			if(mp.imgIdx):	# idx==0は未使用のフラグ
				if(mp.imgIdx not in loadedImgDct):
					im = __LoadImage(clips[mp.imgIdx])
					loadedImgDct[mp.imgIdx] = im

				else:
					im = loadedImgDct[mp.imgIdx]

				if(im != None):
					nd = mt.node_tree.nodes.new("ShaderNodeTexImage")
					nd.image = im
	return(mt)

#### =======================================================================
def __LoadImage(clip):
	if(clip.type == ClipIdT.STIL):
		return(__Image(clip.stillName, "FILE"))
	elif(clip.type == ClipIdT.ISEQ):
		return(__Image(clip.seqName, "SEQUENCE"))
	elif(clip.type == ClipIdT.ANIM):
		return(__Image(clip.animName, "MOVIE"))
	else:#	if(clip.type == ClipIdT.XREF):
		pass

	return(None)

#### --------------------------------
def __Image(name, srcType):
	# .loadではファイル無し/不正パスでraiseするので空img作成後にパスを入れる(確認されない).
	nm = name.rsplit("/",1)[-1].rsplit("\\",1)[-1].rsplit(":",1)[-1].rsplit(".",1)[0]
	im = bpy.data.images.new(name=nm, width=32, height=32)
	im.source = srcType
	im.alpha_mode = "STRAIGHT"	# PREMUL
	im.colorspace_settings.name = "Raw"

	fpath = name[:]
	if((fpath[1] == ":") and ((fpath[2] != "\\") or (fpath[2] != "/"))):
		fpath = "{}/{}".format(fpath[0:2], fpath[2:])	# c:hoge.extが有り得る
	im.filepath = fpath

	return(im)
