﻿# -*- coding: utf-8 -*-

from mathutils import Matrix

#### =======================================================================
__all__ = ("OptsObject",
	"pGScl", "pGPivt", "pGHide", "pGMatl", "pGImgs",
	"pMeshs","pMEdge", "pMMorf", "pMWght", "pMSels", "pMColr", "pMPart", "pMPick", "pMNorm", "pMVCol",
	"pCrvs", "pCMode", "pCDegr", "pBones", "pBLayr", "pBWght", "pBAMod", "pBSPar",
)

#### ========================================================================
class bProp(object):
	__slots__=()
	@classmethod
	def kwds(cls):
		return({"name":cls.NAME, "description":cls.DESC, "default":cls.DEF})

class eProp(object):
	__slots__=()
	@classmethod
	def kwds(cls):
		return({"name":cls.NAME, "description":cls.DESC, "default":cls.DEF, "items":cls.ITEM})

class iProp(object):
	__slots__=()
	@classmethod
	def kwds(cls):
		return({"name":cls.NAME, "description":cls.DESC, "default":cls.DEF, "max":cls.MAX, "min":cls.MIN})

class fProp(object):
	__slots__=()
	@classmethod
	def kwds(cls):
		return({"name":cls.NAME, "description":cls.DESC, "default":cls.DEF, "min":cls.MIN, "precision":cls.PRE, "step":cls.STEP})

#### --------------------------------
class pGScl(fProp):
	__slots__= fProp.__slots__
	KEY = "scaling"
	NAME= "倍率"
	DESC= "形状の拡大縮小"
	DEF = 1.0
	MIN = 0.001
	PRE = 3
	STEP= 100

class pGPivt(bProp):
	__slots__= bProp.__slots__
	KEY = "usePivot"
	NAME= "Pivotを適用"
	DESC= "Pivotを頂点とトランスフォームに適用する"
	ICON= ("PIVOT_BOUNDBOX","PIVOT_CURSOR")
	DEF = False

class pGHide(bProp):
	__slots__= bProp.__slots__
	KEY = "ignoreHide"
	NAME= "不可視を無視"
	DESC= "不可視レイヤを無視"
	ICON= ("RESTRICT_VIEW_OFF","RESTRICT_VIEW_ON")
	DEF = True

class pGMatl(bProp):
	__slots__= bProp.__slots__
	KEY = "useExistMtl"
	NAME= "既存Mtlを使用"
	DESC= "同名のマテリアルが既存の場合、新規作成せずに流用する"
	ICON= "MATERIAL_DATA"
	DEF = False

class pGImgs(bProp):
	__slots__= bProp.__slots__
	KEY = "useImgs"
	NAME= "Imageを読込"
	DESC= "画像を読み込む"
	ICON= ("FILE_HIDDEN", "FILE_IMAGE")
	DEF = True

#### --------------------------------
class pMeshs(bProp):
	__slots__= bProp.__slots__
	KEY = "buildMesh"
	NAME= "メッシュを作成"
	DESC= NAME
	ICON= "MESH_DATA"
	DEF = True

class pMEdge(bProp):
	__slots__= bProp.__slots__
	KEY = "useEdge"
	NAME= "エッジウェイト"
	DESC= "エッジウェイト(WGHT:EdgeWeight/VMED:Mode)を辺クリースに適用"
	ICON= "EDGESEL"
	DEF = True

class pMMorf(bProp):
	__slots__= bProp.__slots__
	KEY = "useMorf"
	NAME= "モーフマップ"
	DESC= "モーフマップ(MORF/SPOT)をシェイプキーに作成"
	ICON= "SHAPEKEY_DATA"
	DEF = True

class pMWght(bProp):
	__slots__= bProp.__slots__
	KEY = "useWght"
	NAME= "ウェイトマップ"
	DESC= "ウェイトマップ(WGHT)を頂点グループに作成\n値は0～1で切取り"
	ICON= "GROUP_VERTEX"
	DEF = True

class pMSels(bProp):
	__slots__= bProp.__slots__
	KEY = "useSels"
	NAME= "選択セット"
	DESC= "選択セット(PICK)を頂点グループに作成"
	ICON= "VERTEXSEL"
	DEF = True

class pMColr(bProp):
	__slots__= bProp.__slots__
	KEY = "useColr"
	NAME= "カラータグ"
	DESC= "ポリゴンタグ: カラーをフェイスマップに作成"
	ICON= "FACE_MAPS"
	DEF = True

class pMPart(bProp):
	__slots__= bProp.__slots__
	KEY = "usePart"
	NAME= "パートタグ"
	DESC= "ポリゴンタグ: パートをフェイスマップに作成"
	ICON= "FACE_MAPS"
	DEF = True

class pMPick(bProp):
	__slots__= bProp.__slots__
	KEY = "usePick"
	NAME= "選択タグ"
	DESC= "ポリゴンタグ: 選択をフェイスマップに作成"
	ICON= "FACE_MAPS"
	DEF = True

class pMNorm(bProp):
	__slots__= bProp.__slots__
	KEY = "useNorm"
	NAME= "面頂点法線"
	DESC= "面毎頂点の法線(NORM)をカスタム分割法線データに作成"
	ICON= "NORMALS_VERTEX_FACE"
	DEF = False

class pMVCol(bProp):
	__slots__= bProp.__slots__
	KEY = "useVCol"
	NAME= "頂点カラー"
	DESC= "頂点カラー(RGB/RGBA)を頂点カラーに作成"
	ICON= "GROUP_VCOL"
	DEF = True

#### --------------------------------
class pCrvs(bProp):
	__slots__= bProp.__slots__
	KEY = "buildCrv"
	NAME= "カーブを作成"
	DESC= NAME
	ICON= "CURVE_DATA"
	DEF = True

class pCMode(eProp):
	__slots__= eProp.__slots__
	KEY = "crvPerLyr"
	NAME= "カーブの作成方法"
	DESC= NAME
	DEF = "perLyr"
	LYR = (DEF, "レイヤ単位", "レイヤ内のカーブをオブジェクトに纏めて作成", "MOD_CURVE", 0)
	CRV = ("perCrv", "カーブ毎", "レイヤ内のカーブ毎にオブジェクトを作成", "MOD_INSTANCE", 1)
	ITEM= (LYR, CRV)

class pCDegr(iProp):
	__slots__= iProp.__slots__
	KEY = "degree"
	NAME= "次数"
	DESC= "カーブの次数?(1～5)"
	DEF = 3
	MIN = 1
	MAX = 5

#### --------------------------------
class pBones(bProp):
	__slots__= bProp.__slots__
	KEY = "buildBone"
	NAME= "ボーンを作成"
	DESC= "スケルゴンからアーマチュアとボーンを作成"
	ICON= "ARMATURE_DATA"
	DEF = True

class pBLayr(iProp):
	__slots__= iProp.__slots__
	KEY = "boneLyr"
	NAME= "レイヤ"
	DESC= "使用するボーンのレイヤー[0～31]"
	ICON= "BONE_DATA"
	DEF = 0
	MIN = 0
	MAX = 31

class pBWght(bProp):
	__slots__= bProp.__slots__
	KEY = "renWght"
	NAME= "ウェイト改名"
	DESC= "頂点グループ名を参照元のスケルゴン名で改名してアーマチュアに紐付けさせる"
	ICON= "GROUP_VERTEX"
	DEF = False

class pBAMod(bProp):
	__slots__= bProp.__slots__
	KEY = "assignAM"
	NAME= "モディファイアを適用"
	DESC= "同一レイヤ内にメッシュとスケルゴンがある場合に限り、\nメッシュにアーマチュアモディファイアを適用する"
	ICON= "MOD_ARMATURE"
	DEF = True

class pBSPar(bProp):
	__slots__= bProp.__slots__
	KEY = "useFltPar"
	NAME= "浮動型親子"
	DESC= "スケルゴンエディタによる浮動ボーンの親子関係指定があれば適用する\nOFFでは頂点共有型の親子関係のみを扱う"
	ICON= "OUTLINER"
	DEF = True

#### ========================================================================
class OptsObject(object):
	__slots__ = (
		pMeshs.KEY, pCrvs.KEY,  pBones.KEY,
		"scale",    pGScl.KEY,  pGPivt.KEY, pGHide.KEY, pGMatl.KEY, pGImgs.KEY,
		pMEdge.KEY, pMMorf.KEY, pMWght.KEY, pMSels.KEY, pMColr.KEY, pMPart.KEY, pMPick.KEY, pMNorm.KEY, pMVCol.KEY,
		pCMode.KEY, pCDegr.KEY, pBLayr.KEY, pBWght.KEY, pBAMod.KEY, pBSPar.KEY,
		"mxAxisConv",
	)
	#### -----------------------------------------------------------------------
	def __init__(self, **kwd):
		self.buildMesh = kwd.get(pMeshs.KEY, pMeshs.DEF)	# Mesh作成
		self.buildCrv  = kwd.get(pCrvs.KEY,  pCrvs.DEF)		# Curve作成
		self.buildBone = kwd.get(pBones.KEY, pBones.DEF)	# Bone作成

		scl = kwd.get(pGScl.KEY, pGScl.DEF)
		self.scaling   = scl			# 倍率(UI用)
		self.scale     = (scl,scl,scl)	# build用
		self.usePivot  = kwd.get(pGPivt.KEY, pGPivt.DEF)	# ピボット適用
		self.ignoreHide= kwd.get(pGHide.KEY, pGHide.DEF)	# 不可視属性Layerを無視
		self.useExistMtl=kwd.get(pGMatl.KEY, pGMatl.DEF)	# 既存の同名Mtlを共用
		self.useImgs    =kwd.get(pGImgs.KEY, pGImgs.DEF)	# 画像があればMtlの新規作成時に読込＆作成

		self.useEdge = kwd.get(pMEdge.KEY, pMEdge.DEF)	# エッジウェイト適用
		self.useMorf = kwd.get(pMMorf.KEY, pMMorf.DEF)	# モーフ作成
		self.useWght = kwd.get(pMWght.KEY, pMWght.DEF)	# ウェイトマップを頂点グループへ
		self.useSels = kwd.get(pMSels.KEY, pMSels.DEF)	# 頂点選択セット(PICK)
		self.useColr = kwd.get(pMColr.KEY, pMColr.DEF)	# 面カラー(COLR)
		self.usePart = kwd.get(pMPart.KEY, pMPart.DEF)	# 面パート(PART)
		self.usePick = kwd.get(pMPick.KEY, pMPick.DEF)	# 面選択(PICK)
		self.useNorm = kwd.get(pMNorm.KEY, pMNorm.DEF)	# pntPerPol法線(NORM)
		self.useVCol = kwd.get(pMVCol.KEY, pMVCol.DEF)	# 頂点カラーの適用

		self.crvPerLyr = kwd.get(pCMode.KEY, True)			# 複数のカーブを1オブジェクトで作成
		self.degree    = kwd.get(pCDegr.KEY, pCDegr.DEF)	# カーブ次数

		self.boneLyr  = kwd.get(pBLayr.KEY, pBLayr.DEF)	# ボーンレイヤ
		self.renWght  = kwd.get(pBWght.KEY, pBWght.DEF)	# ボーンと頂点グループの紐付けの為に強制改名
		self.assignAM = kwd.get(pBAMod.KEY, pBAMod.DEF)	# 同一レイヤにメッシュとスケルゴンがある場合にモディファイヤ適用
		self.useFltPar= kwd.get(pBSPar.KEY, pBSPar.DEF)	# SkelegonEditorの浮動親子を適用

		self.mxAxisConv = Matrix(((1, 0, 0, 0),(0, 0, 1, 0),(0, 1, 0, 0),(0, 0, 0, 1)))

