﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import print_function

from .LWO_ParseHeader import PolyNameT as lwoPolyNameT
from .LXO_ParseHeader import PolyNameT as lxoPolyNameT
from .LXO_ParseHeader import ITEM_LAYR_Mask

#### =======================================================================
__all__ = ("DebugPrint", )

#### ========================================================================
class DebugPrint(object):
	__slots__=()

	#### ========================================================================
	def __init__(self, *arg, **kwd):
		pass

	#### ========================================================================
	@property
	def DEBUG(self):
		return(self.VisFlags.DEBUG)
	@DEBUG.setter
	def DEBUG(self, v):
		self.VisFlags.DEBUG = bool(v)
	#### --------------------------------
	@property
	def VERB(self):
		return(self.VisFlags.VERB)
	@DEBUG.setter
	def VERB(self, v):
		self.VisFlags.VERB = bool(v)
	#### --------------------------------
	@property
	def FLAGS(self):
		return(self.VisFlags.FLAGS)
	@FLAGS.setter
	def FLAGS(self, flags):
		self.VisFlags.FLAGS = flags

	#### ========================================================================
	def Show_Layer(self, idx, **kwd):
		u'''Fn(<o=False>) print無し'''
		lyr = self.lwo.layers[idx]

		msg = []
		msg.append(u"# レイヤ {}".format(idx))
		msg.extend(self.__Layer(lyr, lyr.isHide))
		msg.append(u"")

		msg.append(u"  PNTS total: {} pts".format(lyr.pntNum))
		msg.extend(self.__Pnts_Pols(lyr.pnts, lyr.pntNum))
		msg.append(u"")

		msg.append(u"  POLS total: {} pols".format(lyr.polNum))
		msg.extend(self.__Pnts_Pols(lyr.pols, lyr.polNum))
		msg.append(u"")

		for nameKey in lyr.uvs.keys():
			msg.append(u"{:4}UV / {} ( {} )".format("", "VMap", nameKey))
			msg.extend(self.__UV(lyr.uvs[nameKey].vmap))
			msg.append(u"")

		for nameKey in lyr.uvs.keys():
			msg.append(u"{:4}UV / {} ( {} )".format("", "VMad", nameKey))
			msg.extend(self.__UV(lyr.uvs[nameKey].vmad))
			msg.append(u"")

		res = u"\n".join(msg) + "\n"
		if(kwd.get("o", True)):
			print(res)
		return(res)
		"""
		self.crvs  = ()			# ((flag, pntIdx,pntIdx,...),(...),...)
		self.bones = {}			# { idx: BoneObject, idx:... }
		self.picks  = {}		# { name: (pntIdx,pntIdx,...) }
		self.weights= {}		# { name: {pntIdx: w} }
		self.rgb    = {}		# { name: VMapObject: vmap{pntIdx: (r,g,b,_)}, vmad{polIdx: {pntIdx: (r,g,b,_)}} }
		self.rgba   = {}		# { name: VMapObject: vmap{pntIdx: (r,g,b,a)}, vmad{polIdx: {pntIdx: (r,g,b,a)}} }
		self.uvs    = {}		# { name: VMapObject: vmap{pntIdx: (u,v)}, vmad{polIdx: {pntIdx: (u,v)}} }
		self.absMorphs = {}		# { name: {pntIdx: (x,y,z)} }
		self.relMorphs = {}		# { name: {pntIdx: (x,y,z)} }
		self.tagColors = {}		# { colName: [polIdx,...] }
		self.tagParts  = {}		# { partName: [polIdx,...] }
		self.edgeWgts  = {}		# VMad/WGHT, { (id1 id2): val, (id1 id2): val,... }
		self.edgeWgtsM = {}		# VMed, { (id1 id2): val, (id1 id2): val,... }
		self.normals   = {}		# VMad/WGHT, { name: {polIdx: {pntIdx: (x,y,z)}} }
		self.surfTags  = {}		# { tagIdx: [polIdx,...] }
		(self.pntNum, self.polNum, self.crvNum, self.bonNum ) = (0, 0, 0, 0)
		(self.hasEdge, self.hasEdgeM) = (False, False)
		"""

	#### --------
	def Show_Tags(self):
		self.__DoPrintMsg(self.__Tags(self.lwo.tags))

	#### ========================================================================
	def DebugPrint_ItemTree_Lxo(self, treeDct, padding):
		u'''疑似ツリー辞書表示確認'''
		for (i, treeSub) in sorted(treeDct[0].items(), key=lambda x:x[0]):
			print("{:{PAD}}: {}".format(i, treeSub[1], PAD=padding))
			self.DebugPrint_ItemTree_Lxo(treeSub, padding+4)

	#### ========================================================================
	def __DoPrintMsg(self, msg):
		if(msg):
			print(u"\n".join(msg) + "\n")

	#### --------
	def __Pnts_Pols(self, pnts, pntNum):
		pad = len(str(pntNum))
		return([ u"{:4}{:{PAD}}: {}".format("", i, v, PAD=pad) for (i, v) in enumerate(pnts) ])

	#### --------
	def DebugPrint_Pnts(self, name, pnts, pntNum):
		if(self.DEBUG and self.VisFlags.PNT):
			msg = self.__Pnts_Pols(pnts, pntNum)
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"  PNTS 読込 ( {} ) total: {} pts".format(name, pntNum)),

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Pols(self, name, lwid, pols):
		if(self.DEBUG and self.VisFlags.POL):
			msg = self.__Pnts_Pols(pols, len(pols))
#			pad = len(str(len(pols)))
#			msg = [ u"{:4}{:{PAD}}: {}".format("", i, v, PAD=pad) for (i, v) in enumerate(pols) ]
		else:
			msg = []

		if(self.VERB):
			tag = lxoPolyNameT[lwid] if(self.fileType == "LXOB") else lwoPolyNameT[lwid]
			msg.insert(0, u"  {} 読込 ( {} ) total: {} pols".format(tag, name, len(pols)))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Crvs(self, name, tag, crvs, crvNum):
		if(self.DEBUG and self.VisFlags.CRV):
			pad = len(str(crvNum))
			msg = [ u"{:4}{:{PAD}}: {}".format("", i, v, PAD=pad) for (i, v) in enumerate(crvs) ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"  {} 読込 ( {} ) total: {} crvs".format(tag, name, crvNum))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Pick(self, name, pick):
		if(self.VERB):
			print(u"{:4}PICK / VMap ( {} )\n".format("", name))

		if(self.DEBUG and self.VisFlags.SELS):
			print("{:6}{}\n".format("", pick))

	#### --------
	def DebugPrint_Weight_VMap(self, name, wgt):
		if(self.VERB):
			print(u"{:4}WGHT / VMap ( {} )\n".format("", name))

		if(self.DEBUG and self.VisFlags.WGHT):
			print(u"{:6}{}\n".format("", wgt))

	#### --------
	def DebugPrint_EdgeWeight(self, name, tag, wgt):
		if(self.DEBUG and self.VisFlags.EDGE):
			msg = [ u"{:6}{}".format("", wgt) ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}EdgWgt / {} ( {} )".format("", tag, name))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Morph(self, name, vmapDict):
		if(self.DEBUG and self.VisFlags.MORF):
			pad = len(str(len(vmapDict)))
			msg = [ u"{:6}{:{PAD}}: {}".format("", k, v, PAD=pad) for (k, v) in sorted(vmapDict.items(), key=lambda x:x[0]) ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}MORF / VMap ( {} )".format("", name))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_RGBA(self, tag, name, vmapDict):
		if(self.DEBUG and self.VisFlags.VCOL):
			pad = len(str(len(vmapDict)))
			msg = [ u"{:6}{:{PAD}}: {}".format("", k, v, PAD=pad) for (k, v) in sorted(vmapDict.items(), key=lambda x:x[0]) ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}{} ( {} )".format("", tag, name))

		self.__DoPrintMsg(msg)

	#### --------
	def __UV(self, vmapDict):
		pad = len(str(len(vmapDict)))
		return([ u"{:6}{:{PAD}}: {}".format("", k, v, PAD=pad) for (k, v) in sorted(vmapDict.items(), key=lambda x:x[0]) ])

	#### --------
	def DebugPrint_UV(self, tag, name, vmapDict):
		if(self.DEBUG and self.VisFlags.UV):
			msg = self.__UV(vmapDict)
#			pad = len(str(len(vmapDict)))
#			msg = [ u"{:6}{:{PAD}}: {}".format("", k, v, PAD=pad) for (k, v) in sorted(vmapDict.items(), key=lambda x:x[0]) ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}UV / {} ( {} )".format("", tag, name))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_NORM(self, name, vmapDict):
		if(self.DEBUG and self.VisFlags.NORM):
			pad = len(str(len(vmapDict)))
			msg = [ u"{:6}{:{PAD}}: {}".format("", k, v, PAD=pad) for (k, v) in sorted(vmapDict.items(), key=lambda x:x[0]) ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}NORM / Vmad ( {} )".format("", name))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Bone(self, name, bones, bonNum):
		if(self.DEBUG and self.VisFlags.BONE):
			pad = len(str(len(bones)))
			msg = [ u"{:4}{:{PAD}}: ({}, {}) {}".format("", k, bo.pnt0, bo.pnt1, bo.childs, PAD=pad) for (k, bo) in bones.items() ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"  Bone 読込 ( {} ) total: {} bones".format(name, bonNum))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_PTagBone(self, tag, name, bones, typeName):
		if(self.DEBUG and self.VisFlags.BONE):
			pad = len(str(len(bones)))
			msg = [ u"{:6}{:{PAD}}: {}".format("", k, bo[tag], PAD=pad) for (k, bo) in bones.items() ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}PTag {}( {} )".format("", typeName, name))
			msg.append(u"{:6}End of PTag: {}".format("", typeName))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_PTag(self, tag, name, dat):
		if(self.DEBUG and self.VisFlags.PTAG):
			pad = len(str(len(dat)))
			msg = [ u"{:6}{:{PAD}}: {}".format("", k, v, PAD=pad) for (k, v) in dat.items() ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}PTag {}( {} )".format("", tag, name))
			msg.append(u"{:6}End of PTag: {}".format("", tag))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Surf(self, surf):
		msg = []
		if(self.DEBUG):
			if(self.VisFlags.SURF and self.VisFlags.IMAP):
				msg = []
				APD = msg.append
				for x in surf.__slots__:
					if(x != "imap"):
						APD(u"  {}: {}".format(x, surf[x]))
					else:
						for (i ,im) in enumerate(surf.imap):
							APD("  imap: [{}]".format(i))
							for z in im.__slots__:
								APD(u"{:4}{}: {}".format("", z, surf.imap[i][z]))
			elif(self.VisFlags.SURF):
				msg = [ u"  {}: {}".format(x, surf[x]) for x in surf.__slots__ if(x != "imap") ]

		if(self.VERB):
			msg.insert(0, u"# SURF 読込 ( {} )".format(surf.name))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Clip(self, clip):
		if(self.DEBUG and self.VisFlags.CLIP):
			msg = [ u"  {}: {}".format(x, clip[x]) for x in clip.__slots__ ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"# Clip 読込 ( {} )".format(clip.id))

		self.__DoPrintMsg(msg)

	#### --------
	def __Layer(self, lyr, flags):
		msg = [
			u"  Name: {}".format(lyr.name),
			u"  LayerIdx: {} (Parent: {})".format(lyr.idx, lyr.parent),
			u"  flag: {:x}".format(flags),
			u"  pivot: {}".format(lyr.piv),
			]
		if(self.fileType == "LXOB"):
			msg.extend([
				u"  scalePivot: {}".format(lyr.scalePiv),
				u"  sdsLevel: {}".format(lyr.sdsLvl),
				u"  curveLevel: {}".format(lyr.curveLvl),
				u"  splinePatchLevel: {}".format(lyr.splinePatchLvl),
				u"  itemRef: {}".format(lyr.itemRef),
				])
		return(msg)

	#### --------
	def DebugPrint_Layer(self, lyr, flags):
		if(self.VERB):
			msg = [u"# レイヤ {} を処理".format(lyr.idx)]
		else:
			msg = []

		if(self.DEBUG and self.VisFlags.LAYR):
			msg.extend(self.__Layer(lyr, flags))

		self.__DoPrintMsg(msg)

	#### --------
	def __Tags(self, tags):
		msg = [ u"  {},".format(a) for a in tags ]
		return([u"# TAGS:"] + msg)

	#### --------
	def DebugPrint_Tags(self, tags):
		if(self.DEBUG and self.VisFlags.TAGS):
			self.__DoPrintMsg(self.__Tags(tags))

	#### --------
	def DebugPrint_EdgeSelect(self, name, tag, sel):
		if(self.DEBUG and self.VisFlags.EDGE):
			msg = [ u"{:6}{}".format("", sel) ]
		else:
			msg = []

		if(self.VERB):
			msg.insert(0, u"{:4}EdgSel / {} ( {} )".format("", tag, name))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Items(self, it, idx):
		def __DctToStr(dct):
			if(dct):
				return("\n".join( [ "    {}".format(tpl) for tpl in dct.values() ] ))
			return("    []")
		def __LstToStr(lst):
			if(lst):
				return("\n".join( [ "    {}".format(tpl) for tpl in lst ] ))
			return("    []")
		#### --------
		if(self.VERB):
			msg = [u"# ITEM [{}] 読込 ( {} )".format(idx, it.name)]
		else:
			msg = []

		if(self.DEBUG):
			z = zip(
				(ITEM_LAYR_Mask.VISIBLE,ITEM_LAYR_Mask.HIDE,ITEM_LAYR_Mask.FG,ITEM_LAYR_Mask.BG,ITEM_LAYR_Mask.BBOX,ITEM_LAYR_Mask.LINEARUV),
				("Visible","Hidden","Foreground","Background","BBox","LinearSdsUV",)
				)
			lyrVis = " | ".join([ s for (shift, s) in z if(it.lyrMask & (0x1 << shift)) ])

			msg.extend((
				u"  type: {}".format(it.type),
				u"  referenceId: {}".format(it.referenceId),
				u"  packageName: {}".format(it.package),
				u"  xrefIdx: {}".format(it.xrefIdx),
				u"  xrefPath: {}".format(it.xrefPath),
				u"  xrefIdentifier: {}".format(it.xrefIdentifier),
				u"  layerIndex: {}".format(it.lyrIdx),
				u"  layerVisibility: 0x{:08X} / 0b{:08b} ( {} )".format(it.lyrMask, it.lyrMask, lyrVis),
				u"  layerWireColor: {}".format(hex(it.lyrRGBA).upper()[2:-1]),
				u"  itemLinks:\n{}".format(__DctToStr(it.itemLinks)),
				u"  chanLinks:\n{}".format(__LstToStr(it.chanLinks)),
				u"  userChans:\n{}".format(__LstToStr(it.userChans)),

				u"  scalarChans:\n{}".format(__DctToStr(it.scalarChans)),
				u"  vectorChans:\n{}".format(__DctToStr(it.vectorChans)),
				u"  stringChans:\n{}".format(__DctToStr(it.stringChans)),
				u"  gradientChans:\n{}".format(__DctToStr(it.gradientChans)),
				u"  chanVals:\n{}".format(__DctToStr(it.chanVals)),

				u"  customChans:\n{}".format(__LstToStr(it.customChans)),
				u"  itemTags:\n{}".format(__LstToStr(it.itemTags)),
				u"  uniqueIdentifier: {}".format(it.uniqueIdentifier),
				u"  userUniqueIndex: {}".format(it.userUniqueIndex),
				u"  visibleName: {}".format(it.visibleName),
				u"  B Channel Type: {}".format(it.BChanType),
				u"  B Channel Data Id: {}".format(it.BChanDataId),
				))

		self.__DoPrintMsg(msg)

	#### --------
	def DebugPrint_Actions(self, it, idx):
		def __ToStr(ar):
			if(ar):
				return("\n".join( [ "    {}".format(tpl) for tpl in ar ] ))
			return("    []")
		#### --------
		if(self.VERB):
			msg = [u"# ACTN [{}] 読込 ( {} )".format(idx, it.name)]
		else:
			msg = []

		if(self.DEBUG):
			msg.extend((
				u"  layerType: {}".format(it.layerType),
				u"  layerIdx: {}".format(it.layerIdx),
				u"  referenceId: {}".format(it.referenceId),
				u"  chanVals:\n{}".format(__ToStr(it.chanVals)),
				u"  gradientChans:\n{}".format(__ToStr(it.gradientChans)),
				u"  stringChans:\n{}".format(__ToStr(it.stringChans)),
				u"  namedChans:\n{}".format(__ToStr(it.namedChans)),
				u"  parentId: {}".format(it.parentId),
				))

		self.__DoPrintMsg(msg)
