﻿# -*- coding: utf-8 -*-

#### =======================================================================
__all__ = (
	"LwoObject", "LayerObject", "BoneObject", "VMapObject", "SurfaceObject",
	"ClipObject", "ImapBlokObject",
	"DictEx",
	)

#### ========================================================================
class DictEx(dict):
	'''dictのinst.key表記でも参照追加出来る版. Order非対応.'''
	def __init__(self, *arg, **kwd):
		super(DictEx, self).__init__(*arg, **kwd)
		self.__dict__ = self

#### ========================================================================
class LWO_Chunks(object):
	u'''instでの運用のみを想定'''
	# 一応の__dict__生成対策
	__slots__ = ()

	def items(self):
		return([ (k, getattr(self, k)) for k in self.__slots__ if(not k.startswith("__")) ])
	def values(self):
		return([ getattr(self, k) for k in self.__slots__ if(not k.startswith("__")) ])
	def keys(self):
		return([ k for k in self.__slots__ if(not k.startswith("__")) ])
	def __getitem__(self, k):
		return(getattr(self, k))
	def __setitem__(self, k, v):
		setattr(self, k, v)
# 不要
#	def __init__(self, *arg, **kwd):
#		pass

#### ========================================================================
class LwoObject(object):
	__slots__ = (
		"fileName",
		"tags", "layers", "surfs", "clips",
		)

	type = "LWO"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		self.fileName = ""

		self.tags   = []		# TAGS
		self.layers = []		# LAYR
		self.surfs  = {}		# SURF
		self.clips  = {}		# CLIP

#### ========================================================================
class LayerObject(LWO_Chunks):
	u'''レイヤ毎のデータ'''
	__slots__ = (
		"name", "idx", "isHide", "parent", "piv", "pols", "crvs",
		"bones", #"boneMaps", "boneNames", "boneRolls",
		"pnts", "picks", "weights", "rgb", "rgba", "uvs", "absMorphs", "relMorphs",
		"tagColors", "tagParts", "edgeWgts", "edgeWgtsM", "normals", "surfTags",
		"pntNum", "polNum", "bonNum", "crvNum", "hasEdge", "hasEdgeM", "lwid",
	)

	defaultName = "unnamed"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		self.name  = ""
		self.idx   = -1
		self.isHide= False
		self.parent= -1
		self.piv   = (0.0,)*3	# (x,y,z)
		self.pnts  = []			# [(x,y,z),(...),...]
		self.pols  = []			# [(pntIdx,pntIdx,...),(...),...]
		self.crvs  = ()			# ((flag, pntIdx,pntIdx,...),(...),...)
		self.bones = {}			# { idx: BoneObject, idx:... }
#		self.boneMaps={}		# 未使用
		self.picks  = {}		# { name: (pntIdx,pntIdx,...) }
		self.weights= {}		# { name: {pntIdx: w} }
		self.rgb    = {}		# { name: VMapObject: vmap{pntIdx: (r,g,b,_)}, vmad{polIdx: {pntIdx: (r,g,b,_)}} }
		self.rgba   = {}		# { name: VMapObject: vmap{pntIdx: (r,g,b,a)}, vmad{polIdx: {pntIdx: (r,g,b,a)}} }
		self.uvs    = {}		# { name: VMapObject: vmap{pntIdx: (u,v)}, vmad{polIdx: {pntIdx: (u,v)}} }
#		self.vmpa   = {}		#
		self.absMorphs = {}		# { name: {pntIdx: (x,y,z)} }
		self.relMorphs = {}		# { name: {pntIdx: (x,y,z)} }
		self.tagColors = {}		# { colName: [polIdx,...] }
		self.tagParts  = {}		# { partName: [polIdx,...] }
		self.edgeWgts  = {}		# VMad/WGHT, { (id1 id2): val, (id1 id2): val,... }
		self.edgeWgtsM = {}		# VMed, { (id1 id2): val, (id1 id2): val,... }
		self.normals   = {}		# VMad/WGHT, { name: {polIdx: {pntIdx: (x,y,z)}} }
		self.surfTags  = {}		# { tagIdx: [polIdx,...] }
		(self.pntNum, self.polNum, self.crvNum, self.bonNum) = (0, 0, 0, 0)
		(self.hasEdge, self.hasEdgeM) = (False, False)
		self.lwid = 0			# POLS:SSSS(サブチャンク)のint値

#### ========================================================================
class BoneObject(LWO_Chunks):
	u'''レイヤ毎のボーン'''
	__slots__ = (
		"name", "childs", "pnt0", "pnt1", "weight", "bnup",
		"skid", "skpr", "skbm", "skbr",
	)
#		"sked", "skon", "skcl",

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		if(kwd):
			self.name = kwd.get("name", None)		# name
			self.childs=kwd.get("childs", [])		# [child boneIdxs,...]
			self.pnt0 = kwd.get("pnt0", None)		# pntIdx
			self.pnt1 = kwd.get("pnt1", None)		# pntIdx
			self.weight=kwd.get("weight", None)		# weight map name
			self.bnup = kwd.get("bnup", None)		# upVector, (flt, flt, flt)
			self.skid = kwd.get("skid", None)		# id on skEdit
			self.skpr = kwd.get("skpr", None)		# parent id on skEdit
			self.skbm = kwd.get("skbm", None)		# bankMode, 0-8
			self.skbr = kwd.get("skbr", None)		# bankRotate, radian
#			self.sked = kwd.get("sked", None)		# skEdit tag str
#			self.skon = kwd.get("skon", None)		# name on skEdit
#			self.skcl = kwd.get("skcl", None)		# sketchColor on skEdit
		else:
			self.name = None
			self.childs=[]
			self.pnt0 = None
			self.pnt1 = None
			self.weight=None
			self.bnup = None
			self.skid = None
			self.skpr = None
			self.skbm = None
			self.skbr = None
#			self.sked = None
#			self.skon = None
#			self.skcl = None

#### ========================================================================
class VMapObject(LWO_Chunks):
	__slots__ = (
		"vmap", "vmad",
		)

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		self.vmap = {}
		self.vmad = {}

#### ========================================================================
class SurfaceObject(LWO_Chunks):
	u'''サーフェス毎のデータ'''
	__slots__ = (
		"name",
#		"srcName",
		"colr", "diff", "lumi", "spec", "refl", "tran", "trnl", "glos", 
#		"shrp",
		"bump", "side", "sman",
#		"rfop", "rimg", "rsan",
#		"rblr", "rind",
#		"trop", "timg", 
#		"tblr",
#		"clrh", "clrf", "adtr", "glow", "line", "alph",
		"vcol", "imap", "norm",
	)

	defaultName = "Default"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		self.name = kwd.get("n", self.defaultName)
#		self.srcName= ""
		self.colr = [1.0, 1.0, 1.0]
		self.diff = 1.0 # Diffuse
		self.lumi = 0.0 # Luminosity
		self.spec = 0.0 # Specular
		self.refl = 0.0 # Reflectivity
		self.tran = 0.0 # Transparency
		self.trnl = 0.0 # Translucency
		self.glos = 0.4 # Glossiness
#		self.shrp = 0.0 # Diffuse Sharpness
		self.bump = 1.0 # Bump Intensity
		self.side = 0   # polySide, front:0,1, F&back:3
		self.sman = -1.56259327931 # Smooth Threshould, if Disable, Default(-89.53, 0xBFC8030E)
#		self.rfop = 
#		self.rimg = 
#		self.rsan = 
#		self.rblr = 0.0 # Reflection Bluring
#		self.rind = 1.0 # RT Transparency IOR
#		self.trop = 
#		self.timg = 
#		self.tblr = 0.0 # Refraction Bluring
#		self.clrh = 
#		self.clrf = 
#		self.adtr = 
#		self.glow = 
#		self.line = 
#		self.alph = 
		self.vcol = ()	# (name, lwid, intensity)
		self.imap = []	# [ImapBlokObject,ImapBlokObject,...]
		self.norm = ""	# mapName

#### ========================================================================
class ClipObject(LWO_Chunks):
	u'''イメージクリップ'''
	__slots__ = (
		"id", "type",
		"stillName",
		"seqDigit", "seqFlags", "seqOffset", "seqReserve", "seqStart", "seqEnd", "seqName", "seqExt",
		"animName", "animServer", "animFlags",
		"xrefIdx", "xrefStr",
#		"stccLo", "stccHi", "stccName",
		"stTime", "duration", "fps",
		"neg",
#		"cont", "brit", "satr", "hue", "gamm", "iflt", "pflt",
	)
	#### -----------------------------------------------------------------------
	def __init__(self):
		# super不要
		self.id   = 0
		self.type = 0
		self.stillName= ""
		(self.seqDigit, self.seqFlags, self.seqOffset, self.seqReserve) = (0, 0, 0, 0)
		(self.seqStart, self.seqEnd) = (0, 0)
		(self.seqName, self.seqExt, self.animName, self.animServer) = ("", "", "", "")
		self.animFlags = 0
		self.xrefIdx   = 0
		self.xrefStr   = ""
#		(self.stccLo, self.stccHi) = (0, 0)
#		self.stccName = ""
		(self.stTime, self.duration, self.fps) = (0, 0, 0)
		self.neg = 0

#### ========================================================================
class ImapBlokObject(LWO_Chunks):
	u'''イメージマップ'''
	__slots__ = (
		"ord", "chan", "opacType", "opac", "enable", "negative",
		"displAxis", "center", "size", "rotate", "project", "prjAxis", "imgIdx",
		"wrapOptW", "wrapOptH", "wrapAmountW", "wrapAmountH", "vmapName",
	)
	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		self.ord = 0
		self.chan = ""
		self.opacType = 0
		self.opac = 0.0
		self.enable   = False
		self.negative = False
		self.displAxis= 0
		(self.center, self.size, self.rotate) = ([0.0]*3, [0.0]*3, [0.0]*3)
		self.project = 0
		self.prjAxis = 0
		self.imgIdx  = 0
		(self.wrapOptW, self.wrapOptH, self.wrapAmountW, self.wrapAmountH) = (0, 0, 0, 0)
		self.vmapName = ""
