﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys
from collections import namedtuple

from . import LWO_Object

if(sys.modules["Load_LWO"].DO_RELOAD):
#	print("reload parse")
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("LWO_Object",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

# LWO Objectで定義済. LXO Objectとして再配布.
from .LWO_Object import VMapObject, SurfaceObject

#### =======================================================================
__all__ = (
	"LxoObject", "LayerObject", "ItemObject", "TransformObject", "LocatorObject", "MeshObject",
	"iLINK", "iCLNK", "iUCHN", "iCHNL", "iCHNV", "iCHNS", "iGRAD", "iCHAN", "iITAG",
	"aCHNN", "aCHNS", "aGRAD", "aCHAN",

	"VMapObject", "SurfaceObject",
	)

#### ========================================================================
if(sys.version_info[0] >= 3):
	parm = {}
else:
	parm = dict(verbose=False)

iLINK = namedtuple("iLINK", ["n", "id", "idx"], **parm)
iCLNK = namedtuple("iCLNK", ["n", "fromName", "refId", "toName", "fromIdx", "toIdx"], **parm)
iUCHN = namedtuple("iUCHN", ["n", "typ", "mode", "flags", "defInt", "defFlt", "hintCnt", "hints"], **parm)
iCHNL = namedtuple("iCHNL", ["n", "typ", "v"], **parm)
iCHNV = namedtuple("iCHNV", ["n", "typ", "dim", "v"], **parm)
iCHNS = namedtuple("iCHNS", ["n", "s"], **parm)
iGRAD = namedtuple("iGRAD", ["n", "idx", "interp"], **parm)
iCHAN = namedtuple("iCHAN", ["idx", "typ", "v", "n"], **parm)
iITAG = namedtuple("iITAG", ["id", "v", "lwid"], **parm)

aCHAN = namedtuple("aCHAN", ["idx", "typ", "envIdx", "v", "n"], **parm)
aGRAD = namedtuple("aGRAD", ["idx", "envIdx", "interp", "n"], **parm)
aCHNS = namedtuple("aCHNS", ["n", "idx", "s", "chnm"], **parm)
aCHNN = namedtuple("aCHNN", ["n", "typ", "envIdx", "v"], **parm)

del parm

#### ========================================================================
class ActnObject(LWO_Object.LWO_Chunks):
	__slots__ = (
		"name", "layerType", "layerIdx",
		"referenceId",
		"chanVals", "gradientChans", "stringChans", "namedChans",
		"parentId",
		)

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		self.name = None				# ActionLayerName
		self.layerType = None			# ActionLayerType
		self.layerIdx = -1				# ActionLayerIndex
		self.referenceId = -1			# root.LAYR.itemRefとの紐付. layer.indexでは無い.
		self.chanVals = LWO_Object.DictEx()		# { CHNM:(CHNM idx, 値種, ENVLidx, 値|(値,...), CHNM) }
		self.gradientChans = LWO_Object.DictEx()# gradientチャネル{ CHNM:(CHNM idx, ENVLidx, 補間種, CHNM) }
		self.stringChans = LWO_Object.DictEx()	# 文字列チャネル{ CHNM:(name, CHNM idx, 値, CHNM) }
		self.namedChans = []			# ( (name, 値種?, 値, envelope?, 値),... )
		self.parentId = -1				# 

#### ========================================================================
class ItemObject(LWO_Object.LWO_Chunks):
	__slots__ = (
		"type", "name", "referenceId", "package",
		"xrefIdx", "xrefPath", "xrefIdentifier",
		"lyrIdx", "lyrMask", "lyrRGBA",
		"itemLinks", "chanLinks", "userChans", "scalarChans", "vectorChans", "stringChans",
		"gradientChans", "chanVals", "customChans",
		"itemTags", "uniqueIdentifier", "userUniqueIndex", "visibleName",
		"BChanType", "BChanDataId",
		)

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		"""
		type:
			(A)locator, mesh, meshInst, camera, sunLight, groupLocator, 
			(B)transform(pivot類の仮想層?),
			(C)translation, rotation, scale, 
			scene, environment, backdrop, 
			shaderFolder, videoStill, 
			advancedMaterial, mask, lightMaterial, envMaterial, 
			renderOutput,
			(*)polyRender, 
			他

		B.itemLink.xfrmCore → A

		C.chanVals.piv
			C.itemLink.xfrmCore → A
			C.itemLink.xfrmLocal → B
		C.chanVals.pivc / core
			C.itemLink.xfrmCore → A

		itemLinks:
			n: parent, source, meshInst, xfrmCore, xfrmLocal, deformer?

		vectorChans:
			type:
				n: pos, rot, scl

		chanVals:
			type: str
				v: piv(==PivotPosition), pivc, inv, core(標準t/r/s?), sys(== PivotRotation), 7(== PivotRotationCompensation)
			order: str
				xyz...

		インスタンス構成
		LAYR.itemRef → ITEM(type:mesh, link:meshInst) ← ITEM(type:meshInst, link:source)
		"""
		self.type = None
		self.name = None				# ユーザ定義名称. 明示的定義が無い時は空.
		self.referenceId = -1			# 別itemから[]で参照する等. items[root.LAYR.itemRef]とか.
		self.package = None				# 

		self.xrefIdx = -1				# 
		self.xrefPath = None			# 
		self.xrefIdentifier = None		# 

		self.lyrIdx = -1				# UIのツリー表示の上からの順番. 上[0]～下[N]. その場合parentも無い.
										# 何かの子になっている場合は-1. layer.indexでは無い.
		self.lyrMask = 0				# objectとしての可視性等
		self.lyrRGBA = 0				# UIでのワイヤ表示色. u_int32_t. RGBA.

		self.itemLinks = LWO_Object.DictEx()	# 親(parent), deformer等. { type:[(type, id, 順),...] }
		self.chanLinks = []						# channelFollower的な物?( (リンク種, fromChanName, fromRefId, toChanName, fromLinkIdx, toLinkIdx),... )
		self.userChans = []						# カスタム(ユーザー定義)チャネル
		self.scalarChans = LWO_Object.DictEx()	# スカラーチャネル{ name:(name, 値種, 値) }
		self.vectorChans = LWO_Object.DictEx()	# ベクターチャネル{ name:(name, 値種, 次数, (値,...)) }
		self.stringChans = LWO_Object.DictEx()	# 文字列チャネル{ name:(name, 値) }
		self.gradientChans = LWO_Object.DictEx()# gradientチャネル{ name:(name, ENVLidx, 補間種) }
		self.chanVals = LWO_Object.DictEx()		# { CHNM:(CHNM idx, 値種, 値|(値,...), CHNM) }
		self.customChans = []					# 
		self.itemTags = []						# [(lwid, string, lwid文字),...]

		self.uniqueIdentifier = None	# 一意の名称
		self.userUniqueIndex = -1		# 名称重複時のUI上の表示index
		self.visibleName = None			# 表示名？ name→uniq→visName？ 事実上不要？
		self.BChanType = None			# 
		self.BChanDataId = -1			# 

#### ========================================================================
class TransformObject(LWO_Object.LWO_Chunks):
	u'''groupLocator'''
	__slots__ = (
		"name",
		"isVis",
		"parentUniqueIdentifier",
		"parentOrderIdx",
		"userUniqueIndex",
		"lyrRGBA",

		"buildedObject",
		"ownItem",
		"treeParentsList",

		"translate", "rotate", "scale",
		"rotateOrder",

		"pivot",
		"pivotRotate", "pivotRotateOrder",
		)

	type = "transform"
	defaultName = "Group_Locator"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# super不要
		tmp = kwd.get("n", self.defaultName)
		self.name = self.defaultName if(not tmp) else tmp	# 空文字対策

		# ITEM(groupLocator/locator/mesh).lyrMask&(0x1<<0)
		self.isVis = kwd.get("v", True)

		# ITEM(groupLocator/locator/mesh).iLink(parent)→parentItem.uniqueIdentifier
		self.parentUniqueIdentifier = kwd.get("pui", None)
		# ITEM(groupLocator/locator/mesh).iLink(parent).idx
		self.parentOrderIdx = kwd.get("poi", -1)

		# ITEM(groupLocator/locator/mesh).userUniqueIndex
		self.userUniqueIndex = kwd.get("uui", 1)

		# ワイヤ表示色 ITEM:0xXXXXXX → float(r,g,b)
		self.lyrRGBA = kwd.get("wc", None)

		# 作成したmesh, group, locator固有の名やdagpの格納用
		self.buildedObject = None

		# ITEM[]内の自身item
		self.ownItem = kwd.get("oi", None)

		# 親子階層のuniqName. [rootUniq, ..., ownUniq]
		# 一時的に階層毎のparentIdx格納にも使う. [rootIdx, ..., ownIdx]
		self.treeParentsList = None

		# ITEM(t/r/s).LINK(xfrmCore)→ITEM(g/locator)
		self.translate = kwd.get("t", [0.0]*3)	# ITEM(translation).CHNV.pos
		self.rotate = kwd.get("r", [0.0]*3)		# ITEM(rotation).CHNV.rot
		self.scale = kwd.get("s", [1.0]*3)		# ITEM(scale).CHNV.scl

		self.rotateOrder = kwd.get("ro", "zxy")	# ITEM(rotation).CHAN.order.v

		self.pivot = kwd.get("piv", [0.0]*3)

		self.pivotRotate = kwd.get("pvr", [0.0]*3)
		self.pivotRotateOrder = kwd.get("pro", "zxy")

#### ========================================================================
class LocatorObject(TransformObject):
	u'''locator'''
	__slots__ = TransformObject.__slots__ + (
		"size",
		)

	type = "locator"
	defaultName = "Locator"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		super(LocatorObject, self).__init__(self, *arg, **kwd)

		# defaultNameの為に再履行
		tmp = kwd.get("n", self.defaultName)
		self.name = self.defaultName if(not tmp) else tmp	# 空文字対策

		# ITEM(locator).CHAN.size
		# 同.FBX_LocatorSize はインポートの名残？
		self.size = kwd.get("ls", 10.0)

#### ========================================================================
class MeshObject(TransformObject):
	u'''mesh/meshInst'''
	__slots__ = TransformObject.__slots__ + (
		"sdsLvl", "isPlaceHolder", "isSds",
		"buildedCrvs",
		)

	type = "mesh"
	defaultName = "Mesh"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		super(MeshObject, self).__init__(self, *arg, **kwd)

		# defaultNameの為に再履行
		tmp = kwd.get("n", self.defaultName)
		self.name = self.defaultName if(not tmp) else tmp	# 空文字対策

		self.sdsLvl = 0.0		# sds level. layer.sdsLvl(float)
		self.isSds = False		# SUBD/PSUB由来? ModoとMayaはLWの様に面単位のSDS化は出来ない.

		# Layer/Instanceが未構築で、階層構築の為にtransformを作った場合はTrue
		self.isPlaceHolder = False

		# カーブ
		self.buildedCrvs = []

#### ========================================================================
class LxoObject(object):
	__slots__ = (
		"fileName",
		"saverName", "majorVer", "minorVer",
		"encoding",
		"tags", "layers", "surfs", "clips", "items", "actions",
		"channelNames", 

		"transforms",
		"locators",
		"meshes",
		)

	type = "LXO"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		self.fileName = ""
		self.saverName= ""
		self.majorVer = 0
		self.minorVer = 0
		self.encoding = 0

		self.tags   = []		# TAGS
		self.layers = []		# LAYR
		self.surfs  = {}		# SURF(lwo), lxoではITEMからの生成物をSURFとして格納
		self.clips  = {}		# CLIP(lwo), ToDo:lxoではITEMからの生成物をCLIPとして格納
		self.items  = []		# ITEM
		self.actions = []		# ACTN

		self.channelNames = []	# CHNM

		self.transforms = {}	# ITEMから抽出したtransformアイテム
		self.locators = {}		# ITEMから抽出したlocatorアイテム
								# { uniqueIdentifier: trs/locObject }
		self.meshes = {}		#

#### ========================================================================
class LayerObject(LWO_Object.LayerObject):
	u'''レイヤ毎のデータ'''
	__slots__ = LWO_Object.LayerObject.__slots__ + (
		"bcrvs",
		"scalePiv", "sdsLvl", "curveLvl", "splinePatchLvl", "itemRef",

		"tagSelects",
		"edgeSel",

		#### ----------------
		"uniqueIdentifier"
		)

	defaultName = "Mesh"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		# LWOのLayerObject初期化が必要
		super(LayerObject, self).__init__(*arg, **kwd)

		self.bcrvs  = ()			# ((flag, pntIdx,pntIdx,...),(...),...)

		self.scalePiv = (0.0,)*3	# (x,y,z) 未使用？
		self.sdsLvl = 1.0
		self.curveLvl = 1.0
		self.splinePatchLvl = 1.0
		self.itemRef = -1			# ITEM[itemRef]

		self.tagSelects = {}		# { selName: [polIdx,...] }
		self.edgeSel = []			# VMED:EPCK

		# layerからmeshes辞書で名前参照可能にする
		uniqueIdentifier = ""	# ITEM[lyr.itemRef].uniqueIdentifier
