﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

from . import Chunk_Object

if(sys.modules["Load_LWO"].DO_RELOAD):
	from . import LWO_ParseHeader

	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Chunk_Object", "LWO_ParseHeader"):
		if(f in loc):
			Fn(eval(f))
	del f, loc

# LWO Headerで定義済. LXO Headerとして再配布.
from .LWO_ParseHeader import ReadT, RSize

#### =======================================================================
__all__ = (
	"ReadT", "RSize",

	"ENCO_Type", "ITEM_LAYR_Mask", "ITEM_UCHN_Mode", "ITEM_CHNL_Type", "ITEM_GRAD_Flag", "ITEM_CHAN_Type", 
	"ENVL_TANI_SlopeType", "ENVL_TANI_WeightType", "ENVL_TANO_Flag", "ENVL_PRE_Type", 

	"ITEM_ITAG_COLR",

	"PolyIdT", "VMapIdT", "VMedIdT", "PTagIdT", "RefsIdT", "ItemIdT", "EnvlIdT", 
	"ActnIdT", "AnniIdT", "IassIdT", "SubsIdT", "XrefIdT", "ChunkIdT",

	"PolyNameT",
	"ChunkNameT",
	)
#	"_PolyT", "_VMapT", "_VMedT", "_PTagT", "_RefsT", "_ItemT", 
#	"_EnvlT", "_ActnT", "_AnniT", "_IassT", "_SubsT", "_XrefT", "_ChunkT",

#### =======================================================================
#### ポリゴンタイプ ----------------
class _PolyT(Chunk_Object.ChunkObject):
	FACE = b"FACE"	# A standard polygon.
	CURV = b"CURV"	# Catmull-Rom curve.
	LINE = b"LINE"	# Polyline.
	BEZR = b"BEZR"	# Bezier curve.
	SPCH = b"SPCH"	# Spline patch
	TEXT = b"TEXT"	# Text polygons
	SUBD = b"SUBD"	# Subdivision surface polygons.
	PSUB = b"PSUB"	# Catmull-Clark subdivision surface, Pixar Subdivision surface.
	HCRV = b"HCRV"	# Hermite curve. As of 701 CURV is depricated in favour of HCRV.
	BCRV = b"BCRV"	# Bezier curve. As of 701 BEZR is depricated in favour of BCRV.
	OPNT = b"OPNT"	# One point polygon. Added in 13.1v1 RC2 Build 559310.
	BSPL = b"BSPL"	# Bezier spline curve. (Same layout as with HCRV/BCRV).
	__slots__=()

#### 頂点マップタイプ ----------------
class _VMapT(Chunk_Object.ChunkObject):
	# VMAD, VVEC兼(PICK除く)
	MORF = b"MORF"	# 相対モーフ
	NORM = b"NORM"	# perPolPnt法線
	PICK = b"PICK"	# 選択セット
	RGB_ = b"RGB "	# 頂点カラー(RGB)
	RGBA = b"RGBA"	# 頂点カラー(RGBA)
	SPOT = b"SPOT"	# 絶対モーフ
	TXUV = b"TXUV"	# UV
	WGHT = b"WGHT"	# ウェイト
	TBAS = b"TBAS"	# タンジェント
	EPCK = b"EPCK"	# エッジ選択(名前のみ) VMEDのみを見ればよい
	__slots__=()

#### Modo LWO ----------------
class _VMedT(Chunk_Object.ChunkObject):
	SUBV = b"SUBV"	# A subdivision edge weight map.
	EPCK = b"EPCK"	# A edge selection set map.
	__slots__=()

#### ポリゴンタグ ----------------
class _PTagT(Chunk_Object.ChunkObject):
	# TTGS兼
	MATR = b"MATR"	# material
	PART = b"PART"	# part
	COLR = b"COLR"	# pol color
	PICK = b"PICK"	# selection
	FONT = b"FONT"	# 
	JUST = b"JUST"	# 
	TEXT = b"TEXT"	# 
	SMGP = b"SMGP"	# smoothing group
	__slots__=()

####  ----------------
class _RefsT(Chunk_Object.ChunkObject):
	IDEL = b"IDEL"	# none7
	__slots__=()

####  ----------------
class _ItemT(Chunk_Object.ChunkObject):
	PAKG = b"PAKG"
	XREF = b"XREF"
	LAYR = b"LAYR"
	BBOX = b"BBOX"	# none7. under layr/item
	LINK = b"LINK"
	CLNK = b"CLNK"
	UCHN = b"UCHN"
	CHNL = b"CHNL"
	CHNV = b"CHNV"
	CHNS = b"CHNS"
	GRAD = b"GRAD"
	CHAN = b"CHAN"
	CHNC = b"CHNC"	# none7
	ITAG = b"ITAG"
	UNIQ = b"UNIQ"
	UIDX = b"UIDX"	# none7
	VNAM = b"VNAM"	# none7
	BCHN = b"BCHN"	# none7
	__slots__=()

####  ----------------
class _EnvlT(Chunk_Object.ChunkObject):
	TANI = b"TANI"
	TANO = b"TANO"
	KEY_ = b"KEY "
	PRE_ = b"PRE "
	POST = b"POST"
	__slots__=()

####  ----------------
class _ActnT(Chunk_Object.ChunkObject):
	ITEM = b"ITEM"
	CHAN = b"CHAN"
	GRAD = b"GRAD"
	CHNS = b"CHNS"
	PRNT = b"PRNT"	# only7
	CHNN = b"CHNN"	# only7
	__slots__=()

####  ----------------
class _AnniT(Chunk_Object.ChunkObject):
	AAIT = b"AAIT"	# none7
	AASE = b"AASE"	# none7
	__slots__=()

####  ----------------
class _IassT(Chunk_Object.ChunkObject):
	XREF = b"XREF"	# none7
	__slots__=()

####  ----------------
class _SubsT(Chunk_Object.ChunkObject):
	LOPT = b"LOPT"	# none7
	IREF = b"IREF"	# none7
	__slots__=()

####  ----------------
class _XrefT(Chunk_Object.ChunkObject):
	IDEL = b"IDEL"	# none7
	XMAN = b"XMAN"	# none7
	__slots__=()

#### ----------------
# [FORM][size][LXOB][TAGS]...
class _ChunkT(Chunk_Object.ChunkObject):
	FORM = b"FORM"
	LXOB = b"LXOB"

	AUTH = b"AUTH"	# only7
	CPY_ = b"(c) "	# only7 #### 括弧不可
	ANNO = b"ANNO"	# only7

	VRSN = b"VRSN"
	APPV = b"APPV"	# none7
	THUM = b"THUM"	# none7
	PRVW = b"PRVW"
	DESC = b"DESC"	# none7
	ENCO = b"ENCO"	# none7
	TAGS = b"TAGS"	# none7
	CHNM = b"CHNM"

	LAYR = b"LAYR"
	PNTS = b"PNTS"
	BBOX = b"BBOX"	# none7. under layr/item
	VMPA = b"VMPA"
	VMAP = b"VMAP"
	VMAD = b"VMAD"
	VMED = b"VMED"
	POLS = b"POLS"
	PTAG = b"PTAG"

	BAKE = b"BAKE"	# none7
	GRP3 = b"3GRP"	#### 行頭数値不可
	SRF3 = b"3SRF"	#### 行頭数値不可
	VRTS = b"VRTS"
	TRIS = b"TRIS"
	VVEC = b"VVEC"
	TTGS = b"TTGS"
	DATA = b"DATA"	# none7

	REFS = b"REFS"	# none7

	ITEM = b"ITEM"

	ENVL = b"ENVL"

	ACTN = b"ACTN"

	ANNI = b"ANNI"	# none7

	IASS = b"IASS"	# none7

	SUBS = b"SUBS"	# none7

	XREF = b"XREF"	# none7

	__slots__=()

#### =======================================================================
# str : id
PolyIdT = Chunk_Object.ChunkKeyId("PolyIdT", _PolyT)
VMapIdT = Chunk_Object.ChunkKeyId("VMapIdT", _VMapT)
VMedIdT = Chunk_Object.ChunkKeyId("VMedIdT", _VMedT)
PTagIdT = Chunk_Object.ChunkKeyId("PTagIdT", _PTagT)
RefsIdT = Chunk_Object.ChunkKeyId("RefsIdT", _RefsT)
ItemIdT = Chunk_Object.ChunkKeyId("ItemIdT", _ItemT)
EnvlIdT = Chunk_Object.ChunkKeyId("EnvlIdT", _EnvlT)
ActnIdT = Chunk_Object.ChunkKeyId("ActnIdT", _ActnT)
AnniIdT = Chunk_Object.ChunkKeyId("AnniIdT", _AnniT)
IassIdT = Chunk_Object.ChunkKeyId("IassIdT", _IassT)
SubsIdT = Chunk_Object.ChunkKeyId("SubsIdT", _SubsT)
XrefIdT = Chunk_Object.ChunkKeyId("XrefIdT", _XrefT)
ChunkIdT= Chunk_Object.ChunkKeyId("ChunkIdT",_ChunkT)

#### ----------------
# id : str
PolyNameT = Chunk_Object.ChunkIdKey("PolyNameT", _PolyT)
"""
VMapNameT = Chunk_Object.ChunkIdKey("VMapNameT", _VMapT)
VMedNameT = Chunk_Object.ChunkIdKey("VMedNameT", _VMedT)
PTagNameT = Chunk_Object.ChunkIdKey("PTagNameT", _PTagT)
RefsNameT = Chunk_Object.ChunkIdKey("RefsNameT", _RefsT)
ItemNameT = Chunk_Object.ChunkIdKey("ItemNameT", _ItemT)
EnvlNameT = Chunk_Object.ChunkIdKey("EnvlNameT", _EnvlT)
ActnNameT = Chunk_Object.ChunkIdKey("ActnNameT", _ActnT)
AnniNameT = Chunk_Object.ChunkIdKey("AnniNameT", _AnniT)
IassNameT = Chunk_Object.ChunkIdKey("IassNameT", _IassT)
SubsNameT = Chunk_Object.ChunkIdKey("SubsNameT", _SubsT)
XrefNameT = Chunk_Object.ChunkIdKey("XrefNameT", _XrefT)
"""
ChunkNameT= Chunk_Object.ChunkIdKey("ChunkNameT",_ChunkT)

#### =======================================================================
class ENCO_Type(object):
	DEFAULT = 0
	ANSI    = 1
	UTF8    = 2
	SJIS    = 3
	EUCJP   = 4
	EUCKR   = 5
	GB2312  = 6
	BIG5    = 7
	sizeof  = 8
	NAMES = ("Default", "ANSI", "UTF-8", "Shift-JIS", "EUC-JP", "EUC-KR", "GB2312", "BIG5")
	__slots__=()

#### ----------------
class ITEM_LAYR_Mask(object):
	VISIBLE  = 0
	HIDE     = 1
	FG       = 2
	BG       = 3
	BBOX     = 4
	LINEARUV = 8
	__slots__=()

class ITEM_UCHN_Mode(object):
	SCALAR = 0
	XY     = 1
	XYZ    = 2
	RGB    = 3
	RGBA   = 4
	__slots__=()

class ITEM_CHNL_Type(object):
	# CHNV兼
	I4 = 1
	F4 = 2
	S0 = 3
	XX = 4		# 701.pdf CUSTOM扱い. 14.2には無い.
	# 対象がVector(pos,rot,scl,color)の時の実測値(0x22). 本来の値の時もある.
	# 33,34,35が存在する. 実測からの予想型. 0xN1, 0xN2, 0xN3, N>=0？
	IVEC= 33
	FVEC= 34
	SVEC= 35

	INTS = { I4, IVEC }
	FLTS = { F4, FVEC }
	STRS = { S0, SVEC }
	__slots__=()

class ITEM_GRAD_Flag(object):
	# ACTN_GRAD兼
	CURVE   = 0
	LINEAR  = 1
	STEPPED = 2
	__slots__=()

class ITEM_CHAN_Type(object):
	# ACTN_CHAN兼
	INT = 1
	FLT = 2
	STR = 3
	ENV_INT = 17
	ENV_FLT = 18
	ENV_STR = 19
	# 33,34,35が存在する. 実測からの予想型. 0xN1, 0xN2, 0xN3, N>=0？
	INT_ = 33
	FLT_ = 34
	STR_ = 35

	INTS = { INT, ENV_INT, INT_ }
	FLTS = { FLT, ENV_FLT, FLT_ }
	STRS = { STR, ENV_STR, STR_ }
	__slots__=()

#### ----------------
class ENVL_TANI_SlopeType(object):
	# TANO兼
	MANUAL     = 0
	AUTOMATIC  = 1
	LINEAR_IN  = 2
	LINEAR_OUT = 4
	FLAT       = 8
	AUTO_FLAT  = 16
	STEPPED    = 32
	SMOOTH     = 64
	__slots__=()

class ENVL_TANI_WeightType(object):
	# TANO兼
	AUTOMATIC = 0
	MANUAL    = 256
	__slots__=()

class ENVL_TANO_Flag(object):
	NONE   = 0
	VALUE  = 1
	SLOPE  = 2
	WEIGHT = 3
	__slots__=()

class ENVL_PRE_Type(object):
	# POST兼
	RESET     = 0
	CONST_SMT = 1
	REPEAT    = 2
	OSC_SMT   = 3
	OFFSET    = 4
	LINEAR    = 5
	CONST     = 6
	OSCILLATE = 7
	OFFSET_REP= 8
	__slots__=()

#### ----------------
def _ToFlt(*iv):
	cof = 1.0/255.0
	return(iv[0] * cof, iv[1] * cof, iv[2] * cof)
class ITEM_ITAG_COLR(Chunk_Object.ChunkObject):
	#### ----------------
	red        = _ToFlt(226, 93, 74)
	pink       = _ToFlt(202, 150, 151)
	orange     = _ToFlt(234, 179, 83)
	yellow     = _ToFlt(226, 198, 93)
	green      = _ToFlt(102, 183, 103)
	lightgreen = _ToFlt(169, 217, 169)
	blue       = _ToFlt(126, 183, 208)
	lightblue  = _ToFlt(159, 198, 221)
	purple     = _ToFlt(174, 159, 207)
	lightpurple= _ToFlt(207, 188, 234)
	white      = _ToFlt(234, 234, 234)
	grey       = _ToFlt(183, 183, 183)
	__slots__=()
