﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import print_function

import maya.cmds as mc

from . import Defines

#### =======================================================================
__all__ = ("Apply", "Remove", "ApplyWeightsByAttr")

#### ========================================================================
def Apply(*arg):
	ApplyWeightsByAttr()

#### ------------------------------------------------------------------------
def __GetAttrOwner(ob):
	u'''アトリビュート持ちのshapeを返す.'''
#	ob = filter(lambda o: mc.attributeQuery(Defines.WgtAttr.LN, n= o, ex=True), ob)
	ob = [ o for o in ob if(mc.attributeQuery(Defines.WgtAttr.LN, n= o, ex=True)) ]
	if(not ob):
		print("Aya?\n"),
	return(ob)

#### ------------------------------------------------------------------------
def __GetSelection():
	ob = mc.listRelatives(mc.ls(sl=True, l=True), c=True, s=True, f=True, typ= Defines.NdType.MESH)
	if(not ob):
		print("aya?\n"),
	return(ob)

#### ------------------------------------------------------------------------
def Remove(*arg):
	ob = __GetSelection()
	if(not ob):
		return

	owners = __GetAttrOwner(ob)	# Attr所有者を探す.
	if(not owners):
		return

	for sh in owners:
		mc.deleteAttr(sh, at= Defines.WgtAttr.LN)
		print(u"{} アトリビュートを除去した.\n".format(sh)),

#### ========================================================================
def __GetWeightsFromAttr(o):
	ch = mc.attributeQuery(Defines.WgtAttr.LN, n= o, lc=True)
	if(not ch):
		return(None)

	wgtDct = {}
	for c in ch:
		wgtDct[c] = mc.getAttr("{}.{}.{}".format(o, Defines.WgtAttr.LN, c))
	return(wgtDct)

#### ------------------------
def __NormalizeWgts(wgtDct):
	pntNum = len(wgtDct.values()[0])
	arMaxs = [0] * pntNum
	arPntCnt = range(pntNum)

	for ar in wgtDct.values():
		for i in arPntCnt:
			arMaxs[i] += ar[i]

	for i in arPntCnt:
		if(arMaxs[i] != 0.0):
			arMaxs[i] = 1.0 / arMaxs[i]

	for ar in wgtDct.values():
		for i in arPntCnt:
			ar[i] *= arMaxs[i]

	return(wgtDct)

#### ------------------------
def __MakeWgtValueArray(wgtDct, idx):
	if(len(wgtDct.values()[0]) < idx):
		return(None)

	res = []
	APD = res.append
	maxVal = 0
	for (nm, vals) in wgtDct.items():
		maxVal += vals[idx]
		APD([nm, vals[idx]])

	if(maxVal > 0.0):
		return(res)

	return(None)

#### ------------------------
def __ExcludeWeightsBySC(skn, wgtDct):
	hst = mc.listHistory(skn, il=2, pdo=False, lf=True, lv=1) or []
	fea = []
#	fea = mc.listHistory(skn, il=2, pdo=False, lf=True, lv=1, f=True) or []

#	jnts = map(lambda j: j.rsplit("|",1)[-1], mc.ls(list(set(hst + fea)), typ= Defines.NdType.JNT))
	jnts = [ j.rsplit("|",1)[-1] for j in mc.ls(list(set(hst + fea)), typ= Defines.NdType.JNT) ]

	for at in wgtDct.keys():	# attr名
		if(at not in jnts):
			try:
				del wgtDct[at]
			except KeyError:
				continue
	return(wgtDct)

#### ------------------------------------------------------------------------
def ApplyWeightsByAttr(*arg):
	# 付与したAttrを保持するShapeはバインド設定によって位置づけが変わる.
	# 中間Shapeも取った上でAttr所有者を探す.
	ob = __GetSelection()
	if(not ob):
		return

	owners = __GetAttrOwner(ob)	# Attr所有者を探す.
	if(not owners):
		return

	for sh in owners:
		# 所有者のヒストリを軽くさらった上でlsでフィルタしてskinClusterを拾う.
		# beforeではヒストリ、afterではフューチャーで辿る必要あり
		hst = mc.listHistory(sh, il=2, pdo=True, lf=True, lv=0) or []
		fea = mc.listHistory(sh, il=2, pdo=True, lf=True, lv=0, f=True) or []
		skns = list(set(mc.ls(hst + fea, typ= Defines.NdType.SKN)))
		if(not skns):
			mc.warning(u"skinCluster が見つからない."),
			continue
		skn = skns[0]

		# SCの先にあるmeshを取る
		meshs = mc.ls(mc.listHistory(skn, f=True, il=2, pdo=False, lf=True, lv=0) or [], typ= Defines.NdType.MESH)
		if(not meshs):
			continue

		wgtDct = __GetWeightsFromAttr(sh)			# Attrから辞書{ bone:[v,v,...], bone:[v,v,...]}
		if(not wgtDct):
			continue

		wgtDct = __ExcludeWeightsBySC(skn, wgtDct)	# SCで未使用のwgt名を除外
		if(not wgtDct):
			continue

		wgtDct = __NormalizeWgts(wgtDct)			# vを正規化して{ bone:[n,n,...], bone:[n,n,...]}

		for me in meshs:
			arPnts = mc.ls("{}.vtx[*]".format(me), fl=True)	# 展開した頂点文字列を得る
			for (i, pnt) in enumerate(arPnts):
				r = __MakeWgtValueArray(wgtDct, i)	# tv=用の[[bone,w][bone,w],...]を作る
				if(r):
					mc.skinPercent(skn, pnt, nrm=True, zri=True, tv= r)
			print(u"{} から {} へ適用した.\n".format(me, skn)),
