﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

import maya.cmds as mc

if(sys.modules["Load_LWO"].USE_OM2):
	from .om2 import Meshes, Curves, Joints, Dags
else:
	from .om1 import Meshes, Curves, Joints, Dags

from . import Opts, Defines

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Opts", "Meshes", "Curves", "Joints", "Dags", "Defines",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

# メモ
# 2022になっても以下は無い.
# oa2.MFnBlendShape, oa2.MFnIkJoint, om2.MFnXXXShader

#### =======================================================================
__all__ = ("BuildObjects",)

#### ========================================================================
def BuildObjects(lwo, **kwd):

	opts = kwd.get("o", kwd.get("opts", None))
	if((not opts) or (not isinstance(opts, Opts.OptsObject))):
		opts = Opts.OptsObject()

	# actListはfile内格納順に応じたboolのリスト, 便宜上,[空]も認める
	actList = kwd.get("al", kwd.get("actList", None))
	if(actList == None):	# LWO単位での選択, 指定とフラグからactListを作成
		actList = tuple( (not (lyr.isHide & opts.skipHide)) for lyr in lwo.layers )
	else:					# Layer単位での選択, 保険として比較
		if(len(lwo.layers) != len(actList)):
			mc.warning(u"GUIとファイルでレイヤ数が不一致"),
			return

	for (lyr, isActive) in zip(lwo.layers, actList):
		if(isActive):
			if(opts.isMesh):
				Meshes.MakeMeshObject(lwo, lyr, opts)

			if(opts.isCrv):
				arTrs = Curves.MakeCurveObject(lyr, opts.vScl, opts.deg)
				Dags.Grouping(Defines.GrpVisT.SHOW, arTrs, "{}_Curve_grp".format(lyr.name))

			if(opts.isBone):
				arTrs = Joints.MakeJointObject(lyr, opts.vScl, opts.isZAxis)
				Dags.Grouping(Defines.GrpVisT.SHOW, arTrs, "{}_Joint_grp".format(lyr.name))
