﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

import maya.cmds as mc

if(sys.modules["Load_LWO"].USE_OM2):
	from .om2 import Meshes, Curves, Joints, Dags
else:
	from .om1 import Meshes, Curves, Joints, Dags

from . import Opts, Defines

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Opts", "Meshes", "Curves", "Joints", "Dags", "Defines",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

from ..LXO_ParseHeader import PolyIdT

# メモ
# 2022になっても以下は無い.
# oa2.MFnBlendShape, oa2.MFnIkJoint, om2.MFnXXXShader

#### =======================================================================
__all__ = ("BuildObjects",)

#### ========================================================================
def BuildObjects(lxo, **kwd):
	opts = kwd.get("o", kwd.get("opts", None))
	if((not opts) or (not isinstance(opts, Opts.OptsObject))):
		opts = Opts.OptsObject()
	else:
		# lxoではlayer.pivは使わない. ITEM指定で処理.
		opts.usePivot = False

	# actListはfile内格納順に応じたboolのリスト, 便宜上,[空]も認める
	actList = kwd.get("al", kwd.get("actList", None))
	if(actList == None):	# LWO単位での選択, 指定とフラグからactListを作成
		actList = tuple( (not (lyr.isHide & opts.skipHide)) for lyr in lxo.layers )

	else:					# Layer単位での選択, 保険として比較
		if(len(lxo.layers) != len(actList)):
			mc.warning(u"GUIとファイルでレイヤ数が不一致"),
			return

	for (lyr, isActive) in zip(lxo.layers, actList):
		if(isActive):
			if(opts.isMesh):
				dagp = Meshes.MakeMeshObject(lxo, lyr, opts)
				if(dagp):
					obj = lxo.meshes[lyr.uniqueIdentifier]
					obj.buildedObject = dagp
					obj.sdsLvl = lyr.sdsLvl
					obj.isSds = bool(lyr.lwid in {PolyIdT.PSUB, PolyIdT.SUBD})

			if(opts.isCrv):
				arTrs = Curves.MakeCurveObject(lyr, opts.vScl, opts.deg)
				dagp = Dags.Grouping(Defines.GrpVisT.SHOW, arTrs, "{}_Curve_grp".format(lyr.name))
				if(dagp):
					obj = lxo.meshes[lyr.uniqueIdentifier]
					obj.buildedCrvs.append(dagp)

	# mesh(instance)作成
	Dags.MeshInstancing(lxo)

	# lyrでもitemでもない中間obj, mesh/instanceを埋めてから参照用にコピー
	totalDct = lxo.transforms.copy()
	totalDct.update(lxo.locators)
	totalDct.update(lxo.meshes)

	Dags.SetMeshesNodeAttr(lxo, opts.useWire, opts.scl)

	Dags.MakeNodes(lxo, opts.scl, totalDct)

	Dags.ParentAndMove(lxo, opts.scl, totalDct)

	Dags.Rename(lxo)

	Dags.Set_OutlinerColor(lxo)
