﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .. import LWO_Object

#### =======================================================================
__all__ = ("OptsObject",)

#### ========================================================================
class OptsObject(LWO_Object.LWO_Chunks):
	__slots__ = (
		"isMesh", "isBone", "isCrv", "__scl", "__vScl", "deg", "useEdge", "usePivot", "mergeUV",
		"loadRM", "loadAM", "useRM", "useAM", "useWgt", "useXML", "isZAxis", "isDblSide",
		"useVCol", "useMiVC", "shareSG", "useColT", "useBmpT", "useArnold",
		"usePick", "useColr", "usePart",
		"usePPic", "useWire",
		"mtlType", "skipHide",
	)
	defaultScl = 100.0
	defaultVScl = [defaultScl, defaultScl, -defaultScl]
	defaultDeg = 3
	(LAMB, BLIN, PHON, SURFS) = (0,1,2,3)	# build用故0から
	#### -----------------------------------------------------------------------
	def __init__(self, **kwd):
		# super不要
		scl = kwd.get("s", self.defaultScl)
		self.isMesh  = True		# Mesh作成
		self.isBone  = False	# Bone作成
		self.isCrv   = False	# Curve作成
		self.__scl   = scl		# 倍率(UI用)
		self.deg     = kwd.get("d", self.defaultDeg)	# Curve次数
		self.useEdge = True		# エッジウェイト適用
		self.usePivot= False	# ピボット適用
		self.mergeUV = True		# Mesh作成後のマージUV
		self.loadRM  = False	# モーフ(相対)形状作成
		self.loadAM  = False	# モーフ(絶対)形状作成
		self.useRM   = False	# モーフ(相対)をBlendShapeNodeに適用
		self.useAM   = False	# モーフ(絶対)をBlendShapeNodeに適用
		self.shareSG = True		# 既存の同名SGを共用
		self.useVCol = False	# 頂点カラーのmeshNodeへの適用
		self.useMiVC = False	# mentalrayVertexColorsNodeの適用
		self.usePick = False	# 頂点選択セット(PICK)をSetsに適用
		self.useColr = False	# 面カラー(COLR)をSetsに適用
		self.usePart = False	# 面パート(PART)をSetsに適用
		self.usePPic = False	# 面選択(PICK)をSetsに適用(lxo)
		self.useWire = False	# WireColorをshape.objectColorに適用(lxo)
		self.useColT = False	# Colorの画像からfileNode+layeredTextureを作成/接続
		self.useBmpT = False	# Bumpの画像からfileNode+layeredTexture+Bump2dを作成/接続
		self.useArnold= False	# file/aiImage
		self.mtlType = kwd.get("m", self.LAMB)	# 使用マテリアルタイプ(0:lambert, 1:blinn, 2:phong, x:SurfaceShader)
		self.skipHide= False	# 不可視属性Layerを無視
		self.useWgt  = False	# (Layer内にBoneがある場合)ウェイト値をMesh|Shape.attrに書込み
		self.useXML  = False	# (Layer内にBoneがある場合)ウェイト値をdeformerWeight用XMLを書き出し
		self.isZAxis = True		# LWスタイルのボーンとして扱う
		self.isDblSide=False	# Shapeオプション: 両面
		self.__vScl = [scl,scl,-scl]	# Z座標反転した倍率ベクトル(Build用)
	#### --------------------------------
	@property
	def vScl(self):
		return(self.__vScl)
	@vScl.setter
	def vScl(self, v):
		self.__scl = v
		self.__vScl = [v, v, -v]
	#### --------------------------------
	@property
	def scl(self):
		return(self.__scl)
	@scl.setter
	def scl(self, v):
		self.__scl = v
		self.__vScl = [v, v, -v]
