﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

import maya.cmds as mc

from . import Defines

if(sys.modules["Load_LWO"].DO_RELOAD):
	from .. import LWO_ParseHeader

	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("LWO_ParseHeader", "Defines",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

from ..LWO_ParseHeader import VMapIdT, SurfIdT, TXPRJ

#### ========================================================================
__all__ = ("SetSurfaceAttr", "MakeNode",)

#### ========================================================================
def __BuildTextureMap(imaps, clips, newName, useColT, useBmpT, isStdSurf, useArnold):
	# 現状有効なマップタイプをピックアップ
	activeMaps = sorted([ mp for mp in imaps if(mp.project == TXPRJ.UVMAP) ], key= lambda x: x.ord)	# 序数でソート

	(colNum, bmpNum) = (0, 0)
	for mp in activeMaps:					# レイヤ用、現状で有効な属性毎のレイヤ数
		if(  mp.chan == SurfIdT.COLR): colNum += 1
		elif(mp.chan == SurfIdT.BUMP): bmpNum += 1

	(colLtName, bumpLtName) = ("", "")
	if(colNum * useColT):
		colLtName  = MakeNode(Defines.NdType.LAYR, n= newName.replace(Defines.SFixNM.MTL, Defines.SFixNM.LWCOL), t=True)
		colFMT = "{}.bc" if(isStdSurf) else "{}.c"
		mc.connectAttr("{}.oc".format(colLtName), colFMT.format(newName), f=True)

	if(bmpNum * useBmpT):
		bumpLtName = MakeNode(Defines.NdType.LAYR,  n= newName.replace(Defines.SFixNM.MTL, Defines.SFixNM.LWBMP), t=True)
		bmpName = MakeNode(Defines.NdType.BUMP, n= Defines.NameFMT.B2D.format(NAME= newName.replace(Defines.SFixNM.MTL, "")), t=True)
		mc.setAttr("{}.ail".format(bumpLtName), True)								# alphaIsLuminance
		mc.connectAttr("{}.oa".format(bumpLtName), "{}.bv".format(bmpName), f=True)	# outAlpha->bumpValue
		mc.connectAttr("{}.o".format(bmpName), "{}.n".format(newName), f=True)		# outNormala->normalCamera

	for mp in activeMaps:
		if(mp.imgIdx == 0):	# ファイル未指定時は0、clips[0]は存在しない
			fpath = ""
		else:
			fpath = clips[mp.imgIdx].stillName
			# drv:\pathでなくdrv:pathの場合の対処. パスとしてのセパレータは \ で.
			if((fpath[1] == ":") and (fpath[2] != "\\")):
				fpath = fpath[0:2] + "\\" + fpath[2:]

		if(fpath != ""):
			ndName = fpath.replace("\\", "/").rsplit("/", 1)[-1].split(".",1)[0] + Defines.SFixNM.MTL
		else:
			ndName = newName

		bsName = ndName.replace(Defines.SFixNM.MTL, Defines.SFixNM.FILE)
		if(useArnold):
			(name, _) = AiImageNode(n= bsName, ftn= fpath)
			outColorFMT = "{}.out"
		else:
			(name, _) = FileNode(n= bsName, ftn= fpath)
			outColorFMT = "{}.oc"

		if(useColT and (mp.chan == SurfIdT.COLR)):
			idx = mc.getAttr("{}.cs".format(colLtName), s=True)
			mc.connectAttr(outColorFMT.format(name), "{}.cs[{}].c".format(colLtName, idx), f=True)

		if(useBmpT and (mp.chan == SurfIdT.BUMP)):
			idx = mc.getAttr("{}.cs".format(bumpLtName), s=True)
			mc.connectAttr(outColorFMT.format(name), "{}.cs[{}].c".format(bumpLtName, idx), f=True)

#### ------------------------------------------------------------------------
def __ConnectVCol(vcolName, mapNames, newName, isStdSurf):
	# surfaceでVcol指定をしていてもmeshにVcolが無い事がある
	if(mapNames):
		if(vcolName in mapNames):
			colFMT = "{}.bc" if(isStdSurf) else "{}.c"	# outColor, baseColor|color
			mc.connectAttr("{}.oc".format(Defines.NameFMT.VCOL.format(NAME= mapNames[vcolName])), colFMT.format(newName), f=True)

#### ------------------------------------------------------------------------
def SetSurfaceAttr(surfs, clips, mtlDct, rgbNames, rgbaNames, opts):
	u'''mtlDct{ surfIdx:[tagName, mtName, sgName] }'''
	# StandardSurfaceShader自体が2020からなのでver縛り.
	if((opts.mtlType == Defines.ShdType.SURFS) and (int(mc.about(v=True))>=2020)):
		for (tagName, mtlName, _) in mtlDct.values():
			# lxoではptagと紐付くsurfが無い(値取得していない)物もある事への対応.
			if(tagName not in surfs):
				continue

			srf = surfs[tagName]
			mc.setAttr("{}.baseColor".format(mtlName),    *srf.colr, **Defines.VType.DBL3)			# color
			mc.setAttr("{}.base".format(mtlName),          srf.diff)								# diffuse
			mc.setAttr("{}.emissionColor".format(mtlName),*([srf.lumi]*3), **Defines.VType.DBL3)	# ambientColor
			mc.setAttr("{}.transmission".format(mtlName),  srf.tran)								# transparency

#			if(opts.useMiVC and (len(srf.vcol) != 0)):
#				mapNames = rgbNames if(srf.vcol[1] == VMapIdT.RGB_) else rgbaNames
#				__ConnectVCol(srf.vcol[0], mapNames, mtlName, True)

#			if((opts.useColT|opts.useBmpT) and len(srf.imap)):
#				__BuildTextureMap(srf.imap, clips, mtlName, opts.useColT, opts.useBmpT, True, opts.useArnold)
	else:
		for (tagName, mtlName, _) in mtlDct.values():
			# lxoではptagと紐付くsurfが無い(値取得していない)物もある事への対応.
			if(tagName not in surfs):
				continue

			srf = surfs[tagName]
			mc.setAttr("{}.c".format(mtlName),    *srf.colr, **Defines.VType.DBL3)			# color
			mc.setAttr("{}.dc".format(mtlName),    srf.diff)								# diffuse
			mc.setAttr("{}.ambc".format(mtlName), *([srf.lumi]*3), **Defines.VType.DBL3)	# ambientColor
			mc.setAttr("{}.it".format(mtlName),   *([srf.tran]*3), **Defines.VType.DBL3)	# transparency
			mc.setAttr("{}.tc".format(mtlName),    srf.trnl)								# translucence

			if(opts.useMiVC and (len(srf.vcol) != 0)):
				mapNames = rgbNames if(srf.vcol[1] == VMapIdT.RGB_) else rgbaNames
				__ConnectVCol(srf.vcol[0], mapNames, mtlName, False)

			if((opts.useColT|opts.useBmpT) and len(srf.imap)):
				__BuildTextureMap(srf.imap, clips, mtlName, opts.useColT, opts.useBmpT, False, opts.useArnold)

#### ========================================================================
__gZip_FileAttr = (
	(".c",  ".c"),		# coverage
	(".tf", ".tf"),		# translateFrame
	(".rf", ".rf"),		# rotateFrame
	(".mu", ".mu"),		# mirrorU
	(".mv", ".mv"),		# mirrorV
	(".s",  ".s"),		# stagger
	(".wu", ".wu"),		# wrapU
	(".wv", ".wv"),		# wrapV
	(".re", ".re"),		# repeatUV
	(".of", ".of"),		# offset
	(".r",  ".ro"),		# rotateUV
	(".n",  ".n"),		# noiseUV
	(".vt1",".vt1"),	# vertexUvOne
	(".vt2",".vt2"),	# vertexUvTwo
	(".vt3",".vt3"),	# vertexUvThreevertexCameraOne
	(".vc1",".vc1"),	# vertexCameraOne
	(".o",  ".uv"),		# outUV/uvCoord
	(".ofs",".fs"),		# outUvFilterSize/uvFilterSize
)

__gNodeCategoryDct = dict(
	s= "defaultShaderList1.s",
	u= "defaultRenderUtilityList1.u",
	t= "defaultTextureList1.tx",
	l= "lightList1.l",
	r= "defaultRenderingList1.r",
	p= "postProcessList1.p",
	i= "defaultColorMgtGlobals",
)

#### -----------------------------------------------------------------------
def MakeNode(typ, *arg, **kwd):
	# s:shader, t:tex, l:lgt, p:posPro, u:uty, r:rend, i:colMan
	n = kwd.get("n", kwd.get("name", None))
	parm = dict(n= n) if(n) else {}
	nd = mc.createNode(typ, ss=True, **parm)

	for k in kwd.keys():
		dl = __gNodeCategoryDct.get(k, None)
		if(dl):
			mc.connectAttr("{}.msg".format(nd), dl, na=True)

	return(nd)

#### =======================================================================
def FileNode(n=None, *arg, **kwd):
	u'''ファイルノード作成
  FileNode(name, <ft=0>, <f=0.01>, <ftn="">, <autoTx=False>, <dc=(.5,.5,.5)>, <pre="P2D_">, <post="_file">, <psd=False>)
    name= str, ノード名, 未指定ならdefault.
    ft(filter)= int, 0:No, 1:MipMap, 2:Box, 3:Quadratic, 4:Quartic 5:Gaussian
    f(filterValue)= float
    ftn(fileTexName)= str, "filePath/fileName"
    dc(defaultColor)= (float, float, float)
    autoTx = False
    pre= str, ノード名へのprefix
    post= str, ノード名へのpostfix
    psd= bool PSDノード
    useP2D= bool P2D使用, 未指定時はmaya設定依存'''

	filterType  = kwd.get("ft", 0)
	filterValue = kwd.get("f", 0.01)
	fileTexName = kwd.get("ftn", "")
	defaultColor = kwd.get("dc", None)	# default 0.5
	autoTx = kwd.get("autoTx", False)	# default True
	mipmapBias = kwd.get("mipmapBias", 0)	# default 0

	P2D_Prefix  = kwd.get("pre",  "P2D_")
	FILE_Postfix= kwd.get("post", "_file")

	isPSD = kwd.get("psd", False)

	parm = dict(n= n) if(n) else {}
	fd = MakeNode(Defines.NdType.FILE if(not isPSD) else "psdFileTex", t=True, **parm)
	useP2D = kwd.get("useP2D", mc.optionVar(q= "createTexturesWithPlacement"))
	if(useP2D):
		parm = dict(n= P2D_Prefix + n.replace(FILE_Postfix, "")) if(n) else {}
		pd = MakeNode(Defines.NdType.P2D, t=True, **parm)
	else:
		pd = None

	mc.setAttr("{}.ft".format(fd), filterType)		# filterType
	mc.setAttr("{}.f".format(fd), filterValue)		# filter, ft問わず値を与える

	if(fileTexName):
		mc.setAttr("{}.ftn".format(fd), fileTexName, **Defines.VType.STR)	# fileTextureName
	if(defaultColor):
		mc.setAttr("{}.dc".format(fd), *defaultColor, **Defines.VType.DBL3)	# defaultColor

	if(useP2D):
		for (pa,fa) in __gZip_FileAttr:
			mc.connectAttr(pd + pa, fd + fa, f=True)

	if(mc.attributeQuery("aiFilter", n= fd, ex=True)):
		mc.setAttr("{}.aiFilter".format(fd), 0 if(not filterType) else 3)	# closest or smartbicubic
	if(mc.attributeQuery("aiAutoTx", n= fd, ex=True)):
		mc.setAttr("{}.aiAutoTx".format(fd), autoTx)
	if(mc.attributeQuery("aiMipBias", n= fd, ex=True)):
		mc.setAttr("{}.aiMipBias".format(fd), mipmapBias)

	return(fd, pd)

#### -----------------------------------------------------------------------
def AiImageNode(n=None, *arg, **kwd):
	u'''ファイルノード作成
  AiImageNode(name, <ft=0>, <ftn="">, <autoTx=False>, <dc=(.5,.5,.5)>, <mipmapBias=0>, <post="_file">)
    name: str, ノード名, 未指定ならdefault.
    ft(aiFilter): int, 0:Closest, 1:bilinear, 2:bicubic, 3:smart-bicubic
    ftn: str, "filePath/fileName"
    autoTx = False
    uv: str, uvMap名'''

	filterType  = kwd.get("ft", 0)
	fileName = kwd.get("ftn", "")
	autoTx = kwd.get("autoTx", False)	# default True
	defaultColor = kwd.get("dc", (.5, .5, .5))	# default 0
	mipmapBias = kwd.get("mipmapBias", 0)	# default 0

	parm = dict(n= n) if(n) else {}
	fd = MakeNode("aiImage", t=True, **parm)

	mc.setAttr("{}.filter".format(fd), filterType)
	mc.setAttr("{}.autoTx".format(fd), autoTx)
	mc.setAttr("{}.ignoreMissingTextures".format(fd), True)
	mc.setAttr("{}.mipmapBias".format(fd), mipmapBias)

	if(defaultColor):
		mc.setAttr("{}.missingTextureColor".format(fd), *defaultColor, **Defines.VType.DBL3)

	if(fileName):
		mc.setAttr("{}.filename".format(fd), fileName, **Defines.VType.STR)

	if("uv" in kwd):
		mc.setAttr("{}.uvset".format(fd), kwd["uv"], **Defines.VType.STR)

	return(fd, None)
