﻿# -*- coding: utf-8 -*-

import math

import maya.cmds as mc

#### ========================================================================
def NormalSmooth(fullPathName, tags, surfs, surfTags, buildedPolIdxs, numPolygons):
	u'''法線スムージング, 無効時のsmanは－値'''
	smoothThresholdLst = []
	for (idx, mappedPolIdxs) in surfTags.items():
		tagName = tags[idx]
		if(tagName in surfs):
			polIdxs = tuple( buildedPolIdxs[polIdx] for polIdx in mappedPolIdxs if(buildedPolIdxs[polIdx] is not None) )
			smoothThresholdLst.append( [math.degrees(max(surfs[tagName].sman, 0.0)), polIdxs] )

	if(smoothThresholdLst):
		FMT = "{}.f[{{}}]".format(fullPathName)
		# 許容値の大きい部分から適用していく. LWとMayaは見掛けは近似するが, Modoは異なる. Blenderは?
		for (degree, polIdxs) in sorted(smoothThresholdLst, key=lambda x:x[0], reverse=True):
			if(len(polIdxs) == numPolygons):
				mc.polySoftEdge(FMT.format("*"), a= degree, ch=False)
				break
			else:
				tmp = [ FMT.format(s) for s in __IdxValsToRangeStrs(polIdxs) ]
				if(tmp):	# 空の場合もあり
					mc.polySoftEdge(*tmp, a= degree, ch=False)

#### ========================================================================
def __IdxValsToRangeStrs(idxLst):
	u'''[int,...]を["int","int:int",...]へ
src:[38, 39, 40, 18, 19, 20, 21, 22, 28, 29, 8, 9, 8, 7, 6, 100, 101, 101, 100, 99, 15, 15, 17]
res:['38:40', '18:22', '28:29', '6:9', '99:101', '15', '17']'''
	#### --------
	def func1(start, before, a, APD, buf):
		if(2 > (a - before) > -2):
			buf.append(a)
			return(start, a)

		func2(APD, buf)

		del buf[:]	# ptrなので注意, buf=[]不可
		buf.append(a)
		return(a, a)

	#### --------
	def func2(APD, buf):
		t = sorted(list(set(buf[:])))
#		if(start == before):
		if(len(t) == 1):
			APD("{}".format(t[0]))
		else:
			APD("{}:{}".format(t[0],t[-1]))

	#### --------
	if((not idxLst) or (len(idxLst) < 2)): return([])

	res = []
	APD = res.append
	start = before = idxLst[0]
	buf = [start]
	for i in idxLst[1:]:
		(start, before) = func1(start, before, i, APD, buf)

	func2(APD, buf)	# 終端部の補足処理として必要

	return(res)
