﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import print_function

import os

import maya.cmds as mc

from . import Defines

#### =======================================================================
__all__ = ("MakeWeightMap",)

#### ========================================================================
def MakeWeightMap(meName, bones, weights, pnts, pols, buildedPolIdxs, opts):
	# {vmap:[v,v,...], vmap:[v,v,...],...}
	pntNum = len(pnts)

#	wgtDct = __MakeWgtDict(bones, weights, pntNum)

	# polで使われていた頂点をプロット, [0,None,3,4,...]
	usedPnts = [None] * pntNum
	for polIdx in buildedPolIdxs:
		if(polIdx != None):
			for pntIdx in pols[polIdx]:
				usedPnts[pntIdx] = pntIdx

	# 点ポリ,線ポリ,スケルゴンの分を除外する, [0,3,4,...]
	# 要素[N]は実在する事になった頂点のidxに相当し、値VがLWO上のidxを指す
	activePnts = tuple( pnt for pnt in usedPnts if(pnt!=None) )

	wgtDct = __MakeWgtDict(bones, weights, activePnts)
	if(wgtDct):
		if(opts.useXML):
			xm = __DeformerWeightsXML(meName, wgtDct, pnts, activePnts, opts.vScl)
			__WriteXML(meName, xm)

		if(opts.useWgt):
			__SetAttr(meName, wgtDct, pnts, activePnts)	# 素のAttr版
#			__MetaDataStruct(meName, wgtDct, pnts, activePnts)	# metadata版テスト

#### ========================================================================
# metadata版 editの実行コストが異常に高いので実用性無し
"""
def __MetaDataStruct(meName, wgtDct, pnts, activePnts):
	ST_NAME = "st_lwWgtMap"
	V_NAME  = "val"
	metas = mc.dataStructure(q=True)
	if(ST_NAME not in metas):
		mc.dataStructure(fmt= "raw", asString= "name={}:float={}".format(ST_NAME, V_NAME))
	for (bone, weights) in wgtDct.items():
		mc.addMetadata(meName, stn= bone, str= ST_NAME, cn= "vertex")
		for (i, wgt) in enumerate((weights[pntIdx] for pntIdx in activePnts)):
			mc.editMetadata("{}.vtx[{}]".format(meName, i), stn= bone, mn= V_NAME, v= wgt)
		#	mc.editMetadata(meName, stn= bone, mn= V_NAME, idx= i, v= wgt)
"""
#### ========================================================================
def __SetAttr(meName, wgtDct, pnts, activePnts):
	# 消すのは親Attrだけでよい(子も消える). 工程上既設である事は無い.
	#if(mc.attributeQuery(Defines.WgtAttr.SN, n= meName, ex=True)): mc.deleteAttr(meName, at= Defines.WgtAttr.LN)

	# floatArrayを有効vmap数持つ複合Attrとして親Attrを作る
	mc.addAttr(meName, ln= Defines.WgtAttr.LN, sn= Defines.WgtAttr.SN, at="compound", nc= len(wgtDct))

	# 子Attrを全て作ってからでないとSetAttr出来ない. key==bone名でソート
	for bone in sorted(wgtDct.keys()):
		mc.addAttr(meName, p= Defines.WgtAttr.LN, ln= bone, dt= Defines.VType.FLTA["typ"])	# FLTAのvalueを参照

	actPntsIdxs = range(len(activePnts))
	FMT_STR = "{}.{}.{{}}".format(meName, Defines.WgtAttr.LN)
	for (bone, weights) in wgtDct.items():	# *tuple()ではない
		mc.setAttr(FMT_STR.format(bone), tuple(weights[pntIdx] for pntIdx in actPntsIdxs), **Defines.VType.FLTA)

#### ========================================================================
def __WriteXML(meName, xmlStr):
#:	path = mc.workspace(q=True, rd=True) + mc.workspace(fre="scene")	# 2017から？ このfre表記でないとNG
	path = os.path.join(mc.workspace(q=True, rd=True), mc.workspace(fre="scene"), "").replace("\\", "/")
	# 2017から？ このfre表記でないとNG
	if(not os.path.exists(path)):
		print("not found {}".format(path))
		return

	xmlFile = os.path.join(path, "{}.xml".format(meName.rsplit("|",1)[-1])).repalce("\\", "/")
#:	fp = open("{}/{}.xml".format(path, meName.rsplit("|",1)[-1]), "w")
	fp = open(xmlFile, "w")
	if(not fp):
		print(u"file io error")
		return

	fp.write(xmlStr)
	fp.close()

#### ========================================================================
def __DeformerWeightsXML(meName, wgtDct, pnts, activePnts, vScl):
	#mc.deformerWeightsのxml

	shapeName = meName.rsplit("|",1)[-1]
	actPntNum = len(activePnts)
	lastIdx = actPntNum -1

	res = []
	APD = res.append

	APD(u'<?xml version="1.0"?>')
	APD(u'<deformerWeight>')

	trsPos = (0.0, 0.0, 0.0)
	APD(u'  <headerInfo worldMatrix="1.0 0.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 0.0 1.0 0.0 {:.6f} {:.6f} {:.6f} 1.0"/>'.format(
		trsPos[0], trsPos[1], trsPos[2],
	))

	APD(u'  <shape name="{}" group="{}" stride="{}" size="{}" max="{}">'.format(shapeName, 0, 3, actPntNum, lastIdx))
	# stride==valueの次数?, group==export時にscに刺さってたgroupIDの数

	pntCnt = 0
	for pntIdx in activePnts:	# polで使われていた頂点idx
		pnt = pnts[pntIdx]
		APD(u'    <point index="{}" value="{:.6f} {:.6f} {:.6f}"/>'.format(pntCnt, pnt[0]*vScl[0], pnt[1]*vScl[1], pnt[2]*vScl[2]))
		pntCnt += 1
	APD(u'  </shape>')

	weightCnt = 0
	for (wgtName, weights) in wgtDct.items():
		tmp = []
		TMP = tmp.append
		maxIdx = 0
		pntCnt = 0	# enumerの代わり
		for pntIdx in activePnts:
			if(weights[pntCnt] != 0.0):
				TMP(u'    <point index="{:d}" value="{:.6f}"/>'.format(pntCnt, weights[pntCnt],))
				maxIdx = pntCnt
			pntCnt += 1

		APD(u'  <weights deformer="{}" source="{}" shape="{}" layer="{}" defaultValue="0.0" size="{}" max="{}">'.format(
			"dummy", wgtName, shapeName, weightCnt, len(tmp), maxIdx))
		res.extend(tmp)
		APD(u'  </weights>')

		"""
		APD(u'  <weights deformer="{}" source="{}" shape="{}" layer="{}" defaultValue="0.0" size="{}" max="{}">'.format(
			"dummy", wgtName, shapeName, weightCnt, actPntNum, lastIdx))

		pntCnt = 0	# enumerの代わり
		for pntIdx in activePnts:
			APD(u'    <point index="{:d}" value="{:.6f}"/>'.format(pntCnt, weights[pntIdx],))
			pntCnt += 1
		APD(u'  </weights>')
		"""
		weightCnt += 1
	APD(u'</deformerWeight>\n')

	return("\n".join(res))

#### ========================================================================
def __MakeWgtDict(bones, weights, activePnts):
	# ボーンが参照している(有効な)vmap. [[vmapName, boneName],[],...]
	# vmap割り当てが有る場合,有効な変形用ボーンとして拾う
	activeMaps = tuple( (bon.weight, bon.name) for (idx, bon) in bones.items() if(bon.weight != None) )

	# pntIdxからvmap毎の値配列を用意. idxがSkelegonPntなら拾わない.
	# {vmap:[v,v,...], vmap:[v,v,...],...}
	wgtDct = {}
	for (vmap, bone) in activeMaps:
		if(vmap not in weights):
			print(u"Bone: {} の WeightMap: {} が見つからない.".format(bone, vmap))
		else:
			wgt = weights[vmap]
			tmp = []
			APD = tmp.append
			for i in activePnts:
				APD(wgt[i] if(i in wgt) else 0)
			wgtDct[bone] = tmp
	return(wgtDct)

"""
def __MakeWgtDict(bones, weights, pntNum):
	u'''Skelegonが使っているpntIdx配列(降順)の作成.'''
	arTmp = []	# Skelegonが使っている頂点idxの配列
	arEXT = arTmp.extend

	activeMaps = []	# ボーンが参照している(有効な)vmap. [[vmapName, boneName],[],...]
	amAPD = activeMaps.append

	for (idx, bon) in bones.items():
		arEXT( [bon.pnt0, bon.pnt1] )
		if(bon.weight != None):	# vmap割り当てが有る場合,有効な変形用ボーンとして拾う
			amAPD( (bon.weight, bon.name) )

	boneVertIdxs = sorted(set(arTmp), key= arTmp.index)	# 重複削除,昇順.

	# pntIdxからvmap毎の値配列を用意. idxがSkelegonPntなら拾わない.
	# {vmap:[v,v,...], vmap:[v,v,...],...}
	wgtDct = {}
	arTmpPntsIdxs = range(pntNum)
	for (vmap, bone) in activeMaps:
		wgt = weights[vmap]
		tmp = []
		APD = tmp.append
		for i in arTmpPntsIdxs:		# pntがSkelegonのpntIdxに含まれるか?
			if(i not in boneVertIdxs):
				APD(wgt[i] if(i in wgt) else 0)
		wgtDct[bone] = tmp
	return(wgtDct)
"""
