﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import maya.OpenMaya as om

from ... import FnCalc

#### =======================================================================
__all__ = ("Make",)

#### ========================================================================
def Make(newPolNum, fpaVerts, miaPols, miaOrd, lyr, opts):
	(dagpMesh, moMesh) = __Make_wTransforms(lyr.name, lyr.pntNum, newPolNum, fpaVerts, miaPols, miaOrd)
	if(opts.usePivot):
		__ApplyPivotOffsetToTransform(dagpMesh, opts.vScl, lyr.piv)
	return(dagpMesh, moMesh)

#### --------------------------------
def __Make_wTransforms(name, pntNum, newPolNum, fpaVerts, miaPols, miaOrd):
	FnTr = om.MFnTransform()
	FnTr.create()
	trName = FnTr.setName(name)		# 重複時は既定の名称変化,+1,2...

	FnMe = om.MFnMesh()
	FnMe.create(pntNum, newPolNum, fpaVerts, miaPols, miaOrd, FnTr.object())	# moはparent指定次第
	FnMe.setName("{}Shape".format(trName))

	dagp = om.MDagPath()
	FnMe.getPath(dagp)
	return(dagp, FnMe.object())

#### --------------------------------
def __ApplyPivotOffsetToTransform(dagp, vScl, pivot):
	u'''pivotが原点となるように頂点座標を修正'''
	FnTr = om.MFnTransform(dagp.transform())	# FnTr.setTranslationはkWorldを受けない
	FnTr.setTranslation(om.MVector(*FnCalc.VMul(pivot, vScl)), om.MSpace.kTransform)
