﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

import maya.cmds as mc
import maya.OpenMaya as om

from ... import FnCalc
from .. import Defines, Opts
from . import FnOM1

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Defines", "Opts", "FnCalc", "FnOM1"):
		if(f in loc):
			Fn(eval(f))
	del f, loc

#### ========================================================================
__all__ = ("MakeCurveObject",)

#### ========================================================================
"""
def __MakeCurve_CpStdKnots(cvNum, deg):
	u'''開いたカーブをcpで作る時のknot'''
	k = ([0.0]*(deg-1)) + [float(i) for i in range(cvNum-deg)] + ([float(cvNum-deg)]*deg))
	return(FnOM1.MDoubleArrayFromList(k))
"""

#### ========================================================================
def __Do_HCRV(names, deg, crvs, pnts, vScl, moTrsLst):
	FnCrv= om.MFnNurbsCurve()
	FnDag= om.MFnDagNode()
	mpa  = om.MPointArray()

	for (name, crv) in zip(names, crvs):
		# crv[flag, idx,idx,,,]
		if(crv[0]):	# [0]Flag
			form = FnCrv.kPeriodic
			isUniformParam = True
			crvPnts = crv[2:-1]
			# 前後の1点づつ落とし, 循環状態にしてからepで作る.
			# 始終がコントロール且つ開いた形状は考慮していない.
		else:		# 
			form = FnCrv.kOpen
			isUniformParam = False
			crvPnts = crv[1:]

		mpa.clear()	# 再利用, 前処理
		mpa.setLength(len(crvPnts))

		for (i, id) in enumerate(crvPnts):
			mpa.set(i, *FnCalc.VMul(pnts[id], vScl))

		# om1ではどちらでも周期(円)を作れる. om2ではTrueでのみ可. 注意※開放時は0/1でcpの位置が変わる.
		rational = True
		moTrs = FnCrv.createWithEditPoints(mpa, deg, form, False, rational, isUniformParam)

		FnDag.setObject(moTrs)
		FnDag.setName(name)
		FnCrv.setName("{}Shape".format(name))

		moTrsLst.append(moTrs)

#### ------------------------------------------------------------------------
def __Do_BCRV(names, deg, crvs, pnts, vScl, moTrsLst):
	# crv==[None, 点A,出A, 入B,点B,出B, 入C,点C,出C,,,入N,点N]
	# close時はN==A

	FnDag= om.MFnDagNode()
	for (name, crv) in zip(names, crvs):
#		if(crv[1] == crv[-1]):	# close
#		else:	# open
		# 便宜的だけでも「閉じたベジェ」は作れない？ 作成方法不明。
		# EPで周期円作成→変換でなくnurbsToBezierノード(ep.local→nb.create)
		# この間は閉じている→切断→(閉じているが表示だけ?)→何かの拍子で開く
		p = [ FnCalc.VMul(pnts[id], vScl) for id in crv[1:] ]
		seg = (len(p) // 3) + 1
		k = [ i for i in range(seg) for j in (0,1,2)]
		trsName = mc.curve(bez=True, d=deg, per=False, p= p, k= k)

		moTrs = FnOM1.GetDependNode(FnOM1.MakeSelectionList(trsName), 0)

		FnDag.setObject(moTrs)
		FnDag.setName(name)

		dagp = om.MDagPath()
		FnDag.getPath(dagp)
		dagp.extendToShape()
		FnDag.setObject(dagp)
		FnDag.setName("{}Shape".format(name))
		moTrsLst.append(moTrs)

#### ========================================================================
def MakeCurveObject(lyr, vScl= Opts.OptsObject.defaultVScl, deg= Opts.OptsObject.defaultDeg):
	if(lyr.crvNum == 0):
		return([])

	baseName = lyr.name.replace(" ", "_")	# maya自体の変換もあるけど呪い
	if(lyr.crvNum == 1):
		names = (Defines.NameFMT.CRVNAME.format(NAME= baseName),)
	else:
		FMT = Defines.NameFMT.CRVNAMES.format(NAME= baseName, SPC= u"_" if(baseName[-1].isdigit()) else "")
		names = tuple( FMT.format(CNT= i) for i in range(1, lyr.crvNum+1) )

	moTrsLst = []

	__Do_HCRV(names, deg, lyr.crvs, lyr.pnts, vScl, moTrsLst)

	__Do_BCRV(names[len(lyr.crvs):], deg, lyr.bcrvs, lyr.pnts, vScl, moTrsLst)

	return(moTrsLst)
