﻿# -*- coding: utf-8 -*-

import maya.OpenMaya as om

#### =======================================================================
__all__ = ("ApplyCreaseEdge",)

#### ========================================================================
def ApplyCreaseEdge(moMesh, lyr):
	u'''エッジウェイト適用'''
	if((not lyr.hasEdge) and (not lyr.hasEdgeM)):
		return

	(coef, edgDict) = __DetermineEdgeWeight(lyr)

	MSU = om.MScriptUtil(0)
	msI2= MSU.asInt2Ptr()

	FnMe = om.MFnMesh(moMesh)
	idxs = om.MUintArray()		# edgeIdx
	wgts = om.MDoubleArray()	# wgt
	idxAPD = idxs.append
	wgtAPD = wgts.append
	for i in range(FnMe.numEdges()):
		FnMe.getEdgeVertices(i, msI2)
		p0 = MSU.getInt2ArrayItem(msI2, 0, 0)
		p1 = MSU.getInt2ArrayItem(msI2, 0, 1)
		tpl = (p0,p1)
		if(tpl in edgDict):
			idxAPD(i)
			wgtAPD(edgDict[tpl] * coef)
	FnMe.setCreaseEdges(idxs, wgts)

#### ----------------
def __DetermineEdgeWeight(lyr):
	u'''Modoは0~1(0.2が2に相当), LWは0~1(1が2に相当とする)
	maya(smooth/smoothPrev)での実効値0~2. 2以上は要mrSubdivApprox.'''
	# LWOの最終更新がMODOならVMEDがある.優先.
	if(lyr.hasEdgeM):
		return(10.0, lyr.edgeWgtsM)

	# LWOの最終更新がLWだとVMEDは無い.
	# ModoでVMEDを消して,VMADが残っていた場合もこっちになるので要オプション？
	return(2.0, lyr.edgeWgts)
