﻿# -*- coding: utf-8 -*-

import maya.OpenMaya as om

#### =======================================================================
__all__ = ("MItSelectionList", "NumberOfShapesDirectlyBelow", "NumberOfShapes", "GetChildShapeDagPathList",
			"GetSelectionStrings", "GetFullPathNames", "MakeSelectionList", "GetComponent", "GetDagPath", "GetDependNode",
			"GetActiveSelectionList", "GetFunctionSetList", "GetHiliteList",
			"MPointArrayFromList", "MDoubleArrayFromList", "MIntArrayFromList", "MxToList",
			)

#### =======================================================================
def __GetKeyFromKwd(kwd, sn, ln, default):
	return(kwd.get(sn, kwd.get(ln, default)))

#### =======================================================================
class MItSelectionList(om.MItSelectionList):
	u'''om2的に扱う為のラッパ'''
	def __init__(self, *arg):
		super(MItSelectionList, self).__init__(*arg)

	#### ----------------
	def GetDependNode(self):
		mo = om.MObject()
		self.getDependNode(mo)
		return(mo)

	#### ----------------
	def GetDagPath(self):
		dagp = om.MDagPath()
		self.getDagPath(dagp)
		return(dagp)

	#### ----------------
	def GetComponent(self):
		dagp = om.MDagPath()
		comp = om.MObject()
		self.getDagPath(dagp, comp)
		return(dagp, comp)

	#### ----------------
	def GetStrings(self):
		s = []
		self.getStrings(s)
		return(s)

#### =======================================================================
def NumberOfShapesDirectlyBelow(dagp):
	u'''中間objを除いたshape数'''
	ptr = om.MScriptUtil(0).asUintPtr()
	dagp.numberOfShapesDirectlyBelow(ptr)	# 中間objはカウントされない, dagp==Root(name=='')だと例外
	return(om.MScriptUtil.getUint(ptr))

def NumberOfShapes(dagp):
	u'''中間objを含んだshapeの総数'''
	cnt = 0
	for i in range(dagp.childCount()):
		cnt += int(dagp.child(i).hasFn(om.MFn.kShape))
	return(cnt)

#### =======================================================================
def GetChildShapeDagPathList(dagp, kTypes= (om.MFn.kShape,), **kwd):
	u'''dagpの子shapeDagPathを返す,
Fn(dagp, <kTypes=(om.MFn.kShape,)|(typeName,)>, <ni/noIntermediate=True>)
return: [dagp,dagp,...]'''
	if(not isinstance(kTypes, (list, tuple))):
		kTypes = (kTypes,)
	noItmed = __GetKeyFromKwd(kwd, "ni", "noIntermediate", True)

	res = []
	APD = res.append

	if(isinstance(kTypes[0], int)):
		if(noItmed):
			for i in range(NumberOfShapesDirectlyBelow(dagp)):		# 中間objはカウントされない
				ch = om.MDagPath(dagp)
				ch.extendToShapeDirectlyBelow(i)					# 中間objは対象にならない
				for typ in kTypes:
					if(ch.hasFn(typ)):
						APD(ch)
						break;
		else:
			FnDag = om.MFnDagNode()
			for i in range(dagp.childCount()):
				mo = dagp.child(i)
				for typ in kTypes:
					if(mo.hasFn(typ)):
						FnDag.setObject(mo)
						p = om.MDagPath()
						FnDag.getPath(p)
						APD(p)
						break;
	else:
		typeNameSet = set(kTypes)
		noItmed ^= 1
		FnDag = om.MFnDagNode()
		for i in range(dagp.childCount()):
			FnDag.setObject(dagp.child(i))
			if((not FnDag.isIntermediateObject()) | noItmed):
				if(FnDag.typeName() in typeNameSet):
					p = om.MDagPath()
					FnDag.getPath(p)
					APD(p)
	return(res)

#### =======================================================================
def GetSelectionStrings(sel):
	s = []
	sel.getSelectionStrings(s)
	return(tuple(s))

#### =======================================================================
def GetFullPathNames(sel):
	u'''getSelectionStrings()の戻りパスが不均一な事への対応'''
	res = []
	APD = res.append

	FnDep = om.MFnDependencyNode()
	dagp = om.MDagPath()
	mo = om.MObject()

	iter = om.MItSelectionList(sel)
	while(not iter.isDone()):
		iter.getDependNode(mo)
		if(mo.hasFn(om.MFn.kDagNode)):
			iter.getDagPath(dagp)

			if(iter.hasComponents()):
				bs = dagp.fullPathName()
				ss = []
				iter.getStrings(ss)
				for s in ss:
					APD("{}.{}".format(bs, s.rsplit(".",1)[-1]))
			else:
				APD(dagp.fullPathName())
		else:
			FnDep.setObject(mo)
			APD(FnDep.name())
		iter.next()
	return(res)

#### =======================================================================
def MakeSelectionList(*arg):
	sel = om.MSelectionList()
	for ob in arg:
		if(isinstance(ob, basestring)):
			sel.add(ob)
		elif(isinstance(ob, (tuple, list))):
			for o in ob:
				sel.add(o)
	return(sel)

#### =======================================================================
def GetComponent(sel, idx):
	dagp = om.MDagPath()
	mo = om.MObject()
	sel.getDagPath(idx, dagp, mo)
	return(dagp, mo)

#### =======================================================================
def GetDagPath(sel, idx):
	dagp = om.MDagPath()
	sel.getDagPath(idx, dagp)
	return(dagp)

#### =======================================================================
def GetDependNode(sel, idx):
	mo = om.MObject()
	sel.getDependNode(idx, mo)
	return(mo)

#### =======================================================================
def GetActiveSelectionList(order=True):
	sel = om.MSelectionList()
	om.MGlobal.getActiveSelectionList(sel, order)
	return(sel)

#### =======================================================================
def GetFunctionSetList(mo):
	s = []
	om.MGlobal.getFunctionSetList(mo, s)
	return(s)

#### =======================================================================
def GetHiliteList():
	sel = om.MSelectionList()
	om.MGlobal.getHiliteList(sel)
	return(sel)

#### =======================================================================
#### =======================================================================
def MPointArrayFromList(vecs):
	mpa = om.MPointArray(len(vecs))
	for i in range(len(vecs)):
		mpa.set(i, *vecs[i])	# [3/4]可変
	return(mpa)

#### =======================================================================
def MDoubleArrayFromList(ar):
	# MScrUtilにDblArrayFromListは無い
	mda = om.MDoubleArray()
	for d in ar:
		mda.append(d)
	return(mda)

#### =======================================================================
def MFloatArrayFromList(ar):
	mfa = om.MFloatArray()
	om.MScriptUtil.createFloatArrayFromList(ar, mfa)
	return(mfa)

#### =======================================================================
def MIntArrayFromList(ar):
	mia = om.MIntArray()
	om.MScriptUtil.createIntArrayFromList(ar, mia)
	return(mia)

#### =======================================================================
def MxToList(mx):
	# MMatrixからlist[16]. msu.getDbl([i])では[4]を超えた時不正になる.
	msu = om.MScriptUtil.getDoubleArrayItem
	return([ msu(mx[0], i) for i in range(16) ])
