﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

import maya.OpenMaya as om
import maya.cmds as mc

from .. import Shaders, Defines
from . import FnOM1

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Defines", "Shaders", "FnOM1",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

#### ========================================================================
__all__ = ("Make",)

#### ========================================================================
def __GetInitialMaterialOutPlug():
	u'''初期lambertのocプラグを取るだけ'''
	Fn = om.MFnDependencyNode(FnOM1.GetDependNode(FnOM1.MakeSelectionList("lambert1"), 0))
	return(Fn.findPlug("oc", False))

#### ----------------
def __AddMember_ToExsitingSG(sgName, selLst):
	u'''既存のSGにセット'''
	Fn = om.MFnSet(FnOM1.GetDependNode(FnOM1.MakeSelectionList(sgName), 0))
	Fn.addMembers(selLst)

#### ----------------
def __DetermineTargetList(dagpMesh, surfNum, polIdxs):
	u'''コンポーネント選択リスト作成'''
	FnMe = om.MFnMesh(dagpMesh)
	selLst = om.MSelectionList()

	# surfが1つ、pol数一致ならcompでなくdagへの適用
	if((surfNum == 1) and (len(polIdxs) == FnMe.numPolygons())):
		selLst.add(dagpMesh)
	else:
		Fn = om.MFnSingleIndexedComponent()
		comp = Fn.create(om.MFn.kMeshPolygonComponent)
		Fn.addElements(FnOM1.MIntArrayFromList(polIdxs))
		selLst.add(dagpMesh, comp)		# コンポーネント選択リスト作成
	return(selLst)

#### ----------------
def Make(tags, surfTags, surfs, buildedPolIdxs, dagpMesh, opts):
	u'''return: mtlDct{ surfIdx:[tagName, mtName, sgName] }'''
	plgInitialOut = __GetInitialMaterialOutPlug()		# 初期マテリアル outColor

	shaderType = Defines.gShaderTypes[opts.mtlType]	# shaderNode type

	FnSet= om.MFnSet()
	MDGM = om.MDGModifier()

	surfNum = len(surfTags.keys())

	mtlDct = {}
	for idx in surfTags.keys():
		mtName = tagName = tags[idx]

		if(mtName.rfind(Defines.SFixNM.MTL) == -1):	# _MTが無い(_MT6等,中間出現を許容)
			mtName = "{}{}".format(mtName, Defines.SFixNM.MTL)
		sgName = mtName.replace(Defines.SFixNM.MTL, Defines.SFixNM.SG)	# 改名後に合わせてreplacce

		# コンポーネント選択リスト作成
		polIdxs = tuple( buildedPolIdxs[polIdx] for polIdx in surfTags[idx] if(buildedPolIdxs[polIdx] is not None) )
		selLst = __DetermineTargetList(dagpMesh, surfNum, polIdxs)

		if(opts.shareSG and mc.objExists(sgName)):	# 既存SGが有れば利用して次へ
			__AddMember_ToExsitingSG(sgName, selLst)
			continue

		# SG作成/割当. 初期マテリアルが繋がってる. result強制表示2012~20, 消せない.
		FnSet.create(selLst, FnSet.kRenderableOnly)

		# lxoではptagと紐付くsurfが無い(値取得していない)物もある事への対応.
		if(tagName not in surfs):
			FnSet.setName(sgName)
			continue

		# Mtl作成
		tmpShaderName = Shaders.MakeNode(shaderType, n= Defines.SHADER_TEMP_NAME, s=True)
		FnShd = om.MFnDependencyNode(FnOM1.GetDependNode(FnOM1.MakeSelectionList(tmpShaderName), 0))

		plgInp = FnSet.findPlug("ss", False)				# surfaceShader
		MDGM.disconnect(plgInitialOut, plgInp)				# 初期マテリアル切断
		MDGM.connect(FnShd.findPlug("oc", False), plgInp)
		MDGM.doIt()											# MDGModifierはdoIt()で処理確定

		mtName = FnShd.setName(mtName)		# 重複改名があるので先行してsetName
		sgName = FnSet.setName(sgName)
		mtlDct[idx] = [tagName, mtName, sgName]
	return(mtlDct)
