﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import maya.api.OpenMaya as om2

from ... import FnCalc

#### =======================================================================
__all__ = ("Make",)

#### ========================================================================
def Make(fpaVerts, miaPols, miaOrd, lyr, opts):
	(dagpMesh, moMesh) = __Make_wTransforms(lyr.name, fpaVerts, miaPols, miaOrd)
	if(opts.usePivot):
		__ApplyPivotOffsetToTransform(dagpMesh, opts.vScl, lyr.piv)
	return(dagpMesh, moMesh)

#### --------------------------------
def __Make_wTransforms(name, fpaVerts, miaPols, miaOrd):
	FnTr = om2.MFnTransform()
	FnTr.create()
	FnMe = om2.MFnMesh()
	FnMe.create(fpaVerts, miaPols, miaOrd, parent=FnTr.object())	# createの戻りmoは不定？
	trName = FnTr.setName(name)										# 重複時は既定の名称変化,+1,2...
	FnMe.setName("{}Shape".format(trName))
	return(FnMe.getPath(), FnMe.object())

#### --------------------------------
def __ApplyPivotOffsetToTransform(dagp, vScl, pivot):
	'''pivotが原点となるように頂点座標を修正'''
	FnTr = om2.MFnTransform(dagp.transform())	# FnTr.setTranslationはkWorldを受けない
	FnTr.setTranslation(om2.MVector(*FnCalc.VMul(pivot, vScl)), om2.MSpace.kTransform)
