﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

import maya.cmds as mc
import maya.OpenMaya as om
import maya.OpenMayaAnim as oa

from ... import FnCalc
from ..om1 import FnOM1
from .. import Defines
from . import Conv2To1

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Defines", "FnCalc", "FnOM1", "Conv2To1",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

# メモ: 2022になってもoa2.MFnBlendShapeDefomerは無い.

#### ========================================================================
__all__ = ("MakeNode",)

#### ------------------------------------------------------------------------
def __Get_MObject_ByName(names):
	mos = []
	APD = mos.append
	iter = FnOM1.MItSelectionList(FnOM1.MakeSelectionList(names))
	while(not iter.isDone()):
		APD(iter.GetDependNode())
		iter.next()
	return(mos)

#### ------------------------------------------------------------------------
def MakeNode(dagpMesh2, morfDict):	# パラレル
	u''' '''
	# 頂点除外の為のset取得はMFnGeometryFilterから出来るはず.
	# 実用を考えるとBSノード自体が並行/平行でないと, NodeA=原型/移動量0, NodeB:変形有り
	# の様な時に破綻する. ノード上のターゲット間をパラレルにしても恩恵無いかも.
	# om.MFnでorder指定でのノード設置は出来ない. cmdsでやるしかない.

	# om2.dagPath からom1.dagPathへ
	dagpMesh = Conv2To1.DagPath(dagpMesh2)

	FnBl = oa.MFnBlendShapeDeformer()
	FnMe = om.MFnMesh(dagpMesh)
	moMesh = FnMe.object()

	# meshの親transform
	tmpDgp = om.MDagPath(dagpMesh)
	tmpDgp.pop()
	trsName = tmpDgp.partialPathName()

	for (grpName, dct) in morfDict.items():
		bldName = trsName + grpName + Defines.SFixNM.BLSHAPE
		if(mc.objExists(bldName)):		# blendShapeが既存		※※※※※※※
			FnBl.setObject(__Get_MObject_ByName([bldName])[0])
			idxOffset = mc.getAttr("{}.w".format(bldName), s=True)
		else:
			FnBl.create(moMesh)			# ノード作成
			FnBl.setName(bldName)		# BlShapeにグループ名を付ける.
			idxOffset = 0

		for (i, (morfName, mo2)) in enumerate(sorted(dct.items())):
			# om2.mo からom1.moへ
			mo = Conv2To1.MObject(mo2)

			idx = i + idxOffset
			FnBl.addTarget(moMesh, idx, mo, 1.0)			# blShapeNodeは維持される
			# .weight[]のエリアス名が表示される. 初期のエリアス名はshape名. [N]が必要.
			FnBl.setAlias(morfName, "weight[{}]".format(idx), FnBl.findPlug("weight", False))

#### --------
"""
def __XXXBlendShape_Set(dagpMesh, morfDict):	# シーケンスのテスト
	FnBl = oa.MFnBlendShapeDeformer()
	mpElms = om.MIntArray()
	FnMe.setObject(dagpMesh)
	oMe = FnMe.object()

	tmp = om.MDagPath(dagpMesh)
	tmp.pop()
	trsName = tmp.partialPathName()

	for (grpName, dct) in morfDict.items():
		oBld = FnBl.create(oMe)

		num = len(dic.items())
		dif = (1000/num) * 0.001	# 少数3桁以上の精度を持つとNG
		wgt = 1.0					# DecrementでないとNG. Incrementだと3桁が維持出来ない？
		for (i, (morfName, mo)) in enumerate(sorted(dct.items())):
			FnBl.addTarget(oMe, 0, mo, wgt)				# blShapeNode(FnBlのmo)は維持される
			wgt -= dif									# ※wgt1からのDecrement
			# .weight[]のエリアス名が表示される.初期のエリアス名はshape名.
			mp = FnBl.findPlug("weight")				# 配列のルートを取る
			mp.getExistingArrayAttributeIndices(mpElms)	# Int配列[idx,idx,...]
			for c in mpElms:							# 有効な接続が[idx,...]に返ると期待して
				mpTgt = mp.elementByLogicalIndex(c)		# c番目のMPlug
				tgtName = mpTgt.name()					# Node.Attrなので分離
				attr = tgtName.split(".",1)				# 初期値と比較
				if(attr[1] == morfName + Defines.SFixNM.MF_SHAPE):
					FnBl.setAlias(morfName, "weight[{}]".format(i), mp)	# [N]が必要.
					break
		FnBl.setName(trsName + grpName + Defines.SFixNM.BLSHAPE)			# BlShape名にグループ名を付ける.
"""
