﻿# -*- coding: utf-8 -*-

import maya.api.OpenMaya as om2

#### =======================================================================
__all__ = ("ApplyCreaseEdge",)

#### ========================================================================
def ApplyCreaseEdge(moMesh, lyr):
	u'''エッジウェイト適用'''
	if((not lyr.hasEdge) and (not lyr.hasEdgeM)):
		return

	(coef, edgDict) = __DetermineEdgeWeight(lyr)

	FnMe = om2.MFnMesh(moMesh)
	idxs = om2.MUintArray()		# edgeIdx
	wgts = om2.MDoubleArray()	# wgt
	idxAPD = idxs.append
	wgtAPD = wgts.append
	for i in range(FnMe.numEdges):
		tpl = FnMe.getEdgeVertices(i)
		if(tpl in edgDict):
			idxAPD(i)
			wgtAPD(edgDict[tpl] * coef)
	FnMe.setCreaseEdges(idxs, wgts)

#### ----------------
def __DetermineEdgeWeight(lyr):
	u'''Modoは0~1(0.2が2に相当), LWは0~1(1が2に相当とする)
	maya(smooth/smoothPrev)での実効値0~2. 2以上は要mrSubdivApprox.'''
	# LWOの最終更新がMODOならVMEDがある.優先.
	if(lyr.hasEdgeM):
		return(10.0, lyr.edgeWgtsM)

	# LWOの最終更新がLWだとVMEDは無い.
	# ModoでVMEDを消して,VMADが残っていた場合もこっちになるので要オプション？
	return(2.0, lyr.edgeWgts)
