﻿# -*- coding: utf-8 -*-

import maya.api.OpenMaya as om2
import maya.OpenMaya as om

#### =======================================================================
__all__ = ("NumberOfShapesDirectlyBelow", "NumberOfShapes", "GetChildShapeDagPathList",
			"GetFullPathNames", "MakeSelectionList",
			"GetFunctionSetList", "GetHiliteList",
			"MPointArrayFromList", "MDoubleArrayFromList", "MIntArrayFromList", "MxToList",
			)

#### =======================================================================
def __GetKeyFromKwd(kwd, sn, ln, default):
	return(kwd.get(sn, kwd.get(ln, default)))

#### =======================================================================
def NumberOfShapesDirectlyBelow(dagp):
	u'''中間objを除いたshape数'''
	return(dagp.numberOfShapesDirectlyBelow())	# 中間objはカウントされない, dagp==Root(name=='')だと例外

def NumberOfShapes(dagp):
	u'''中間objを含んだshapeの総数'''
	cnt = 0
	for i in range(dagp.childCount()):
		cnt += int(dagp.child(i).hasFn(om2.MFn.kShape))
	return(cnt)

#### =======================================================================
"""
class MItSelectionList(om2.MItSelectionList):
	u''' '''
	def __init__(self, *arg):
		super(MItSelectionList, self).__init__(*arg)
	#### ----------------
	def GetDependNode(self):
		return(self.getDependNode())
	#### ----------------
	def GetDagPath(self):
		return(self.getDagPath())
	#### ----------------
	def GetComponent(self):
		return(self.getetComponent())
	#### ----------------
	def GetStrings(self):
		return(self.getStrings())
"""

#### =======================================================================
def GetChildShapeDagPathList(dagp, kTypes= (om2.MFn.kShape,), **kwd):
	u'''dagpの子shapeDagPathを返す, 
Fn(dagp, <kTypes=(om2.MFn.kShape,)|(typeName,)>, <ni/noIntermediate=True>)
return: [dagp,dagp,...]'''
	if(not isinstance(kTypes, (list, tuple))):
		kTypes = (kTypes,)
	noItmed = __GetKeyFromKwd(kwd, "ni", "noIntermediate", True)

	res = []
	APD = res.append

	if(isinstance(kTypes[0], int)):
		if(noItmed):
			for i in range(dagp.numberOfShapesDirectlyBelow()):		# 中間objはカウントされない
				ch = om2.MDagPath(dagp)
				ch.extendToShape(i)									# 中間objは対象にならない
				for typ in kTypes:
					if(ch.hasFn(typ)):
						APD(ch)
						break
		else:
			FnDag = om2.MFnDagNode()
			for i in range(dagp.childCount()):
				mo = dagp.child(i)
				for typ in kTypes:
					if(mo.hasFn(typ)):
						FnDag.setObject(mo)
						APD(FnDag.getPath())
						break
	else:
		typeNameSet = set(kTypes)
		noItmed ^= 1
		FnDag = om2.MFnDagNode()
		for i in range(dagp.childCount()):
			FnDag.setObject(dagp.child(i))
			if((not FnDag.isIntermediateObject) | noItmed):
				if(FnDag.typeName in typeNameSet):
					APD(FnDag.getPath())
	return(res)

#### =======================================================================
# def GetSelectionStrings(sel):
#	return(sel.getSelectionStrings())

#### =======================================================================
def GetFullPathNames(sel):
	u'''getSelectionStrings()の戻りパスが不均一な事への対応
selList内のDAGはdagpでないとiter.fn()がこける'''
	res = []
	APD = res.append

	FnDep = om2.MFnDependencyNode()

	iter = om2.MItSelectionList(sel)
	while(not iter.isDone()):
		mo = iter.getDependNode()
		if(mo.hasFn(om2.MFn.kDagNode)):
			if(iter.hasComponents()):
				bs = iter.getDagPath().fullPathName()
				for s in iter.getStrings():
					APD("{}.{}".format(bs, s.rsplit(".",1)[-1]))
			else:
				APD(iter.getDagPath().fullPathName())
		else:
			FnDep.setObject(mo)
			APD(FnDep.name())
		iter.next()
	return(res)

#### =======================================================================
def MakeSelectionList(*arg):
	sel = om2.MSelectionList()
	for ob in arg:
		if(isinstance(ob, basestring)):
			sel.add(ob)
		elif(isinstance(ob, (tuple, list))):
			for o in ob:
				sel.add(o)
	return(sel)

#### =======================================================================
# def GetComponent(sel, idx):
#	return(sel.getComponent(idx))

#### =======================================================================
# def GetDagPath(sel, idx):
#	return(sel.getDagPath(idx))

#### =======================================================================
# def GetDependNode(sel, idx):
#	return(sel.getDependNode(idx))

#### =======================================================================
#def GetActiveSelectionList(order=True):
#	return(om2.MGlobal.getActiveSelectionList())

#### =======================================================================
def GetFunctionSetList(mo):
	return(om2.MGlobal.getFunctionSetList(mo))

#### =======================================================================
def GetHiliteList():
	u'''2018になってもom2に無い?'''
	sel = om.MSelectionList()
	om.MGlobal.getHiliteList(sel)
	tmp = []
	sel.getSelectionStrings(tmp)
	om2Sel = om2.MSelectionList()
	for s in tmp:
		om2Sel.add(s)
	return(om2Sel)

#### =======================================================================
#### =======================================================================
def MPointArrayFromList(vecs):
	mpa = om2.MPointArray(len(vecs))
	for i in range(len(vecs)):
		mpa.set(i, *vecs[i])	# [3/4]可変
	return(mpa)

#### =======================================================================
def MDoubleArrayFromList(ar):
	return(om2.MDoubleArray(ar))

#### =======================================================================
def MFloatArrayFromList(ar):
	return(om2.MFloatArray(ar))

#### =======================================================================
def MIntArrayFromList(ar):
	return(om2.MIntArray(ar))

#### =======================================================================
def MxToList(mx):
	return(list(mx))
