﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys

import maya.api.OpenMaya as om2
import maya.cmds as mc

from ... import FnCalc
from .. import Defines
from . import BaseMesh, Dags

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Defines", "FnCalc", "BaseMesh", "Dags",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

#### ========================================================================
__all__ = ("Make",)

#### ========================================================================
def Make(newPolNum, buildedPolIdxs, fpaVerts, miaPols, miaOrd, lyr, opts):
	(hasRM, hasAM) = (len(lyr.relMorphs), len(lyr.absMorphs))
	(relMorfDict, absMorfDict) = ({}, {})
	if(opts.loadRM or opts.loadAM):	# モーフジオメトリ作成ならオブジェクトと対になるグループを作る
		if(hasRM + hasAM):
			FnTrMorfGrp = __Mesh_Group(lyr.name)
		if(opts.loadRM and hasRM):
			relMorfDict = __Mesh(newPolNum, buildedPolIdxs, fpaVerts, miaPols, miaOrd, FnTrMorfGrp, lyr.relMorphs, Defines.MorfT.REL, lyr, opts)
		if(opts.loadAM and hasAM):
			absMorfDict = __Mesh(newPolNum, buildedPolIdxs, fpaVerts, miaPols, miaOrd, FnTrMorfGrp, lyr.absMorphs, Defines.MorfT.ABS, lyr, opts)
	return(hasRM, hasAM, relMorfDict, absMorfDict)

#### ------------------------------------------------------------------------
def __Mesh_ModifyRelFPA(fpaVt, idValDct, vScl):
	for (i, v) in idValDct.items():
		fpaVt[i] += om2.MFloatVector(FnCalc.VMul(v, vScl))
#		fpaVt[i].x += r[0]
#		fpaVt[i].y += r[1]
#		fpaVt[i].z += r[2]

#### ----------------
def __Mesh_ModifyAbsPivFPA(fpaVt, idValDct, vScl, piv):
	offs = FnCalc.VMul(piv, vScl)
	for (i, v) in idValDct.items():
		fpaVt[i] = om2.MFloatPoint(FnCalc.VMulSub(v, vScl, offs))
#		(fpaVt[i].x, fpaVt[i].y, fpaVt[i].z) = FnCalc.VMulSub(v, vScl, offs)

#### ----------------
def __Mesh_ModifyAbsFPA(fpaVt, idValDct, vScl):
	for (i, v) in idValDct.items():
		fpaVt[i] = om2.FloatPoint(FnCalc.VMul(v, vScl))
#		(fpaVt[i].x, fpaVt[i].y, fpaVt[i].z) = FnCalc.VMul(v, vScl)

#### ------------------------------------------------------------------------
def __Mesh_Group(name):
	FnTr = om2.MFnTransform()
	FnTr.create()
	FnTr.setName("{}{}".format(name, Defines.SFixNM.MF_GRP))	# 戻り名不要
	FnTr.findPlug("v", False).setBool(False)
	return(FnTr)

#### ------------------------------------------------------------------------
def __Mesh_MorfsNames_AddDict(baseName, moMesh, morfDict):
	buf = baseName.split(".", 1)	# [grp].[hoge.hoe]
	if(len(buf) == 1):				# 最大1分割,[0~1],len=1~2
		(grpName, morfName) = (Defines.MORF_NO_GRP, buf[0])
	else:
		(grpName, morfName) = (buf[0], buf[1])

	if(grpName in morfDict):	# grpの追加/新規, {grpName: {morf:MObj},{},...}
		morfDict[grpName].update({ morfName: moMesh })
	else:
		morfDict[grpName] = dict({ morfName: moMesh })

	return(grpName, morfName)

#### ------------------------------------------------------------------------
def __Mesh_AddGroupDict(grpName, FnTr, moTrsDict):
	if(grpName in moTrsDict):	# grpの追加/新規, {grpName: [MObj,MObj,...]}
		moTrsDict[grpName].append(FnTr.object())
	else:
		moTrsDict[grpName] = [FnTr.object()]

#### ------------------------------------------------------------------------
def __Mesh(newPolNum, buildedPolIdxs, fpaVerts, miaPols, miaOrd, FnTrMorfGrp, morphs, isRelMorph, lyr, opts):
	FnTr = om2.MFnTransform()	# 汎用
	FnMe = om2.MFnMesh()		# 汎用
	dagp = om2.MDagPath()		# 汎用

	(moTrsDict, morfDict) = ({}, {})
	for (baseName, idValDct) in sorted(morphs.items()):	# {id:pos, id:pos,...}
		# メッシュ作成, 未確定Meshへの変形と削除は不正処理される.
		# 元型create→copy→get/setPoints→deleteVertex == NG
		# 元型create→get/setPoints→deleteVertex == NG
		# PointArrayの複製を加工してcreate→deleteVertexする形にする
		fpaVt = om2.MFloatPointArray(fpaVerts)
		if(isRelMorph):		# 相対
			__Mesh_ModifyRelFPA(fpaVt, idValDct, opts.vScl)
		else:
			if(opts.usePivot):
				__Mesh_ModifyAbsPivFPA(fpaVt, idValDct, opts.vScl, lyr.piv)
			else:
				__Mesh_ModifyAbsFPA(fpaVt, idValDct, opts.vScl)
		(dagpMesh, moMesh) = BaseMesh.Make(fpaVt, miaPols, miaOrd, lyr, opts)

		# grpの追加/新規, {grpName: {morf:MObj},{},...}
		(grpName, morfName) = __Mesh_MorfsNames_AddDict(baseName, moMesh, morfDict)

		FnMe.setObject(moMesh)
		FnMe.setName(morfName + Defines.SFixNM.MF_SHAPE)	# 改名, 戻り名不要

		FnTr.setObject(dagpMesh.transform())				# 親Trs
		FnTr.setName(morfName + Defines.SFixNM.MF_TRS)		# 親Trsを改名, 戻り名不要

		# grpの追加/新規, {grpName: [MObj,MObj,...]}
		__Mesh_AddGroupDict(grpName, FnTr, moTrsDict)

		# mesh/Skelegon/curve混在対策, 未確定Meshの未使用頂点は一つ消せば全て消える
		if(lyr.bonNum):
#-			FnMe.deleteVertex(lyr.bones[0][BoneT.PNTID][0])	# 1本目の第1頂点のidx.
			FnMe.deleteVertex(lyr.bones[0].pnt0)	# 1本目の第1頂点のidx.
		elif(lyr.crvNum):
			FnMe.deleteVertex(lyr.crvs[0][1])	# 1本目の第1頂点のidx.

		FnMe.setObject(dagpMesh)	# dagpだとFix, Fixすると未使用頂点は消せない

	moGrpList = []		# morphGroup毎とlayer毎でグループ化
	APD = moGrpList.append
	for (grpName, moTrs) in moTrsDict.items():	# {grpName: [MObj,MObj,...]}
		dagp = Dags.Grouping(Defines.GrpVisT.SHOW, moTrs, "{}_grp".format(grpName))
		APD(dagp.node())

	tmpName = "{}Morph_grp".format("Rel" if(isRelMorph) else "Abs")
	dagp = Dags.Grouping(Defines.GrpVisT.SHOW, moGrpList, tmpName)	# Rel/Absグループ名の保証
	mo = dagp.node()
	FnTrMorfGrp.addChild(mo)
	FnTr.setObject(mo)
	FnTr.setName(tmpName)

	return(morfDict)
