﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys, re

import maya.api.OpenMaya as om2
import maya.cmds as mc

from .. import Defines

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("Defines",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

#### ========================================================================
__all__ = ("ApplyVColors",)

#### ========================================================================
def ApplyVColors(moMesh, buildedPolIdxs, pols, RGB, useMiVC):
	'''頂点カラーの適用.
ApplyVColors(moMesh, buildedPolIdxs, pols, RGB, useMiVC)
moMesh: メッシュのMObject
buildedPolIdxs: 点ポリ等を省いた実際に作成されたポリゴンのindex配列
pols: ポリゴンのindex配列
RGB: 
useMiVC: mentalrayの頂点カラーノードを設置する'''
	FnMe = om2.MFnMesh(moMesh)

	if(useMiVC): rgbNames = __SetVColorSet_mrVCol(FnMe, RGB)
	else:		 rgbNames = __SetVColorSet(FnMe, RGB)
	if(rgbNames == None):
		return(None)

	for name in RGB.keys():
		FnMe.setCurrentColorSetName(rgbNames[name])		# set切り替え, 旧→真map名
		if(RGB[name].vmap and RGB[name].vmad):
			__ApplyVColors_Mixed(FnMe, RGB[name], buildedPolIdxs, pols)
		else:
			if(RGB[name].vmap):
				__ApplyVColors_VMAP(FnMe, RGB[name].vmap)
			if(RGB[name].vmad):
				__ApplyVColors_VMAD(FnMe, RGB[name].vmad, buildedPolIdxs)

	return(rgbNames)

#### ------------------------------------------------------------------------
def __SetVColorSet_mrVCol(FnMe, rgbs):
	'''meshへの頂点カラー適用とmentalrayVertexColorの作成/接続'''
	colNames = rgbs.keys()	# .[name][vmap/vmad]...
	if(len(colNames) == 0):
		return(None)

	meName = FnMe.fullPathName()					# ※パス無しだと下記getAttr(s)時に余計な物も拾ってくる, miなら平気？
	meNode = "{}.clst[{{}}].clsn".format(meName)	# .colorSet[].colorName

	rex = re.compile("[\[\]= +-/~^|?/*]")

	mapNames = {}	# {旧名:真名}
	for (i, name) in enumerate(colNames):
		newName = FnMe.createColorSet(rex.sub("_", name), FnMe.kRGBA)
		FnMe.setIsColorClamped(newName, True)	# clampさせる
		mapNames.update({ name: newName })

		# RGB/RGBAで二回呼ばれるので作成された位置を取り直す
		curColrSetNames = []
		FnMe.getColorSetNames(curColrSetNames)
		clstIdx = curColrSetNames.index(newName)

		# ColSet名のmrVColを重複作成はしない
		mrvcName = Defines.NameFMT.VCOL.format(NAME= newName)
		if(mc.objExists(mrvcName)):
			__CleaningVColSlot(mrvcName)		# 既存の空きslot削除
		else:
			mrvcName = mc.shadingNode(Defines.NdType.VCOL, at=True, n= mrvcName)

		dst = "{}.s[{}]".format(mrvcName, __GetVColSlotIdx(mrvcName))	# .cpvSets[]"
		mc.connectAttr(meNode.format(clstIdx), dst, f=True)
	return(mapNames)

#### ------------------------------------------------------------------------
def __GetVColSlotIdx(mrvcName):
	'''未使用Slotが無い事を前提にして、若い空きslotを返す'''
	nm = "{}.s".format(mrvcName)		# cpvSets
	sz = mc.getAttr(nm, s=True)			# 配列最大size
	if(not sz):
		return(0)

	mi  = mc.getAttr(nm, mi=True) or []			# 存在するslotIdx, [n,n,...]
	dif = list(set(range(sz)).difference(mi))	# [0,,sizeMax-1] と slotIdxの差
	return(sz if(not dif) else dif[0])

#### ------------------------------------------------------------------------
def __CleaningVColSlot(mrvcName):
	'''mentalrayVertecColor.s[]の未使用slotを除去して空きslotにする'''
	ar = mc.getAttr("{}.s".format(mrvcName), mi=True) or []	# cpvSets
	for i in reversed(ar):							# 詰め対策で末尾から処理
		cur = "{}.s[{}]".format(mrvcName, i)		# mrVC.cpvSets[]
		if(not mc.connectionInfo(cur, id=True)):
			mc.removeMultiInstance(cur, b=True)		# remMulInstで勝手に詰まる

#### ------------------------------------------------------------------------
def __SetVColorSet(FnMe, rgbs):
	'''meshへの頂点カラーのみ適用'''
	colNames = rgbs.keys()	# .[name][vmap/vmad]...
	if(len(colNames) == 0):
		return(None)

	rex = re.compile("[\[\]= +-/~^|?/*]")

	mapNames = {}	# {旧名:真名}
	for name in colNames:
		newName = FnMe.createColorSet(rex.sub("_", name), FnMe.kRGBA)
		FnMe.setIsColorClamped(newName, True)	# clampさせる
		mapNames.update({ name: newName })
	return(mapNames)

#### ------------------------------------------------------------------------
def __ApplyVColors_Mixed(FnMe, rgb, buildedPolIdxs, pols):
	arPols = []
	arPnts = []
	polAPD = arPols.append
	pntAPD = arPnts.append
	mcaCol = om2.MColorArray()
	mcaAPD = mcaCol.append
	for (polIdx, pntIdxs) in enumerate(pols):
		polIdx = buildedPolIdxs[polIdx]
		if(polIdx is not None):
			if(polIdx in rgb.vmad):
				for pntIdx in pntIdxs:
					if(pntIdx in rgb.vmad[polIdx]):
						polAPD(polIdx)
						pntAPD(pntIdx)
						mcaAPD(rgb.vmad[polIdx][pntIdx])
					else:
						if(pntIdx in rgb.vmap):
							polAPD(polIdx)
							pntAPD(pntIdx)
							mcaAPD(rgb.vmap[pntIdx])
			else:
				for pntIdx in pntIdxs:
					if(pntIdx in rgb.vmap):
						polAPD(polIdx)
						pntAPD(pntIdx)
						mcaAPD(rgb.vmap[pntIdx])

	FnMe.setFaceVertexColors(mcaCol, om2.MIntArray(arPols), om2.MIntArray(arPnts))

#### ------------------------------------------------------------------------
def __ApplyVColors_VMAD(FnMe, rgbVmad, buildedPolIdxs):
	for (polIdx, dct) in rgbVmad.items():
		polIdx = buildedPolIdxs[polIdx]
		if(polIdx is not None):
			for (pntIdx, col) in dct.items():
				FnMe.setFaceVertexColor(om2.MColor(*col), polIdx, pntIdx)	# col[0~3], pntidx in mesh

#### ------------------------------------------------------------------------
def __ApplyVColors_VMAP(FnMe, rgbVmap):
	# vmap{pntIdx:(rgba)},{},...
	miaPnts = om2.MIntArray(list(rgbVmap.keys()))
	mcaCol = om2.MColorArray(list(rgbVmap.values()))
	FnMe.setVertexColors(mcaCol, miaPnts)
