﻿# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import print_function

import sys, chunk

from . import LWO_ParseHeader, LWO_Object, Vis_Flags, DebugPrints, FnRead, Vis_Flags

if(sys.modules["Load_LWO"].DO_RELOAD):
	if(sys.version_info[0] >= 3):
		import importlib
		Fn = importlib.reload
	else:
		Fn = reload
	loc = set(locals().keys())
	for f in ("LWO_ParseHeader", "LWO_Object", "Vis_Flags", "DebugPrints", "FnRead",):
		if(f in loc):
			Fn(eval(f))
	del f, loc

from .LWO_ParseHeader import *

#### =======================================================================
__all__ = ("ParseLWO",)

#### ========================================================================
class ParseLWO(FnRead.FnReadObject, DebugPrints.DebugPrint):
	__slots__ = FnRead.FnReadObject.__slots__ + DebugPrints.DebugPrint.__slots__ + (
		"VisFlags", "lwo",
	)

	fileType = "LWO2"

	sdsPntWgtName  = "base"
	sdsEdgeWgtName = "Edge Weight"

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		super(ParseLWO, self).__init__(*arg, **kwd)

		self.VisFlags = Vis_Flags.VisFlags(**kwd)

		self.lwo = None

		fn = kwd.get("fn", kwd.get("fileName", None))
		if(fn):
			self.Read(fn)
		else:
			self.lwo = LWO_Object.LwoObject()

	#### ========================================================================
	"""
	def PreScan(self, fileName, **kwd):
		u'''レイヤ簡易走査'''
		(fp, errMsg) = self.__Check_FileState(fileName, "LWO2")
		if(errMsg):
			print(errMsg),
			return(None)

		print(u"走査開始: {}".format(fileName))

		layers = self.ScanLayer(fp)	# 状態表示有り
		fp.close()

		print(u"-------: {} 個のレイヤ".format(len(layers)))

		if(self.VisFlags.VERB):
			print(u"走査完了."),

		return(layers)
	"""
	#### ========================================================================
	def Read(self, fileName, **kwd):
		u'''ファイルロード. インターフェースの共通化'''

		self._Do_Read(fileName)

		return(self.lwo)

	#### ========================================================================
	def ScanLayer(self, fp, *arg):
		u'''簡易ファイル走査'''
		layers = []

		while(True):
			try:
				curChunk = chunk.Chunk(fp)	# FORM:size:LWO2の子階層
			except EOFError:
				break

			size = curChunk.chunksize

			chunkId = self.FnUnpckLWID(curChunk.chunkname)[0]

			if(chunkId == ChunkIdT.LAYR):
				self.Read_Layr(curChunk.read(size), size, layers)

			elif(chunkId == ChunkIdT.VMAP):
				(lwid, size) = self.Read_LWID(curChunk)

				if(  lwid == VMapIdT.MORF):	# Rel Morph
					self.Scan_Morph_VMap(curChunk.read(size), layers[-1].relMorphs)
				elif(lwid == VMapIdT.SPOT):	# Abs Morph
					self.Scan_Morph_VMap(curChunk.read(size), layers[-1].absMorphs)
				elif(lwid == VMapIdT.RGB_):
					self.Scan_RGB_VMap(curChunk.read(size), layers[-1])
				elif(lwid == VMapIdT.RGBA):
					self.Scan_RGBA_VMap(curChunk.read(size), layers[-1])
				else:
					curChunk.skip()

			elif(chunkId == ChunkIdT.VMAD):
				(lwid, size) = self.Read_LWID(curChunk)

				if(  lwid == VMapIdT.RGB_):
					self.Scan_RGB_VMad(curChunk.read(size), layers[-1])
				elif(lwid == VMapIdT.RGBA):
					self.Scan_RGBA_VMad(curChunk.read(size), layers[-1])
				elif(lwid == VMapIdT.WGHT):
					self.Scan_Weight_VMad(curChunk.read(size), layers[-1])
				elif(lwid == VMapIdT.NORM):
					self.Scan_NORM(curChunk.read(size), layers[-1])
				else:
					curChunk.skip()

			elif(chunkId == ChunkIdT.VMED):
				(lwid, size) = self.Read_LWID(curChunk)

				if(lwid == VMedIdT.SUBV):
					self.Scan_SUBV_VMed(curChunk.read(size), layers[-1])
				else:
					curChunk.skip()

			elif(chunkId == ChunkIdT.POLS):
				(lwid, size) = self.Read_LWID(curChunk)

				if(  lwid == PolyIdT.CURV):
					self.Scan_Crvs(curChunk.read(size), layers[-1])
#x					isExclude = True
				elif(lwid == PolyIdT.BONE):
					self.Scan_Bones(curChunk.read(size), layers[-1])
#x					isExclude = True
				else:
					curChunk.skip()

			else:
				curChunk.skip()
		# end of while
		return(layers)

	#### --------------------------------
	"""
	def Scan_Morph_VMap(self, data, morphs):
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])
		morphs[name] = 1

	#### --------------------------------
	def Scan_RGB_VMap(self, data, lyr):
		(name, nameLen)= self.Read_nameString(data[RSize.U2:])
		if(name not in lyr.rgb):
			lyr.rgb[name] = LWO_Object.VMapObject()
		lyr.rgb[name].vmap.update({1:1})

	#### --------------------------------
	def Scan_RGB_VMad(self, data, lyr):
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])
		if(name not in lyr.rgb):
			lyr.rgb[name] = LWO_Object.VMapObject()
		lyr.rgb[name].vmad.update({1:1})

	#### --------------------------------
	def Scan_RGBA_VMap(self, data, lyr):
		(name, nameLen)= self.Read_nameString(data[RSize.U2:])
		if(name not in lyr.rgba):
			lyr.rgba[name] = LWO_Object.VMapObject()
		lyr.rgba[name].vmap.update({1:1})

	#### --------------------------------
	def Scan_RGBA_VMad(self, data, lyr):
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])
		if(name not in lyr.rgba):
			lyr.rgba[name] = LWO_Object.VMapObject()
		lyr.rgba[name].vmad.update({1:1})

	#### --------------------------------
	def Scan_NORM(self, data, lyr):
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])
		lyr.normals[name] = 1

	#### --------------------------------
	def Scan_SUBV_VMed(self, data, lyr):
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])
		lyr.hasEdgeM = True

	#### --------------------------------
	def Scan_Weight_VMad(self, data, lyr):
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])
		if(name == self.sdsEdgeWgtName):
			lyr.hasEdge = True

	#### --------------------------------
	def Scan_Bones(self, data, lyr):
		lyr.bones["dummy"] = 1
		lyr.bonNum = 1

	#### --------------------------------
	def Scan_Crvs(self, data, lyr):
		lyr.crvs = ((1,))
		lyr.crvNum = 1
	"""
	#### ========================================================================
	def _Read(self, fp):
		curPolIdxOffs = 0
		isExclude = False

		self.lwo = LWO_Object.LwoObject()
		lwo = self.lwo
		lwo.fileName = fp.name

		while(True):
			try:
				curChunk = chunk.Chunk(fp)	# FORM:size:LWO2の子階層
			except EOFError:
				break

			size = curChunk.chunksize

			chunkId = self.FnUnpckLWID(curChunk.chunkname)[0]

			if(chunkId == ChunkIdT.TAGS):
				lwo.tags = self.Read_Tags(curChunk.read(size), size)

			elif(chunkId == ChunkIdT.LAYR):
				curPolIdxOffs = 0
				isExclude = False
				self.Read_Layr(curChunk.read(size), size, lwo.layers)

			elif(chunkId == ChunkIdT.CLIP):	# [LAYR]:[CLIP][CLIP]:...
				self.Read_Clips(curChunk.read(size), size, lwo.clips)

			elif(chunkId == ChunkIdT.PNTS):	# [LAYR]:[PNTS]:...
				self.Read_Pnts(curChunk.read(size), size, lwo.layers[-1])

			elif(chunkId == ChunkIdT.VMAP):	# [LAYR]:[VMPA][VMAP][LWID]:[VMPA][VMAP][LWID]...
				(lwid, size) = self.Read_LWID(curChunk)

				if(  lwid == VMapIdT.WGHT):
					self.Read_Weight_VMap(curChunk.read(size), size, lwo.layers[-1].weights)
				elif(lwid == VMapIdT.MORF):	# Rel Morph
					self.Read_Morph_VMap(curChunk.read(size), size, lwo.layers[-1].relMorphs)
				elif(lwid == VMapIdT.SPOT):	# Abs Morph
					self.Read_Morph_VMap(curChunk.read(size), size, lwo.layers[-1].absMorphs)
				elif(lwid == VMapIdT.TXUV):
					self.Read_UV_VMap(curChunk.read(size), size, lwo.layers[-1])
				elif(lwid == VMapIdT.RGB_):
					self.Read_RGB_VMap(curChunk.read(size), size, lwo.layers[-1])
				elif(lwid == VMapIdT.RGBA):
					self.Read_RGBA_VMap(curChunk.read(size), size, lwo.layers[-1])
				elif(lwid == VMapIdT.PICK):	# Point Selection
					self.Read_Pick_VMap(curChunk.read(size), size, lwo.layers[-1].picks)
				else:
					curChunk.skip()

			elif(chunkId == ChunkIdT.VMAD):	# [LAYR]:[POLS][...][VMPA][VMAD][LWID]:[POLS][...][VMPA][VMAD][LWID]:...
				(lwid, size) = self.Read_LWID(curChunk)

				# VMADのpolIndexはPOLS内でのIndex.
				if(  lwid == VMapIdT.TXUV):
					self.Read_UV_VMad(curChunk.read(size), size, lwo.layers[-1], curPolIdxOffs)
				elif(lwid == VMapIdT.RGB_):
					self.Read_RGB_VMad(curChunk.read(size), size, lwo.layers[-1], curPolIdxOffs)
				elif(lwid == VMapIdT.RGBA):
					self.Read_RGBA_VMad(curChunk.read(size), size, lwo.layers[-1], curPolIdxOffs)
				elif(lwid == VMapIdT.WGHT):
					self.Read_Weight_VMad(curChunk.read(size), size, lwo.layers[-1], curPolIdxOffs)
				elif(lwid == VMapIdT.NORM):
					self.Read_Normal_VMad(curChunk.read(size), size, lwo.layers[-1], curPolIdxOffs)
				else:
					curChunk.skip()

			elif(chunkId == ChunkIdT.VMED):	# [LAYR]:[VMED][LWID]:...
				(lwid, size) = self.Read_LWID(curChunk)

				if(  lwid == VMedIdT.SUBV):
					self.Read_SUBV_VMed(curChunk.read(size), size, lwo.layers[-1])
#				elif(lwid == VMedIdT.EPCK):
#					curChunk.skip()
				else:
					curChunk.skip()

#			elif(chunkId == ChunkIdT.VMPA):

			elif(chunkId == ChunkIdT.POLS):	# [LAYR]:[POLS][LWID]:[POLS][LWID]:...
				(lwid, size) = self.Read_LWID(curChunk)

				if(lwid in { PolyIdT.FACE, PolyIdT.PTCH, PolyIdT.SUBD }):
					isExclude = False
					curPolIdxOffs = self.Read_Pols(curChunk.read(size), size, lwo.layers[-1], lwid)

				elif(lwid == PolyIdT.CURV):
					isExclude = True
					self.Read_Crvs(curChunk.read(size), size, lwo.layers[-1])

				elif(lwid == PolyIdT.BONE):
					self.Read_Bones(curChunk.read(size), size, lwo.layers[-1])
					isExclude = True

#				elif(lwid == PolyIdT.HBLD):	# カスタムポリ:HairBlade:
				else:
					curChunk.skip()

			elif(chunkId == ChunkIdT.PTAG):	# [LAYR]:[PTAG]size[LWID]size:[PTAG]size[LWID]size:...
				(ptag, size) = self.Read_LWID(curChunk)

				if(ptag == PTagIdT.SURF):
					if(isExclude):
						isExclude = False
						curChunk.skip()
					else:
						self.Read_PTag_Surf(curChunk.read(size), size, lwo.layers[-1], curPolIdxOffs)

				elif(ptag == PTagIdT.BONE):
					self.Read_PTag_Bone(curChunk.read(size), size, lwo.layers[-1], lwo.tags)
				elif(ptag == PTagIdT.BNWT):
					self.Read_PTag_BoneWeightMap(curChunk.read(size), size, lwo.layers[-1], lwo.tags)
				elif(ptag == PTagIdT.BNUP):
					self.Read_PTag_BoneUpVec(curChunk.read(size), size, lwo.layers[-1], lwo.tags)

				elif(ptag == PTagIdT.SKBM):
					self.Read_PTag_BoneBankMode(curChunk.read(size), size, lwo.layers[-1], lwo.tags)
				elif(ptag == PTagIdT.SKBR):
					self.Read_PTag_BoneBankRot(curChunk.read(size), size, lwo.layers[-1], lwo.tags)

				elif(ptag == PTagIdT.SKID):
					self.Read_PTag_BoneEditorId(curChunk.read(size), size, lwo.layers[-1], lwo.tags)
				elif(ptag == PTagIdT.SKPR):
					self.Read_PTag_BoneEditorParentId(curChunk.read(size), size, lwo.layers[-1], lwo.tags)

				elif(ptag == PTagIdT.COLR):
					if(isExclude):
						curChunk.skip()
					else:
						self.Read_PTag(curChunk.read(size), size, lwo.layers[-1], lwo.tags, curPolIdxOffs, "tagColors", "COLR")
				elif(ptag == PTagIdT.PART):
					if(isExclude):
						curChunk.skip()
					else:
						self.Read_PTag(curChunk.read(size), size, lwo.layers[-1], lwo.tags, curPolIdxOffs, "tagParts", "PART")

#				elif(ptag == PTagT.SMGP):
#				elif((ptag == PTagT.HBCA) or (ptag == PTagT.HBCP) or \
#					 (ptag == PTagT.HBUV) or (ptag == PTagT.HBRT)):	# HairBlade
				else:
					curChunk.skip()

			elif(chunkId == ChunkIdT.SURF):
				# [LAYR]:[SURF][SURF]:...
				self.Read_Surf(curChunk.read(size), size, lwo.surfs)

			elif(chunkId == ChunkIdT.FORM):	# LWO3 surface
				(tag, size) = self.Read_LWID(curChunk)

				offs = 0
				if(tag == SBlokIdT.SURF):
					while(offs < size):	# SURF内縛り
						try:
							curChunk = chunk.Chunk(fp)
						except EOFError:
							break

						offs += curChunk.chunksize

						chunkId = self.FnUnpckLWID(curChunk.chunkname)[0]

						if(chunkId == SBlokIdT.NAME):
							surf = LWO_Object.SurfaceObject()
							(surf.name, nameLen) = self.Read_nameString(curChunk.read(curChunk.chunksize))
							lwo.surfs[surf.name] = surf

						elif(chunkId == ChunkIdT.FORM):
							tag = self.FnUnpckTAG(curChunk.read(RSize.TAG))[0]
							curChunk.skip()
							break

						else:
							curChunk.skip()
				else:
					curChunk.skip()

			else:
				curChunk.skip()

	#### ========================================================================
	"""
	def Read_Tags(self, data, dataSize):
		u'''各種Tag名. [[tag],[tag],...]
		文末の偶数化0埋めの分が空文字として含まれる."ab"\0[pad]"cde"\0"fg"...'''
		# 2xはdata==str, 3xは==bytes,  2xのfilterはlist, 3xはiter.
		# 後工程を考えbytes→str変換.1回/fileなので分岐コストは考えない.
		if(sys.version_info[0] > 2):
			data = data.decode()
		tags = tuple([ s for s in data.split('\0') if(s!="") ])

		self.DebugPrint_Tags(tags)
		return(tags)
	"""
	#### ========================================================================
	def Read_Layr(self, data, dataSize, layers):
		u'''レイヤ情報
		LAYR { number[U2], flags[U2], pivot[VEC12], name[S0], parent[U2] ? }'''
		lyr = LWO_Object.LayerObject()

		(lyr.idx, flags) = self.FnUnpckFLAG(data[:RSize.FLAG])

		lyr.isHide = (flags & 0x1)	# bitONで不可視
		lyr.piv = self.FnUnpckV3(data[RSize.U4:RSize.LYRN])		# (x,y,z)
		(name, nameLen) = self.Read_nameString(data[RSize.LYRN:])

		# レイヤ名が未設定の場合, 規定名+(idx+1).を適用.
		lyr.name = name[:] if(name != "") else "{}_{}".format(lyr.defaultName, (lyr.idx +1))

		offs = (RSize.LYRN + nameLen)
		if(dataSize > offs):
			lyr.parent = self.FnUnpckI2(data[offs:(offs + RSize.I2)])[0]

		layers.append(lyr)

		self.DebugPrint_Layer(lyr, flags)

	#### ========================================================================
	"""
	def Read_Pnts(self, data, dataSize, lyr):
		u'''頂点配列. [(x,y,z),(...),...]
		PNTS { point-location[VEC12] * }'''
#		APD = lyr.pnts.append
#		offs = 0
#		while(offs < dataSize):
#			APD( self.FnUnpckV3(data[offs:(offs + RSize.VEC3)]) )	# (x,y,z)
#			offs += RSize.VEC3
		lyr.pnts = [ self.FnUnpckV3(data[offs: offs + RSize.VEC3]) for offs in range(0, dataSize, RSize.VEC3) ]
		lyr.pntNum = len(lyr.pnts)

		self.DebugPrint_Pnts(lyr.name, lyr.pnts, lyr.pntNum)
	"""
	#### ========================================================================
	def Read_Pick_VMap(self, data, dataSize, picks):
		u'''Pick(SelectionSet)/VMap
		VMAP { type[ID4], dimension[U2], name[S0], vert[VX]* }
		PICK, dim[U2], name[S0], vert[VX],,,,'''
		(name, nameLen)= self.Read_nameString(data[RSize.U2:])

		pick = []
		APD = pick.append
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			APD(pntIdx)

		picks[name] = tuple(pick)	# { name:(pntIdx,,,,) }

		self.DebugPrint_Pick(name, pick)

	#### ========================================================================
	def Read_Weight_VMap(self, data, dataSize, weights):
		u'''Weight/VMap
		VMAP { type[ID4], dimension[U2], name[S0],( vert[VX], value[F4] # dimension )* }
		WGHT, dim[U2], name[S0], vert[VX], value[F4]'''
		(name, nameLen)= self.Read_nameString(data[RSize.U2:])

		wgt = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			wgt[pntIdx] = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			offs += RSize.F4

		weights[name] = wgt

		self.DebugPrint_Weight_VMap(name, wgt)

	#### ========================================================================
	def Read_Morph_VMap(self, data, dataSize, morphs):
		u'''Morph/VMap
		VMAP { type[ID4], dimension[U2], name[S0],( vert[VX], value[F4] # dimension )* }
		MORF, dim[U2], name[S0], vert[VX], value[VEC12]'''
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])

		mrfs = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			mrfs[pntIdx] = self.FnUnpckV3(data[offs:(offs + RSize.VEC3)])	# (x,y,z)
			offs += RSize.VEC3

		morphs[name] = mrfs

		self.DebugPrint_Morph(name, mrfs)

	#### ========================================================================
	def Read_RGB_VMap(self, data, dataSize, lyr):
		u'''VertexColor, RGB/VMap
		VMAP { type[ID4], dimension[U2], name[S0],( vert[VX], value[F4] # dimension )* }
		RGB, dim[U2], name[S0], vert[VX], value[VEC12]'''
		(name, nameLen)= self.Read_nameString(data[RSize.U2:])

		cols = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			cols[pntIdx] = self.FnUnpckV3(data[offs:(offs + RSize.VEC3)]) + (1.0,)	# (r,g,b,A)RGBAにする
			offs += RSize.VEC3

		if(name not in lyr.rgb):
			lyr.rgb[name] = LWO_Object.VMapObject()
		lyr.rgb[name].vmap.update(cols)

		self.DebugPrint_RGBA("RGB/VMap", name, lyr.rgb[name].vmap)

	#### ========================================================================
	def Read_RGBA_VMap(self, data, dataSize, lyr):
		u'''VertexColor, RGBA/VMap
		VMAP { type[ID4], dimension[U2], name[S0],( vert[VX], value[F4] # dimension )* }
		RGBA, dim[U2], name[S0], vert[VX], value[VEC16]'''
		(name, nameLen)= self.Read_nameString(data[RSize.U2:])

		cols = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			cols[pntIdx] = self.FnUnpckV4(data[offs:(offs + RSize.VEC4)])	# (r,g,b,a)
			offs += RSize.VEC4

		if(name not in lyr.rgba):
			lyr.rgba[name] = LWO_Object.VMapObject()
		lyr.rgba[name].vmap.update(cols)

		self.DebugPrint_RGBA("RGBA/VMap", name, lyr.rgba[name].vmap)

	#### ========================================================================
	def Read_RGB_VMad(self, data, dataSize, lyr, curPolIdxOffs):
		u'''VertexColor, RGB/VMad
		VMAD { type[ID4], dimension[U2], name[S0],( vert[VX], poly[VX], value[F4] # dimension )* } 
		RGB, dim[U2], name[S0],( vert[VX], poly[VX], value[VEC12])'''
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])

		cols = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx

			polIdx += curPolIdxOffs	# polIdxはPOLS内でのIndex
			col = self.FnUnpckV3(data[offs:(offs + RSize.VEC3)]) + (1.0,)	# (r,g,b,A)RGBAにする
			offs += RSize.VEC3
			if(polIdx in cols):
				cols[polIdx][pntIdx] = col
			else:
				cols[polIdx] = dict({ pntIdx: col })

		if(name not in lyr.rgb):
			lyr.rgb[name] = LWO_Object.VMapObject()
		lyr.rgb[name].vmad.update(cols)

		self.DebugPrint_RGBA("RGB/VMad", name, lyr.rgb[name].vmad)

	#### ========================================================================
	def Read_RGBA_VMad(self, data, dataSize, lyr, curPolIdxOffs):
		u'''VertexColor(RGB/RGBA)/VMad
		VMAD { type[ID4], dimension[U2], name[S0],( vert[VX], poly[VX], value[F4] # dimension )* } 
		RGBA, dim[U2], name[S0],( vert[VX], poly[VX], value[VEC16])'''
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])

		cols = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx

			polIdx += curPolIdxOffs	# polIdxはPOLS内でのIndex
			col = self.FnUnpckV4(data[offs:(offs + RSize.VEC4)])	# (r,g,b,a)
			offs += RSize.VEC4
			if(polIdx in cols):
				cols[polIdx][pntIdx] = col
			else:
				cols[polIdx] = dict({ pntIdx: col })

		if(name not in lyr.rgba):
			lyr.rgba[name] = LWO_Object.VMapObject()
		lyr.rgba[name].vmad.update(cols)

		self.DebugPrint_RGBA("RGBA/VMad", name, lyr.rgba[name].vmad)

	#### ========================================================================
	def Read_UV_VMap(self, data, dataSize, lyr):
		u'''UV/VMap. {name:{vmap:{pntIdx:[u,v]}}}
		VMAP { type[ID4], dimension[U2], name[S0], ( vert[VX], value[F4] # dimension )* }
		TXUV, dim[U2], name[S0], vert[VX], value[VEC8]'''
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])

		uvs = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			uvs[pntIdx] = self.FnUnpckUV(data[offs:(offs + RSize.UV)])	# (u,v)
			offs += RSize.UV

		if(name not in lyr.uvs):
			lyr.uvs[name] = LWO_Object.VMapObject()
		lyr.uvs[name].vmap.update(uvs)

		self.DebugPrint_UV("VMap", name, lyr.uvs[name].vmap)

	#### ========================================================================
	def Read_UV_VMad(self, data, dataSize, lyr, curPolIdxOffs):
		u'''UV/VMad. {name:{vmad:{polIdx:{pntIdx:[u,v]}}}}
		VMAD { type[ID4], dimension[U2], name[S0],( vert[VX], poly[VX], value[F4] # dimension )* } 
		TXUV,  dim[U2], name[S0],( vert[VX], poly[VX], value[VEC8])'''
		name, nameLen = self.Read_nameString(data[RSize.U2:])

		uvs = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			uvVal = self.FnUnpckUV(data[offs:(offs + RSize.UV)])	# (u,v)
			offs += RSize.UV

			polIdx += curPolIdxOffs	# polIdxはPOLS内でのIndex
			if(polIdx in uvs):	uvs[polIdx][pntIdx] = uvVal
			else:				uvs[polIdx] = dict( { pntIdx: uvVal} )

		if(name not in lyr.uvs):
			lyr.uvs[name] = LWO_Object.VMapObject()
		lyr.uvs[name].vmad.update(uvs)

		self.DebugPrint_UV("VMad", name, lyr.uvs[name].vmad)

	#### ========================================================================
	def Read_Normal_VMad(self, data, dataSize, lyr, curPolIdxOffs):
		u'''NORM/VMad. {name:{vmad:{polIdx:{pntIdx:[u,v]}}}}
		VMAD { type[ID4], dimension[U2], name[S0],( vert[VX], poly[VX], value[F4] # dimension )* } 
		NORM, dim[U2], name[S0],( vert[VX], poly[VX], value[VEC12])'''
		name, nameLen = self.Read_nameString(data[RSize.U2:])

		norm = {}
		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			vec = self.FnUnpckV3(data[offs:(offs + RSize.VEC3)])	# (x,y,z)
			offs += RSize.VEC3

			polIdx += curPolIdxOffs	# polIdxはPOLS内でのIndex

			if(polIdx in norm):	norm[polIdx][pntIdx] = vec
			else:				norm[polIdx] = dict( { pntIdx: vec} )

		lyr.normals[name] = norm

		self.DebugPrint_NORM(name, norm)

	#### ========================================================================
	def Read_Weight_VMad(self, data, dataSize, lyr, curPolIdxOffs):
		u'''Weight/VMad. {{id1 id2:val},{id1 id2:val},...}
		VMAD { type[ID4], dimension[U2], name[S0],( vert[VX], poly[VX], value[F4] # dimension )* } 
		WGHT,  dim[U2], name[S0],( vert[VX], poly[VX], value[F4])
		Some info: LW stores a face's points in a clock-wize order (with the normal pointing at you).
		This gives edges a 'direction' which is used when it comes to storing CC edge weight values.
		The weight is given to the point preceding the edge that the weight belongs to.'''
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])
		if(name != self.sdsEdgeWgtName):
			return

		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			wgt = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			offs += RSize.F4

			polPnts = lyr.pols[(polIdx + curPolIdxOffs)]	# polIdxはPOLS内でのIndex
			try:
				idx1 = polPnts.index(pntIdx)
			except:
				continue

			pnt2 = polPnts[0] if(idx1 == len(polPnts) -1) else polPnts[idx1 + 1]
			lyr.edgeWgts[(pnt2, pntIdx)] = wgt

		lyr.hasEdge = True

		self.DebugPrint_EdgeWeight("VMad", name, lyr.edgeWgts)

	#### ========================================================================
	"""
	def Read_SUBV_VMed(self, data, dataSize, lyr):
		u'''Modo EdgeWeight. {{id1 id2:val},{id1 id2:val},...}
		VMED { type[ID4], dimension[U2], name[S0],( vert[VX], vert[VX], value[F4] # dimension )* }
		SUBV, dim[U2], name[S0],( vert[VX], vert[VX], value[F4] # dimension )*'''
		(name, nameLen) = self.Read_nameString(data[RSize.U2:])

		offs = RSize.U2 + nameLen
		while(offs < dataSize):
			(idx1, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			(idx2, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			wgt = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			offs += RSize.F4
			lyr.edgeWgtsM[(idx1, idx2)] = wgt

		lyr.hasEdgeM = True

		self.DebugPrint_EdgeWeight("VMed(Modo)", name, lyr.edgeWgtsM)
	"""
	#### ========================================================================
	def Read_Pols(self, data, dataSize, lyr, lwid):
		u'''ポリ配列. [(pntIdx,pntIdx,...),(...),...]
		POLS { type[ID4], ( numvert+flags[U2], vert[VX] # numvert )* }'''
		curPolIdxOffs = len(lyr.pols)	# polIdxのoffsetとして.

		pols = []
		PolAPD = pols.append
		offs = 0
		while(offs < dataSize):
			(pntsNum,) = self.FnUnpckU2(data[offs:(offs + RSize.U2)])
			pntsNum &= 0x03FF # 上位6bitはフラグ, max1023
			offs += RSize.U2
			pntsIdxs= []
			PntAPD = pntsIdxs.append
			for i in range(pntsNum):
				(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
				offs += vx
				PntAPD(pntIdx)		# [pntIdx,pntIdx,...]
			PolAPD(tuple(pntsIdxs))	# [(pntIdx,pntIdx,...),(...),...]

		lyr.pols.extend(pols)

		self.DebugPrint_Pols(lyr.name, lwid, pols)

		lyr.polNum += len(pols)	# 累積する

		lyr.lwid = lwid

		return(curPolIdxOffs)

	#### ========================================================================
	def Read_Crvs(self, data, dataSize, lyr):
		u'''カーブ配列. [(flag, pntIdx,pntIdx,...),(...),...]
		POLS { type[ID4], ( numvert+flags[U2], vert[VX] # numvert )* }'''
		crvs = []
		CrvAPD = crvs.append
		offs = 0
		while(offs < dataSize):
			val = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2
			# カーブ NNNN FFnn nnnn nnnn, N+n=pntNum, F=continuFlag SE--(S=st, E=ed)
			# -C-- 且つ 同一のPntIdxを指定している場合は閉じたカーブ
			pntsNum = (val & 0x03FF)|((val & 0xF000) >> 2)
			pntsIdxs= [((val & 0x0C00) >> 10), ]
			PntAPD = pntsIdxs.append
			for i in range(pntsNum):
				(pntIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
				offs += vx
				PntAPD(pntIdx)		# [flag, pntIdx,pntIdx,...]
			CrvAPD(tuple(pntsIdxs))	# [(flag, pntIdx,pntIdx,...),(...),...]

		lyr.crvs   = tuple(crvs)
		lyr.crvNum = len(crvs)

		self.DebugPrint_Crvs(lyr.name, "CURV", crvs, lyr.crvNum)

	#### ========================================================================
	def Read_Bones(self, data, dataSize, lyr):
		u'''ボーン読込
		POLS { type[ID4], ( numvert+flags[U2], vert[VX] # numvert )* }'''
		bones = {}
		offs = boneIdx = 0
		while(offs < dataSize):
			offs += RSize.U2 # 頂点数は2固定
			(pntIdx0, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			(pntIdx1, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			bones[boneIdx] = LWO_Object.BoneObject(pnt0= pntIdx0, pnt1= pntIdx1)
			boneIdx += 1

		lyr.bones  = bones
		lyr.bonNum = len(bones)

		bs = list(range(lyr.bonNum))
		for i in bs:
			bon = bones[i]
			for j in bs:
				if(bon.pnt1 == bones[j].pnt0):	# 端点と始点の結合を親子組と見做す
					bon.childs.append(j)
		# Jointと見做しての親子組の都合上,端点を有意な点と見做す.
		# 親を持たないボーンの始点[0]はダミーとし生成しない.

		self.DebugPrint_Bone(lyr.name, lyr.bones, lyr.bonNum)

	#### ========================================================================
	def Read_PTag_Bone(self, data, dataSize, lyr, tags):
		u'''ボーン名読込'''
		bones = lyr.bones
		offs = 0
		while(offs < dataSize):
			(boneIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2
			bones[boneIdx].name = tags[tagIdx]

		self.DebugPrint_PTagBone("name", lyr.name, lyr.bones, "Bone BONE")

	#### ========================================================================
	def Read_PTag_BoneWeightMap(self, data, dataSize, lyr, tags):
		u'''ボーンウェイト名読込'''
		bones = lyr.bones
		offs = 0
		while(offs < dataSize):
			(boneIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2
			bones[boneIdx].weight = tags[tagIdx]

		self.DebugPrint_PTagBone("weight", lyr.name, lyr.bones, "Bone WGHT")

	#### ========================================================================
	def Read_PTag_BoneUpVec(self, data, dataSize, lyr, tags):
		u'''スケルゴン UpVector値読込'''
		bones = lyr.bones
		offs = 0
		while(offs < dataSize):
			(boneIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2

			vstr = tags[tagIdx].split(" ")
			bones[boneIdx].bnup = ( float(vstr[0]),float(vstr[1]),float(vstr[2]) )

		self.DebugPrint_PTagBone("bnup", lyr.name, lyr.bones, "Bone BNUP")

	#### ========================================================================
	def Read_PTag_BoneEditorParentId(self, data, dataSize, lyr, tags):
		u'''スケルゴン EditorId読込'''
		bones = lyr.bones
		offs = 0
		while(offs < dataSize):
			(boneIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2
			bones[boneIdx].skpr = int(tags[tagIdx])

		self.DebugPrint_PTagBone("skpr", lyr.name, lyr.bones, "Bone SKPR")

	#### ========================================================================
	def Read_PTag_BoneEditorId(self, data, dataSize, lyr, tags):
		u'''スケルゴン EditorId読込'''
		bones = lyr.bones
		offs = 0
		while(offs < dataSize):
			(boneIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2
			bones[boneIdx].skid = int(tags[tagIdx])

		self.DebugPrint_PTagBone("skid", lyr.name, lyr.bones, "Bone SKID")

	#### ========================================================================
	def Read_PTag_BoneBankMode(self, data, dataSize, lyr, tags):
		u'''スケルゴン BankMode読込'''
		bones = lyr.bones
		offs = 0
		while(offs < dataSize):
			(boneIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2
			bones[boneIdx].skbm = int(tags[tagIdx])

		self.DebugPrint_PTagBone("skbm", lyr.name, lyr.bones, "Bone SKBM")

	#### ========================================================================
	def Read_PTag_BoneBankRot(self, data, dataSize, lyr, tags):
		u'''スケルゴン BankRotate値読込'''
		bones = lyr.bones
		offs = 0
		while(offs < dataSize):
			(boneIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2
			bones[boneIdx].skbr = float(tags[tagIdx])

		self.DebugPrint_PTagBone("skbr", lyr.name, lyr.bones, "Bone SKBR")

	#### ========================================================================
	"""
	def Read_PTag_Surf(self, data, dataSize, lyr, curPolIdxOffs):
		u'''Poly&SurfaceIdx. {TagsIdx:[polIdx,polIdx,...]}
		PTAG { type[ID4], ( poly[VX], tag[U2] )* } 
		POL->TAGS(name)->Surface'''
		offs = 0
		while(offs < dataSize):
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2

			if(tagIdx not in lyr.surfTags):
				lyr.surfTags[tagIdx] = []	# 既設でなければ作成

			lyr.surfTags[tagIdx].append(polIdx + curPolIdxOffs)	# POLS内のIndex

		self.DebugPrint_PTag("SURF", lyr.name, lyr.surfTags)
	"""
	#### ========================================================================
	def Read_Surf(self, data, dataSize, surfs):
		u'''Surface
		SURF { name[S0], source[S0], attributes[SUB-CHUNK] * }'''

		surf = LWO_Object.SurfaceObject()
		(name, nameLen) = self.Read_nameString(data)
		if(name):
			surf.name = name

		(srcName, srcNameLen) = self.Read_nameString(data[nameLen:])
#		if(len(srcName) != 0): surf.srcName = srcName

		offs = nameLen + srcNameLen
		while(offs < dataSize):
			(subId, subLen) = self.FnUnpckTAGU2(data[offs:(offs + RSize.TAGU2)])
			offs += RSize.TAGU2

			if(  subId == SurfIdT.COLR):		# COLR { base-color[COL12], envelope[VX] }
				surf.colr = self.FnUnpckV3(data[offs:(offs + RSize.VEC3)])
			elif(subId == SurfIdT.DIFF):		# DIFF { intensity[F4], envelope[VX] }
				surf.diff = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.LUMI):		# LUMI { intensity[F4], envelope[VX] }
				surf.lumi = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.SPEC):		# SPEC { intensity[F4], envelope[VX] }
				surf.spec = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.REFL):		# REFL { intensity[F4], envelope[VX] }
				surf.refl = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.TRAN):		# TRAN { intensity[F4], envelope[VX] }
				surf.tran = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.TRNL):		# TRNL { intensity[F4], envelope[VX] }
				surf.trnl = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.GLOS):		# GLOS { glossiness[F4], envelope[VX] }
				surf.glos = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
#			elif(subId == SurfIdT.SHRP):		# SHRP { sharpness[F4], envelope[VX] }
#				surf.shrp = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.BUMP):		# BUMP { strength[F4], envelope[VX] } 
				surf.bump = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
			elif(subId == SurfIdT.SIDE):		# SIDE { sidedness[U2] }
				surf.side = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			elif(subId == SurfIdT.SMAN):		# SMAN { max-smoothing-angle[ANG4] }
				surf.sman = self.FnUnpckANG(data[offs:(offs + RSize.ANG4)])[0]

			elif(subId == SurfIdT.VCOL):		# VCOL { intensity[FP4], envelope[VX], vmap-type[ID4], name[S0] }
				intensity = self.FnUnpckF4(data[offs:(offs + RSize.F4)])[0]
				tmp = offs + RSize.F4
				(env, vx) = self.Read_VX(data[tmp:(tmp + RSize.VX)])	# env
				tmp += vx
				lwid = self.FnUnpckLWID(data[tmp:(tmp + RSize.LWID)])[0]
				tmp += RSize.TAG
				(name, tmp1) = self.Read_nameString(data[tmp:])

				surf.vcol = (name, lwid, intensity)
				# offset本体への積算は不要

			elif(subId == SurfIdT.NORM):	# NORM { intensity[FP4], name[S0] }
				(surf.norm, nameLen) = self.Read_nameString(data[offs:])

			elif(subId == SurfIdT.BLOK):	# BLOK { header[SUB-CHUNK], attributes[SUB-CHUNK] * } 
				blok = LWO_Object.ImapBlokObject()

				tmp = offs
				tmpMax = offs + subLen
				while(tmp < tmpMax):
					(head, headLen) = self.FnUnpckTAGU2(data[tmp:(tmp + RSize.TAGU2)])
					tmp += RSize.TAGU2

					if(head == BlokIdT.IMAP):
						blok.ord = self.FnUnpckU2(data[tmp:(tmp + RSize.U2)])[0]

						tmp0 = tmp + RSize.U2
						tmp0Max = tmp + headLen
						while(tmp0 < tmp0Max):
							(blkId, blkLen) = self.FnUnpckTAGU2(data[tmp0:(tmp0 + RSize.TAGU2)])
							tmp0 += RSize.TAGU2

							if(  blkId == ImgIdT.CHAN):
								blok.chan = self.FnUnpckTAG(data[tmp0:(tmp0 + RSize.TAG)])[0]
							elif(blkId == ImgIdT.OPAC):
								(blok.opacType, blok.opac) = self.FnUnpckOPAC(data[tmp0:(tmp0 + RSize.OPAC)])
							elif(blkId == ImgIdT.ENAB):
								blok.enable = self.FnUnpckU2(data[tmp0:(tmp0 + RSize.U2)])[0]
							elif(blkId == ImgIdT.NEGA):
								blok.negative = self.FnUnpckU2(data[tmp0:(tmp0 + RSize.U2)])[0]
							elif(blkId == ImgIdT.AXIS):
								blok.dispAxis = self.FnUnpckU2(data[tmp0:(tmp0 + RSize.U2)])[0]
							else:
								pass
							tmp0 += blkLen

					elif(head == BlokIdT.TMAP):
						tmp0 = tmp
						tmp0Max = tmp + headLen
						while(tmp0 < tmp0Max):
							(blkId, blkLen) = self.FnUnpckTAGU2(data[tmp0:(tmp0 + RSize.TAGU2)])
							tmp0 += RSize.TAGU2

							if(  blkId == TxtrIdT.CNTR):
								blok.center = self.FnUnpckV3(data[tmp0:(tmp0 + RSize.VEC3)])
							elif(blkId == TxtrIdT.SIZE):
								blok.size   = self.FnUnpckV3(data[tmp0:(tmp0 + RSize.VEC3)])
							elif(blkId == TxtrIdT.ROTA):
								blok.rotate = self.FnUnpckV3(data[tmp0:(tmp0 + RSize.VEC3)])
#							elif(blkId == TxtrIdT.FALL):
#							elif(blkId == TxtrIdT.OREF):
#							elif(blkId == TxtrIdT.CSYS):
							else:
								pass
							tmp0 += blkLen

					elif(head == BlokIdT.PROJ):
						blok.project = self.FnUnpckU2(data[tmp:(tmp + RSize.U2)])[0]
					elif(head == BlokIdT.AXIS):
						blok.prjAxis = self.FnUnpckU2(data[tmp:(tmp + RSize.U2)])[0]
					elif(head == BlokIdT.IMAG):
						(blok.imgIdx, vx) = self.Read_VX(data[tmp:(tmp + RSize.VX)])
					elif(head == BlokIdT.WRAP):
						(blok.wrapOptW, blok.wrapOptH) = self.FnUnpckU2U2(data[tmp:(tmp + RSize.U2U2)])
					elif(head == BlokIdT.WRPW):
						blok.wrapAmountW = self.FnUnpckF4(data[tmp:(tmp + RSize.F4)])[0]
					elif(head == BlokIdT.WRPH):
						blok.wrapAmountH = self.FnUnpckF4(data[tmp:(tmp + RSize.F4)])[0]
					elif(head == BlokIdT.VMAP):
						(blok.vmapName, nameLen)= self.Read_nameString(data[tmp:])
#					elif(head == BlokIdT.AAST):
#					elif(head == BlokIdT.PIXB):
#					elif(head == BlokIdT.STCK):
#					elif(head == BlokIdT.TAMP):
					else:
						pass
					tmp += headLen
				surf.imap.append(blok)
				# offset本体への積算は不要

			"""
			elif(subId == SurfIdT.RFOP):		# RFOP { reflection-options[U2] }
			elif(subId == SurfIdT.RIMG):		# RIMG { image[VX] }
			elif(subId == SurfIdT.RSAN):		# RSAN { seam-angle[ANG4], envelope[VX] }
			elif(subId == SurfIdT.RBLR):		# RBLR { blur-percentage[FP4], envelope[VX] }
				(surf.rblr,) = self.FnUnpckF4(data[offs:(offs + RSize.F4)])
			elif(subId == SurfIdT.RIND):		# RIND { refractive-index[F4], envelope[VX] }
				(surf.rind,) = self.FnUnpckF4(data[offs:(offs + RSize.F4)])
			elif(subId == SurfIdT.TROP):		# TROP { transparency-options[U2] }
			elif(subId == SurfIdT.TIMG):		# TIMG { image[VX] }
			elif(subId == SurfIdT.TBLR):		# TBLR { blur-percentage[FP4], envelope[VX] }
				(surf.tblr,) = self.FnUnpckF4(data[offs:(offs + RSize.F4)])
			elif(subId == SurfIdT.CLRH):		# CLRH { color-highlights[FP4], envelope[VX] }
			elif(subId == SurfIdT.CLRF):		# CLRF { color-filter[FP4], envelope[VX] }
			elif(subId == SurfIdT.ADTR):		# ADTR { additive[FP4], envelope[VX] }
			elif(subId == SurfIdT.GLOW):		# GLOW { type[U2], intensity[F4], intensity-envelope[VX], size[F4], size-envelope[VX] }
			elif(subId == SurfIdT.LINE):		# LINE { flags[U2], ( size[F4], size-envelope[VX], ( color[COL12], color-envelope[VX] )? )? }
			elif(subId == SurfIdT.ALPH):		# ALPH { mode[U2], value[FP4] }
			"""

			offs += subLen

		surfs[surf.name] = surf

		self.DebugPrint_Surf(surf)

	#### ========================================================================
	"""
	def Read_PTag_Color(self, data, dataSize, lyr, tags, curPolIdxOffs):
		u'''PTAG_COLR. {"col":[polIdx,,,]}'''
		offs = 0
		while(offs < dataSize):
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2

			tag = tags[tagIdx]	# 色名
			if(tag not in lyr.tagColors):
				lyr.tagColors[tag] = []

			lyr.tagColors[tag].append(polIdx + curPolIdxOffs)

		self.DebugPrint_PTag("COLR", lyr.name, lyr.tagColors)

	#### ========================================================================
	def Read_PTag_Part(self, data, dataSize, lyr, tags, curPolIdxOffs):
		u'''PTAG_PART. {name:[polIdx,,,]}'''
		offs = 0
		while(offs < dataSize):
			(polIdx, vx) = self.Read_VX(data[offs:(offs + RSize.VX)])
			offs += vx
			tagIdx = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]
			offs += RSize.U2

			tag = tags[tagIdx]	# パート名
			if(tag not in lyr.tagParts):
				lyr.tagParts[tag] = []

			lyr.tagParts[tag].append(polIdx + curPolIdxOffs)

		self.DebugPrint_PTag("PART", lyr.name, lyr.tagParts)
	"""
	#### ========================================================================
	def Read_Clips(self, data, dataSize, clips):
		u'''Images, CLIP { idx[U4], attributes[SUB-CHUNK] * }'''
		clip = LWO_Object.ClipObject()

		clip.id = self.FnUnpckU4(data[:RSize.U4])[0]

		offs = RSize.U4
		while(offs < dataSize):
			(subId, subLen) = self.FnUnpckTAGU2(data[offs:(offs + RSize.TAGU2)])

			offs += RSize.TAGU2

			clip.type = ClipNameT[subId]

			if(  subId == ClipIdT.STIL):		# STIL { name[FNAM0] }
				(clip.stillName, tmp) = self.Read_nameString(data[offs:])

			elif(subId == ClipIdT.ISEQ):		# ISEQ { num-digits[U1], flags[U1], offset[I2], reserved[U2], start[I2], end[I2], prefix[FNAM0], suffix[S0] }
				(clip.seqDigit, clip.seqFlags, clip.seqOffset, clip.seqReserve, clip.seqStart, clip.seqEnd,) =\
					self.FnUnpckISEQ(data[offs:(offs + RSize.ISEQ)])
				tmp = offs + 10
				(clip.seqName,len0)= self.Read_nameString(data[tmp:])
				tmp += len0
				(clip.seqExt, len0)= self.Read_nameString(data[tmp:])
				# offset本体への積算はNG

			elif(subId == ClipIdT.ANIM):		# ANIM { filename[FNAM0], server-name[S0], flags[U2], data[...] }
				tmp = offs
				(clip.animName,  len0)= self.Read_nameString(data[tmp:])
				tmp += len0
				(clip.animServer,len0)= self.Read_nameString(data[tmp:])
				tmp += len0
				clip.animFlags = self.FnUnpckU2(data[tmp:(tmp + RSize.U2)])[0]
				# offset本体への積算はNG

			elif(subId == ClipIdT.XREF):		# XREF { index[U4], string[S0] }
				clip.xrefIdx = self.FnUnpckU4(data[offs:(offs + RSize.U4)])[0]
				(clip.xrefName, tmp)= self.Read_nameString(data[(offs + RSize.U4):])

#			elif(subId == ClipIdT.STCC):		# STCC { lo[I2], hi[I2], name[FNAM0] }
#				(surf.refl,) = self.FnUnpckF4(data[offs:(offs + RSize.F4)])

			elif(subId == ClipIdT.TIME):		# TIME { start-time[FP4], duration[FP4], frame-rate[FP4] }
				(clip.stTime, clip.duration, clip.fps) = self.FnUnpckV3(data[offs:(offs + RSize.VEC3)])
			elif(subId == ClipIdT.NEGA):		# NEGA { enable[U2] }
				clip.neg = self.FnUnpckU2(data[offs:(offs + RSize.U2)])[0]

#			elif(subId == ClipIdT.CONT):		# CONT { contrast-delta[FP4], envelope[VX] }
#			elif(subId == ClipIdT.BRIT):		# BRIT { brightness-delta[FP4], envelope[VX] }
#			elif(subId == ClipIdT.SATR):		# SATR { saturation-delta[FP4], envelope[VX] }
#			elif(subId == ClipIdT.HUE_):		# HUE { hue-rotation[FP4], envelope[VX] }
#			elif(subId == ClipIdT.GAMM):		# GAMM { gamma[F4], envelope[VX] }
#			elif(subId == ClipIdT.IFLT):		# IFLT { server-name[S0], flags[U2], data[...] }
#			elif(subId == ClipIdT.PFLT):		# PFLT { server-name[S0], flags[U2], data[...] }
#			elif(subId == ClipIdT.FLAG):		# FLAG { flags[U4] }
			offs += subLen

		clips[clip.id] = clip

		self.DebugPrint_Clip(clip)
