﻿# -*- coding: utf-8 -*-

#### =======================================================================
__all__ = ("VisFlags",)

#### ========================================================================
class VisFlags(object):
	__slots__ = (
		"_PNT",  "_POL",  "_CRV",  "_BONE",
		"_UV",   "_WGHT", "_MORF", "_VCOL",
		"_EDGE", "_NORM", "_SELS", "_DMY1",
		"_SURF", "_IMAP", "_CLIP", "_PSURF",
		"_LAYR", "_TAGS", "_PTAG", "_DMY2",
		"__debug", "__verbose", "__flags",
	)
	ALL = 0xFFFFF

	#### -----------------------------------------------------------------------
	def __init__(self, *arg, **kwd):
		(self.__debug, self.__verbose, self.__flags) = self.GetFromKwd(**kwd)
		self.__Update()

	#### -----------------------------------------------------------------------
	def GetKwd(self):
		u'''kwd辞書の出力'''
		return( dict(d= self.__debug, v= self.__verbose, f= self.__flags) )

	#### -----------------------------------------------------------------------
	def SetKwd(self, **kwd):
		u'''kwd辞書で更新'''
		__GetOpt = lambda kwd,sn,ln,dv: kwd.get(sn,kwd.get(ln,None))
		d = __GetOpt(kwd, "d", "debug")
		if(d is not None):
			self.DEBUG = d
		v = __GetOpt(kwd, "v", "verbose")
		if(v is not None):
			self.VERB = v
		f = __GetOpt(kwd, "f", "flags")
		if(f is not None):
			self.FLAGS = f

	#### -----------------------------------------------------------------------
	@staticmethod
	def GetFromKwd(**kwd):
		u'''kwdから抽出, 不適切な場合はデフォ値へ変更.'''
		__GetOpt = (lambda kwd,sn,ln,dv:(lambda v:v if(isinstance(v,(int,bool))) else dv)(kwd.get(sn,kwd.get(ln,dv))))
		d = __GetOpt(kwd, "d", "debug", False)
		v = __GetOpt(kwd, "v", "verbose", False)
		f = __GetOpt(kwd, "f", "flags", 0)
		return(d, v, f)

	#### -----------------------------------------------------------------------
	def __Update(self):
		u'''flag値を振り分け'''
		f = 0
		for (i, k) in enumerate(self.__slots__[:-3]):
			setattr(self, k, bool(((self.__flags>>i)&0x1)))
	#### ================================
	@property
	def FLAGS(self):
		return(self.__flags)
	@FLAGS.setter
	def FLAGS(self, v):
		if(not isinstance(v, int)):
			raise TypeError
		self.__flags = v
		self.__Update()
	#### --------------------------------
	@property
	def DEBUG(self):
		return(self.__debug)
	@DEBUG.setter
	def DEBUG(self, v):
		if(not isinstance(v, int)):
			raise TypeError
		self.__debug = v
	#### --------------------------------
	@property
	def VERB(self):
		return(self.__verbose)
	@VERB.setter
	def VERB(self, v):
		if(not isinstance(v, int)):
			raise TypeError
		self.__verbose = v

	#### -----------------------------------------------------------------------
	def __UpdateFlags(self, k, v):
		if(not isinstance(v, (bool, int))):
			raise TypeError

		setattr(self, k, bool(v))

		f = 0
		for (i, k) in enumerate(self.__slots__[:-3]):
			f |= getattr(self, k) << i
		self.__flags = f
	#### ================================
	@property
	def PNT(self):
		return(self._PNT)
	@PNT.setter
	def PNT(self, v):
		self.__UpdateFlags("_PNT", v)
	#### --------------------------------
	@property
	def POL(self):
		return(self._POL)
	@POL.setter
	def POL(self, v):
		self.__UpdateFlags("_POL", v)
	#### --------------------------------
	@property
	def CRV(self):
		return(self._CRV)
	@CRV.setter
	def CRV(self, v):
		self.__UpdateFlags("_CRV", v)
	#### --------------------------------
	@property
	def BONE(self):
		return(self._BONE)
	@BONE.setter
	def BONE(self, v):
		self.__UpdateFlags("_BONE", v)

	#### ================================
	@property
	def UV(self):
		return(self._UV)
	@UV.setter
	def UV(self, v):
		self.__UpdateFlags("_UV", v)
	#### --------------------------------
	@property
	def WGHT(self):
		return(self._WGHT)
	@WGHT.setter
	def WGHT(self, v):
		self.__UpdateFlags("_WGHT", v)
	#### --------------------------------
	@property
	def MORF(self):
		return(self._MORF)
	@MORF.setter
	def MORF(self, v):
		self.__UpdateFlags("_MORF", v)
	#### --------------------------------
	@property
	def VCOL(self):
		return(self._VCOL)
	@VCOL.setter
	def VCOL(self, v):
		self.__UpdateFlags("_VCOL", v)

	#### ================================
	@property
	def EDGE(self):
		return(self._EDGE)
	@EDGE.setter
	def EDGE(self, v):
		self.__UpdateFlags("_EDGE", v)
	#### --------------------------------
	@property
	def NORM(self):
		return(self._NORM)
	@NORM.setter
	def NORM(self, v):
		self.__UpdateFlags("_NORM", v)
	#### --------------------------------
	@property
	def SELS(self):
		return(self._SELS)
	@SELS.setter
	def SELS(self, v):
		self.__UpdateFlags("_SELS", v)

	#### ================================
	@property
	def SURF(self):
		return(self._SURF)
	@SURF.setter
	def SURF(self, v):
		self.__UpdateFlags("_SURF", v)
	#### --------------------------------
	@property
	def IMAP(self):
		return(self._IMAP)
	@IMAP.setter
	def IMAP(self, v):
		self.__UpdateFlags("_IMAP", v)
	#### --------------------------------
	@property
	def CLIP(self):
		return(self._CLIP)
	@CLIP.setter
	def CLIP(self, v):
		self.__UpdateFlags("_CLIP", v)
	#### --------------------------------
	@property
	def PSURF(self):
		return(self._PSURF)
	@PSURF.setter
	def PSURF(self, v):
		self.__UpdateFlags("_PSURF", v)

	#### ================================
	@property
	def LAYR(self):
		return(self._LAYR)
	@LAYR.setter
	def LAYR(self, v):
		self.__UpdateFlags("_LAYR", v)
	#### --------------------------------
	@property
	def TAGS(self):
		return(self._TAGS)
	@TAGS.setter
	def TAGS(self, v):
		self.__UpdateFlags("_TAGS", v)
	#### --------------------------------
	@property
	def PTAG(self):
		return(self._PTAG)
	@PTAG.setter
	def PTAG(self, v):
		self.__UpdateFlags("_PTAG", v)
